---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--  
--
--   Package : Pex_Output_Commands
--
--   Version : 
--
--   Subject : PEX 
--
-- 
with X_Types;
with Pex_Types;

package Pex_Output_Commands is

    use X_Types;
    use Pex_Types;

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Clear
    --
    --  <Description>
    --
    --  This procedure resets the provided output command buffer so that
    --  it becomes empty. Its size is not altered.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer to be cleared.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Clear ( Buffer   : in out Output_Command_Buffer);
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Free
    --
    --  <Description>
    --
    --  This procedure frees all the memory used to store the output commands
    --  in the provided output commands buffer. It becomes empty and its size
    --  is set to 0.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer to be freed.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Free ( Buffer   : in out Output_Command_Buffer);
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Size
    --
    --  <Description>
    --
    --  This function returns the size of the provided output command buffer.
    --
    --  <Parameters>
    --
    --  Buffer   : Interrogated Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    function Get_Size (Buffer : in Output_Command_Buffer) return Natural ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Count
    --
    --  <Description>
    --
    --  This function returns the count of output commands in
    --  the provided output command buffer.
    --
    --  <Parameters>
    --
    --  Buffer   : Interrogated Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    function Get_Count (Buffer : in Output_Command_Buffer) return Natural ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Size
    --
    --  <Description>
    --
    --  This procedure sets the size of the provided output command buffer.
    --  If the given size is lower than the actual one, no change is made.
    --  Data already stored in the command buffer are not altered.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer to be resized.
    --  New_Size : Minimum size to which it must be set.
    --             The effective size will be the power of two just
    --             greater than the given size. 
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Size ( Buffer   : in out Output_Command_Buffer ;
                         New_Size : in Natural );
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Marker_Type
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's marker_type attribute. If the specified
    --  marker type is not supported by the PEX server, marker
    --  type 3 ( MarkerAsterisk )is used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Marker_Type
        ( Buffer         : in out Output_Command_Buffer;
          Type_Of_Marker : in Marker_Type);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Marker_Size_Scale_Factor
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's marker_scale attribute. The specified
    --  scale is multiplied by the nominal marker size (see
    --  PEXGetImpDepConstants )and the result is mapped to the
    --  nearest marker size supported by the server.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Marker_Size_Scale_Factor
        ( Buffer  : in out Output_Command_Buffer;
          Size    : in Marker_Size);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Marker_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's marker_color attribute, setting the color
    --  type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Marker_Color_Index
        ( Buffer        : in out Output_Command_Buffer;
          Index         : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Marker_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's marker_color attribute, setting the color
    --  type and value as specified. If the color type is Indexed
    --  and the specified color index is not defined, color
    --  index one is used. If color index one is not defined, the
    --  default values listed in Appendix D are used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Marker_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Marker_Bundle_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's marker_bundle_index attribute. If an
    --  undefined marker bundle index is specified by this output
    --  command, then default bundle index one is used. If
    --  table index one is not defined, the default values listed
    --  in Appendix D are used. An OutputCommand error is
    --  reported if the bundle index in this output command is
    --  less than one.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Marker_Bundle_Index
        ( Buffer       : in out Output_Command_Buffer;
          Bundle_Index : in Marker_Bundle_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Font
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_font_index attribute. The
    --  text_font_index selects which entry in the text font
    --  table (i.e., which font group)will be used to render text
    --  primitives. If an undefined text font index is specified
    --  by this output command, the default index one is used.
    --  If table index one is not defined, the default values
    --  listed in Appendix D are used. An OutputCommand error
    --  is reported if the text font index in this output command
    --  is less than one. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Font
        ( Buffer     : in out Output_Command_Buffer;
          Font_Index : in Text_Font_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Precision
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_precision attribute. When a text or
    --  annotation text output primitive is interpreted, all of
    --  the ISTRING fragments in the text string are rendered
    --  in the same text precision. That is, if the font group
    --  selected by the current text font index consists of
    --  both X and PEX fonts, and if some of the ISTRING
    --  fragments in the string are rendered in X fonts, the
    --  text precision of the entire string must be dropped to at
    --  least Char precision. If a char_set value is not
    --  available in the current font group, then the entire
    --  string is rendered using the default font group. If a
    --  char_set value is not available in the default font
    --  group, then that portion of the string is rendered in
    --  an implemenation dependent manner.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Precision
        ( Buffer    : in out Output_Command_Buffer;
          Precision : in Text_Precision);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Char_Expansion_Factor
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's char_expansion attribute. Only the
    --  magnitude of the specified expansion is considered. The
    --  specified expansion is compared to the minimum and
    --  maximum character expansion factors. These values depend
    --  on the font files that are in the font groups in the
    --  selected font table entry, which in turn depend on which
    --  X or PEX font files the client opened. For example, if
    --  the client opens all PEX font files (that is, all
    --  scalable and rotatable stroke fonts)then a continuous
    --  number of expansions are supported. If the expansion is
    --  smaller than the minimum character expansion factor,
    --  the minimum value is used. If the expansion is larger
    --  than the maximum character expansion factor, the
    --  maximum value is used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Char_Expansion_Factor
        ( Buffer    : in out Output_Command_Buffer;
          Expansion : in Char_Expansion);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Char_Spacing
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's char_spacing attribute. No errors or
    --  defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Char_Spacing
        ( Buffer  : in out Output_Command_Buffer;
          Spacing : in Char_Spacing);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_color attribute, setting the color
    --  type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Color_Index
        ( Buffer        : in out Output_Command_Buffer;
          Index         : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_color attribute, setting the color
    --  type and value as specified. If the color type is Indexed
    --  and the specified color index is not defined, color index
    --  one is used. If color index one is not defined, the
    --  default values listed in Appendix D are used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Character_Height
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's char_height attribute. If the specified
    --  height or the computed width is not supported, the height
    --  or width is mapped to the nearest character height or
    --  width supported. These values depend on the font files
    --  that are in the font groups in the selected font table
    --  entry, which in turn depend on which X or PEX font files
    --  the client opened. For example, if the client opens all
    --  PEX font files (that is, all scalable and rotatable
    --  stroke fonts)then a continuous number of character
    --  sizes are supported.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Character_Height
        ( Buffer  : in out Output_Command_Buffer;
          Height  : in MC.Magnitude);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Character_Up_Vector
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's char_up_vector attribute. If the character
    --  up vector is degenerate (it has a length of zero)the
    --  value <0, 1> is used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Character_Up_Vector
        ( Buffer  : in out Output_Command_Buffer;
          Char_Up_Vector : in MC.Vector_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Path
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_path attribute. An Output_Command
    --  error is reported if the path is not Path_Right, Path_Left,
    --  Path_Up , or Path_Down .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Path
        ( Buffer  : in out Output_Command_Buffer;
          Path    : in Text_Path);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Alignment
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_alignment attribute. An OutputCommand
    --  error is reported if the horizontal alignment is not
    --  HalignNormal , HalignLeft , HalignCenter , or HalignRight
    --  , or if the vertical alignment is not ValignNormal ,
    --  ValignTop , ValignCap , ValignHalf , ValignBase , or
    --  ValignBottom . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Alignment
        ( Buffer    : in out Output_Command_Buffer;
          Alignment : in Text_Alignment);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Annotation_Text_Height
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's atext_height attribute. If the specified
    --  height or the computed width is not supported, the height
    --  or width is mapped to the nearest annotation character
    --  height or width supported. These values depend on the
    --  font files that are in the font groups in the selected
    --  font table entry, which in turn depend on which X or PEX
    --  font files the client opened. For example, if the client
    --  opens all PEX font files (that is, all scalable and
    --  rotatable stroke fonts)then a continuous number of
    --  character sizes are supported. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Annotation_Text_Height
        ( Buffer             : in out Output_Command_Buffer;
          Annotation_Height  : in NPC.Magnitude);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Annotation_Text_Up_Vector
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's atext_up_vector attribute. If the
    --  annotation text up vector is degenerate (it has a length
    --  of zero)the value <0, 1> is used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Annotation_Text_Up_Vector
        ( Buffer                    : in out Output_Command_Buffer;
          Annotation_Char_Up_Vector : in NPC.Vector_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Annotation_Text_Path
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's atext_path attribute. An OutputCommand
    --  error is reported if the path is not PathRight, PathLeft,
    --  PathUp , or PathDown .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Annotation_Text_Path
        ( Buffer          : in out Output_Command_Buffer;
          Annotation_Path : in Text_Path);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Annotation_Text_Alignment
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's atext_alignment attribute. An
    --  OutputCommand error is reported if the horizontal
    --  alignment is not HalignNormal , HalignLeft , HalignCenter
    --  , or HalignRight , or if the vertical alignment is not
    --  ValignNormal , ValignTop , ValignCap, ValignHalf ,
    --  ValignBase , or ValignBottom . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Annotation_Text_Alignment
        ( Buffer               : in out Output_Command_Buffer;
          Annotation_Alignment : in Text_Alignment);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Annotation_Style
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's atext_style attribute. If the specified
    --  style is not supported by the PEX server, annotation
    --  style 1 ( ATextNotConnected )is used. Any integer value
    --  may be specified as the style in this output command. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Annotation_Style
        ( Buffer              : in out Output_Command_Buffer;
          Style_Of_Annotation : in Annotation_Style);

    --
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Text_Bundle_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's text_bundle_index attribute. If an
    --  undefined text bundle index is specified by this output
    --  command, then default bundle index one is used. If
    --  table index one is not defined, the default values listed
    --  in Appendix D are used. An OutputCommand error is
    --  reported if the bundle index in this output command is
    --  less than one. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Text_Bundle_Index
        ( Buffer  : in out Output_Command_Buffer;
          Index   : in Text_Bundle_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Type
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's line_type attribute. If the specified
    --  line type is not supported by the PEX server, line type 1
    --  ( LineTypeSolid )is used. Any integer value may be
    --  specified as the line type in this output command. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Line_Type
        ( Buffer       : in out Output_Command_Buffer;
          Type_Of_Line : in Line_Type);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Width
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's line_width attribute. The specified line
    --  width is multiplied by the nominal line width (see
    --  PEXGetImpDepConstants )The result is mapped to the
    --  nearest line width supported by the server. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Line_Width
        ( Buffer  : in out Output_Command_Buffer;
          Width   : in Line_Width);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's line_color attribute, setting the color
    --  type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Line_Color_Index
        ( Buffer        : in out Output_Command_Buffer;
          Index         : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's line_color attribute, setting the color
    --  type and value as specified. If the color type is Indexed
    --  and the specified color index is not defined, color
    --  index one is used. If color index one is not defined, the
    --  default values listed in Appendix D are used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Line_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Curve_Approximation_Method
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's curve_approx attribute. If the specified
    --  method is not supported by the PEX server, method 1
    --  (implementation dependent)is used. Any integer value
    --  may be specified as the curve approximation method in
    --  this output command. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Curve_Approximation_Method
        ( Buffer        : in out Output_Command_Buffer;
          Approximation : in Curve_Approximation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Polyline_Interpolation_Method
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's polyline_interp attribute. If the
    --  specified interpolation method is not supported by the
    --  PEX server, method 1 ( PolylineInterpNone )is used. Any
    --  integer value may be specified as the interpolation
    --  method in this output command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Polyline_Interpolation_Method
        ( Buffer  : in out Output_Command_Buffer;
          Method  : in Polyline_Interpolation_Method);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Bundle_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's line_bundle_index attribute. If an
    --  undefined line bundle index is specified by this output
    --  command, then default bundle index one is used. If table
    --  index one is not defined, the default values listed in
    --  Appendix D are used. An OutputCommand error is reported
    --  if the bundle index in this output command is less than
    --  one. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Line_Bundle_Index
        ( Buffer   : in out Output_Command_Buffer;
          Index    : in Line_Bundle_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Interior
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's interior_style attribute. If the specified
    --  style is not supported, style 1 ( InteriorStyleHollow )is
    --  used. An OutputCommand error is reported if the style is
    --  not InteriorStyleHollow , InteriorStyleSolid ,
    --  InteriorStylePattern , InteriorStyleHatch , or
    --  InteriorStyleEmpty . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Interior
        ( Buffer        : in out Output_Command_Buffer;
          Interior      : in Surface_Interior);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_color attribute, setting the color
    --  type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Color_Index
        ( Buffer        : in out Output_Command_Buffer;
          Index         : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_color attribute, setting the color
    --  type and value as specified. If the color type is Indexed
    --  and the specified color index is not defined, color index
    --  one is used. If color index one is not defined, the
    --  default values listed in Appendix D are used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Reflection_Attributes
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's reflection_attr attribute. No errors or
    --  defaults are defined. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Reflection_Attributes
        ( Buffer     : in out Output_Command_Buffer;
          Attributes : in Reflection_Attributes);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Reflection_Model
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's reflection_model attribute. If the
    --  specified reflection model is not supported by the PEX
    --  server, method 1 ( ReflectionNoShading )is used. Any
    --  integer value may be specified as the reflection model in
    --  this output command. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Reflection_Model
        ( Buffer  : in out Output_Command_Buffer;
          Model   : in Reflection_Model);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Interpolation_Method
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_interp attribute. If the specified
    --  interpolation method is not supported by the PEX server,
    --  method 1 ( SurfaceInterpNone )is used. Any integer value
    --  may be specified as the interpolation method in this
    --  output command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Interpolation_Method
        ( Buffer  : in out Output_Command_Buffer;
          Method  : in Surface_Interpolation_Method);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Interior
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_interior_style attribute. If the
    --  specified style is not supported, style 1 (
    --  InteriorStyleHollow )is used. An OutputCommand error is
    --  reported if the style is not InteriorStyleHollow ,
    --  InteriorStyleSolid , InteriorStylePattern ,
    --  InteriorStyleHatch , or InteriorStyleEmpty . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Interior
        ( Buffer       : in out Output_Command_Buffer;
          Interior     : in Surface_Interior);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_surface_color attribute, setting the
    --  color type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Color_Index
        ( Buffer        : in out Output_Command_Buffer;
          Index         : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_surface_color attribute, setting the
    --  color type and value as specified. If the color type is
    --  Indexed and the specified color index is not defined,
    --  color index one is used. If color index one is not
    --  defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Reflection_Attributes
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_reflection_attr attribute. No errors or
    --  default are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Reflection_Attributes
        ( Buffer     : in out Output_Command_Buffer;
          Attributes : in Reflection_Attributes);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Reflection_Model
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_reflection_model attribute. If the
    --  specified reflection model is not supported by the PEX
    --  server, method 1 ( ReflectionNoShading )is used. Any
    --  integer value may be specified as the reflection model in
    --  this output command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Reflection_Model
        ( Buffer  : in out Output_Command_Buffer;
          Model   : in Reflection_Model);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Backface_Surface_Interpolation_Method
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's bf_surface_interp attribute. If the
    --  specified interpolation method is not supported by the
    --  PEX server, method 1 ( SurfaceInterpNone )is used. Any
    --  integer value may be specified as the interpolation
    --  method in this output command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Backface_Surface_Interpolation_Method
        ( Buffer  : in out Output_Command_Buffer;
          Method  : in Surface_Interpolation_Method);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Approximation_Method
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_approx attribute. If the specified
    --  method is not supported by the PEX server, method 1
    --  (implementation dependent)is used. Any integer value may
    --  be specified as the surface approximation method in this
    --  output command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Approximation_Method
        ( Buffer        : in out Output_Command_Buffer;
          Approximation : in Surface_Approximation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Facet_Culling_Mode
    --
    --  <Description>
    --
    --  PHIGS requires that the backface interior style index
    --  must be greater than zero for InteriorStylePattern , but
    --  this is difficult for PEX to enforce, so a default action
    --  is defined instead. When processed by a renderer, this
    --  command will modify the renderer's culling_mode
    --  attribute. An OutputCommand error is reported if the mode
    --  is not None , BackFaces , or FrontFaces .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Facet_Culling_Mode
        ( Buffer  : in out Output_Command_Buffer;
          Mode    : in Culling_Mode);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Facet_Distinguish_Flag
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's distinguish attribute. An OutputCommand
    --  error is reported if the mode is not Off or On .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Facet_Distinguish_Flag
        ( Buffer      : in out Output_Command_Buffer;
          Distinguish : in Boolean);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Pattern_Size
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's pattern_size attribute. Only the
    --  magnitudes of the specified pattern size components are
    --  considered. If the current interior_style is
    --  InteriorStylePattern , the specified pattern size
    --  components are used. If either component is zero, the
    --  output command is ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Pattern_Size
        ( Buffer  : in out Output_Command_Buffer;
          Size    : in Mc.Size_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Pattern_Reference_Point
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's pattern_ref_pt , pattern_ref_vec1 , and
    --  pattern_ref_vec2 attributes. The z coordinate of the
    --  reference point will be set to zero, pattern_ref_vec1
    --  will be set to <1,0,0>, and pattern_ref_vec2 will be set
    --  to <0,1,0>. No errors or defaults are defined. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Pattern_Reference_Point
        ( Buffer          : in out Output_Command_Buffer;
          Reference_Point : in MC.Point_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Pattern_Reference_Point_And_Vectors
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's pattern_ref_pt , pattern_ref_vec1 , and
    --  pattern_ref_vec2 attributes. If the pattern vectors
    --  define a degenerate case (that is, if one of the vectors
    --  is zero length or if the vectors are parallel)the output
    --  command is ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Pattern_Reference_Point_And_Vectors
        ( Buffer            : in out Output_Command_Buffer;
          Reference_Point   : in MC.Point_3;
          Reference_Vectors : in MC.Vector_Pair_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Interior_Bundle_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's interior_bundle_index attribute. If an
    --  undefined interior bundle index is specified by this
    --  output command, then default bundle index one is used. If
    --  table index one is not defined, the default values listed
    --  in Appendix D are used. An OutputCommand error is
    --  reported if the bundle index in this output command is
    --  less than one. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Interior_Bundle_Index
        ( Buffer   : in out Output_Command_Buffer;
          Index    : in Interior_Bundle_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Edge_Flag
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_edges attribute. An OutputCommand
    --  error is reported if the flag is not On or Off .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Edge_Flag
        ( Buffer  : in out Output_Command_Buffer;
          Flag    : in Edge_Flag);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Edge_Type
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_edge_type attribute. If the
    --  specified edge type is not supported by the PEX server,
    --  edge type 1 ( SurfaceEdgeSolid )is used. Any integer
    --  value may be specified as the edge type in this output
    --  command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Edge_Type
        ( Buffer       : in out Output_Command_Buffer;
          Surface_Edge : in Surface_Edge_Type);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Edge_Width
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_edge_width attribute. The
    --  specified edge width is multiplied by the nominal edge
    --  width (see PEXGetImpDepConstants )The result is mapped to
    --  the nearest edge width supported by the PEX server.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Edge_Width
        ( Buffer  : in out Output_Command_Buffer;
          Width   : in Edge_Width);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Edge_Color_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_edge_color attribute, setting the
    --  color type to Indexed and the color value to the index
    --  specified by color . If the specified color index is not
    --  defined, color index one is used. If color index one is
    --  not defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Edge_Color_Index
        ( Buffer      : in out Output_Command_Buffer;
          Index       : in Color_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Surface_Edge_Color
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's surface_edge_color attribute, setting the
    --  color type and value as specified. If the color type is
    --  Indexed and the specified color index is not defined,
    --  color index one is used. If color index one is not
    --  defined, the default values listed in Appendix D are
    --  used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Surface_Edge_Color
        ( Buffer  : in out Output_Command_Buffer;
          Color   : in Color_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Edge_Bundle_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's edge_bundle_index attribute. If an
    --  undefined edge bundle index is specified by this output
    --  command, then default bundle index one is used. If table
    --  index one is not defined, the default values listed in
    --  Appendix D are used. An OutputCommand error is reported
    --  if the bundle index in this output command is less than
    --  one. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Edge_Bundle_Index
        ( Buffer    : in out Output_Command_Buffer;
          Index     : in Edge_Bundle_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Individual_Asf
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the specified ASF (aspect source flag)attribute in the
    --  renderer.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Individual_Asf
        ( Buffer      : in out Output_Command_Buffer;
          Aspects     : in Aspect_Mask;
          Source_Flag : in ASF);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Local_Transform
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's local_transform attribute. If comp_type is
    --  PreConcatenate , the specified matrix is pre-concatenated
    --  to the local model transformation matrix. If comp_type is
    --  PostConcatentate, the specified matrix is
    --  post-concatenated to the local modeling transform. If
    --  comp_type is Replace , the specified matrix replaces the
    --  local modeling transform. An OuputCommand error is
    --  reported if the composition type is not PreConcatenate ,
    --  PostConcatenate or Replace . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Local_Transform
        ( Buffer      : in out Output_Command_Buffer;
          Matrix      : in Transformation_Matrix;
          How_Applied : in Composition_Type);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Local_Transform
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's local_transform attribute. This command is
    --  identical to local transform 3D except that the specified
    --  matrix is a 3x3 matrix instead of a 4x4 matrix. Before
    --  the concatenation occurs, the 3x3 matrix represented by 
    --  R J Q g d a h e b j f c H J P will be expanded to a 4x4
    --  matrix as follows: R J Q g d a h e b j f c H J P 
    --  -> R J J Q g 0 d a h 0 e b 0 1 0 0 j 0 f c H J J P An
    --  OutputCommand error is reported if the composition type
    --  is not PreConcatenate , PostConcatenate or Replace .
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Local_Transform
        ( Buffer      : in out Output_Command_Buffer;
          Matrix      : in Transformation_Matrix_2;
          How_Applied : in Composition_Type);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Global_Transform
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will replace
    --  the renderer's global_transform attribute. No errors or
    --  defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Global_Transform
        ( Buffer  : in out Output_Command_Buffer;
          Matrix  : in Transformation_Matrix);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Global_Transform
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will replace
    --  the renderer's global_transform attribute. This command
    --  is identical to "Global transform 3D" except that the
    --  specified matrix is a 3x3 matrix instead of a 4x4 matrix.
    --  Before the replacement occurs, the 3x3 matrix represented
    --  by R J Q g d a h e b j f c H J P will be expanded to a 
    --  4x4 matrix as follows: R J Q g d a h e b j f c H J P 
    --  -> R J J Q g 0 d a h 0 e b 0 1 0 0 j 0 f c H J J P 
    --  No errors or defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Global_Transform
        ( Buffer  : in out Output_Command_Buffer;
          Matrix  : in Transformation_Matrix_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Modelling_Clipping_Indicator
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's model_clip attribute. An OutputCommand
    --  error is reported if the model clip indicator is not Clip
    --  or NoClip . 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Modelling_Clipping_Indicator
        ( Buffer             : in out Output_Command_Buffer;
          Modelling_Clipping : in Clipping_Indicator);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Modelling_Clipping_Volume
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's model_clip_volume attribute. The operator
    --  indicated by operator will be used to combine the
    --  specified list of halfspaces with the current modeling
    --  clipping volume to form a new modeling clipping volume.
    --  The halfspaces are specified in modeling coordinates,
    --  with the z component of each point and vector assumed to
    --  be zero. If the specified operator is not supported by
    --  the PEX server or if any halfspace is degenerate, the
    --  output command is ignored. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Modelling_Clipping_Volume
        ( Buffer      : in out Output_Command_Buffer;
          Operator    : in Model_Clip_Operator;
          Half_Spaces : in MC.Half_Space_List_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Modelling_Clipping_Volume
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's model_clip_volume attribute. The operator
    --  indicated by operator will be used to combine the
    --  specified list of halfspaces with the current modeling
    --  clipping volume to form a new modeling clipping volume.
    --  Each halfspace is defined by a point and a normal in
    --  modeling coordinates. The vector is considered to be a
    --  normal to the plane of the bound of the halfspace and
    --  points in the direction of the halfspace, and the point
    --  is considered to be on the plane. If the specified
    --  operator is not supported by the PEX server or if any
    --  halfspace is degenerate, the output command is ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Modelling_Clipping_Volume
        ( Buffer      : in out Output_Command_Buffer;
          Operator    : in Model_Clip_Operator;
          Half_Spaces : in MC.Half_Space_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Restore_Modelling_Clipping_Volume
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's model_clip_volume attribute. The modeling
    --  clipping volume will be restored to its state as of the
    --  last structure invocation, or to the default state if no
    --  structure was invoked. No errors or defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Restore_Modelling_Clipping_Volume
        ( Buffer  : in out Output_Command_Buffer);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_View_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's view_index attribute. If the specified
    --  view index is not defined, index zero is used. If view
    --  index zero is not defined, the default values listed in
    --  Appendix D are used.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_View_Index
        ( Buffer     : in out Output_Command_Buffer;
          Index      : in View_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Lightsource_State
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's light_state attribute. The current
    --  light_state is modified by activating ("turning on")each
    --  light source whose index is specified in the enable list,
    --  and by deactivating ("turning off")each light source
    --  whose index is specified in the disable list. If any
    --  light in the enable or disable list references an
    --  undefined Light table entry, the light is ignored. An
    --  OutputCommand error is reported if a light in either the
    --  enable or disable list is less than one, or if the same
    --  light is specified in both the lists. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Lightsource_State
        ( Buffer  : in out Output_Command_Buffer;
          Enable  : in Light_Lut_Array;
          Disable : in Light_Lut_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Depth_Cue_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's depth_cue_index attribute. If the
    --  specified depth cue index is not defined, index zero is
    --  used. If depth cue index zero is not defined, the default
    --  values listed in Appendix D are used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Depth_Cue_Index
        ( Buffer  : in out Output_Command_Buffer;
          Index   : in Depth_Cue_Index);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Pick_Id
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's pick_id attribute. No errors or defaults
    --  are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Pick_Id
        ( Buffer          : in out Output_Command_Buffer;
          Pick_Identifier : in Pick_Id);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Hlhsr_Identifier
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's HLHSR_identifier attribute. This output
    --  command is effectively a no-op when used in conjunction
    --  with the currently-registered HLHSR modes. For
    --  non-registered HLHSR modes the effect of this command is
    --  implementation-dependent. It is provided for PHIGS
    --  compatibility purposes. If the specified HLHSR identifier
    --  is not supported by the PEX server, the HLHSR identifier
    --  used is implementation dependent.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Hlhsr_Identifier
        ( Buffer           : in out Output_Command_Buffer;
          Hlhsr_Identifier : in Hlhsr_Id);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Color_Approximation_Index
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's color_approx_index attribute. If the
    --  specified color approximation index is not defined, index
    --  zero is used. If index zero is not defined, the default
    --  values listed in Appendix D are used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Color_Approximation_Index
        ( Buffer  : in out Output_Command_Buffer;
          Index   : in Color_Approximation_Index) ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Rendering_Color_Model
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's rdr_color_model attribute. If the
    --  specified color model is not supported by the PEX server,
    --  model 0 (implementation dependent) is used. Any integer
    --  value may be specified as the color model in this output
    --  command. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Rendering_Color_Model
        ( Buffer     : in out Output_Command_Buffer;
          Model      : in Rendering_Color_Model ) ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Parametric_Surface_Characteristics
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will modify
    --  the renderer's psurf_char attribute. If the specified
    --  type is not supported by the PEX server, type 1 (None)
    --  is used and the data record contents are implementation
    --  dependent. Any integer value may be specified as the
    --  characteristics type in this output command. If an
    --  inconsistent value is specified in the data record, an
    --  OutputCommand error is reported. (For example, if the
    --  type is IsoparametricCurves and the curve placement is
    --  not Uniform or NonUniform , or if the type is
    --  MC_LevelCurves or WC_LevelCurves and the direction vector
    --  has zero length.)
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Parametric_Surface_Characteristics
        ( Buffer          : in out Output_Command_Buffer;
          Characteristics : in Parametric_Characteristics_Data ) ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Add_Names_To_Set
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will add names
    --  to the renderer's name set. If any name in the set of
    --  normal or inverted name set names is outside the
    --  supported range (see PEXGetImpDepConstants )that name is
    --  ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Add_Names_To_Set
        ( Buffer         : in out Output_Command_Buffer;
          Names_To_Add   : in Name_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Remove_Names_From_Set
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will remove
    --  names from the renderer's name set. If any name in the
    --  set of normal or inverted name set names is outside the
    --  supported range (see PEXGetImpDepConstants )that name is
    --  ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Remove_Names_From_Set
        ( Buffer          : in out Output_Command_Buffer;
          Names_To_Remove : in Name_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Execute_Structure
    --
    --  <Description>
    --
    --  When processed by a renderer, this output command
    --  transfers flow-of-control to the specified structure.
    --  Processing of output commands will then commence with the
    --  first structure element in the structure specified by
    --  s_id . When all of the elements in the called structure
    --  have been processed, control will be returned. When this
    --  command is executed, all of the rendering pipeline
    --  attribute values in the renderer are saved. Then, the
    --  current global modeling transform is set to the current
    --  composite modeling transform and the current local
    --  modeling matrix is set to the identity matrix. When
    --  control is returned, the saved rendering pipeline
    --  attribute values are restored. If s_id references a
    --  non-existent structure, an OutputCommand error is
    --  reported.* Note that it is not possible to have
    --  traversal time errors due to non-existent structures,
    --  since Destroy_Structures removes all references to
    --  structures it deletes.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Execute_Structure
        ( Buffer       : in out Output_Command_Buffer;
          Structure_Id : in Structure);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Executed_Structure
    --
    --  <Description>
    --
    --  This function returns the structure identifier found in the first
    --  output command stored in the given output command buffer.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --  Io_Exceptions.Data_Error is raised if either the buffer is empty
    --  or if the first command stored in it is not an 'Execute_Structure'
    --  command.
    --
    --=====================================================================
    
    function Executed_Structure
        ( Buffer  : in Output_Command_Buffer) return Structure ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Label
    --
    --  <Description>
    --
    --  When processed by a renderer, this output command is a
    --  no-op. Its main usefulness is when used as a structure
    --  element to maintain the specified label as an aid to
    --  navigation during structure editing. No errors or
    --  defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Label
        ( Buffer           : in out Output_Command_Buffer;
          Label_Identifier : in Label_Id);
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Label
    --
    --  <Description>
    --
    --  This function returns the label identifier found in the first
    --  output command stored in the given output command buffer.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --  Io_Exceptions.Data_Error is raised if either the buffer is empty
    --  or if the first command stored in it is not a 'Label' command.
    --
    --=====================================================================
    
    function Label ( Buffer : in Output_Command_Buffer ) return Label_Id ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Application_Data
    --
    --  <Description>
    --
    --  When processed by a renderer, this output command is a
    --  no-op. Its main usefulness is when used as a structure
    --  element in order to maintain the specified client
    --  application data. No errors or defaults are defined. Note
    --  that byte-swapping and floating point conversion are not
    --  done on this type of output command.
    --
    --
    --  Data.Size will indicate the number of X_Long_Integer elements to
    --  store in the buffer, which may be less than Data.Data'length.
    --
    --  It is the basic protocol command, which must be used when
    --  the data to store have a complicated representation (for example,
    --  they are made of variable records, arrays or linked lists).
    --
    --  For simple constrained records, the programmer will prefer
    --  the generic package Application_Data_Facilities.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Application_Data
        ( Buffer  : in out Output_Command_Buffer;
          Data    : in Application_Data_Record);
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Application_Data
    --
    --  <Description>
    --
    --  This procedure gets the application data record from the given
    --  buffer. The buffer must contains an "Application_Data" output
    --  command at the beginning of the buffer.
    --  Data.Size indicates the number of X_Long_Integer elements got
    --  from the buffer which may be less than Data.Data'length.
    --  Data.Data will be freed and reallocated only if it is too small.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --  Io_Exceptions.Data_Error is raised if either the buffer is empty
    --  or if the first command stored in it is not an 'Application Data'
    --  command.
    --
    --=====================================================================

    procedure Get_Application_Data
        ( Buffer  : in out Output_Command_Buffer;
          Data    : in out Application_Data_Record);
    
    --=====================================================================
    --  <Package Name>
    --
    --  Application Data
    --
    --  <Description>
    --
    --  This generic package provides the application with the mean to
    --  store or retrieve an object as a structure element.
    --  The object type must be contiguous in memory and of a constant size.
    --
    --  For the retrieval, the application data element must be the first
    --  element of the provided output command buffer.
    --
    --  <Parameters>
    --
    --  Object   : Object type to store or retrieve in the structures.
    --
    --  <Errors Raised>
    --
    --  Io_Exceptions.Data_Error is raised in Retrieve if either the buffer
    --  is empty or if the first command stored in it is not
    --  an 'Application Data' command.
    --
    --=====================================================================
    
    generic
        type Object is private ;
    package Application_Data_Facilities is
        
        procedure Store ( Buffer  : in out Output_Command_Buffer;
                          Data    : in Object);
        
        function Retrieve (Buffer : in Output_Command_Buffer) return Object;
        
    end Application_Data_Facilities ;
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Generalized_Gse
    --
    --  <Description>
    --
    --  When processed by a renderer, the effect of this command
    --  is implementation-dependent. Because of floating point
    --  and color format discrepancies across a network
    --  interface, it is not anticipated that the GSE will
    --  provide a useful extension mechanism, but it is provided
    --  for PHIGS compatibility purposes. If the specified GSE
    --  identifier is not supported by the PEX server, the output
    --  command is ignored. Note that byte-swapping and floating
    --  point conversions may not be done on this type of output
    --  command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Generalized_Gse
        ( Buffer      : in out Output_Command_Buffer;
          Identifier  : in Gse_Id;
          Gse_Data    : in Gse_Data_Record);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Polymarker
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause
    --  marker primitives to be rendered. A marker is a geometric
    --  primitive with only one geometric attribute - its
    --  position. The list points contains a list of 3D
    --  coordinates, each of which specifies the position of a
    --  marker in modeling coordinates. During the rendering
    --  process, the marker's position is transformed to a
    --  position in device coordinates. A marker has no geometric
    --  size, so geometric transformations will not affect the
    --  displayed size of the marker. The marker's color is
    --  transformed only by the depth-cueing computation (the
    --  light-source shading stage of the rendering pipeline
    --  affects only surfaces)and mapped to a device color. The
    --  clipping of markers whose position is inside the clipping
    --  volume but whose rendering is outside the clipping volume
    --  is implementation- dependent. Depending on the setting of
    --  the marker attribute ASF values, the marker_color ,
    --  marker_type , and marker_scale attributes are either
    --  obtained directly from the current marker attribute
    --  values or from the marker_bundle_index 'th entry in the
    --  renderer's marker_bundle .No errors or defaults are
    --  defined. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Polymarker
        ( Buffer  : in out Output_Command_Buffer;
          Points  : in MC.Point_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Polymarker
    --
    --  <Description>
    --
    --  When processed, this command will cause marker primitives
    --  to be drawn. This primitive functions exactly as the 3D
    --  marker primitive except that modeling coordinate
    --  positions are specified using only x- and y- coordinates,
    --  and the z -coordinate is always assumed to be zero. No
    --  errors or defaults are defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Polymarker
        ( Buffer  : in out Output_Command_Buffer;
          Points  : in MC.Point_List_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Text
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  text string to be rendered. The parameter string contains
    --  the text string to be rendered. A string is a list of
    --  data records, each of which contains char_set ,
    --  char_set_width and encoding_state fields, as well as a
    --  list of the characters that actually make up that portion
    --  of the string. The char_set field is an index into the
    --  current font group. Font groups have individual fonts
    --  numbered starting at one, so a value of three would
    --  select the third font in the font group currently being
    --  used. If a char_set value is not available in the current
    --  font group, then the entire string will be rendered using
    --  the default font group. If a char_set value is not
    --  available in the default font group, then that portion of
    --  the string will be rendered in an
    --  implementation-dependent manner. The char_set_width
    --  indicates whether characters in the string are 8-, 16-,
    --  or 32-bit values. Characters in the string are
    --  byte-swapped by the PEX server, if necessary. The
    --  encoding_state field is provided for use by clients and
    --  is not interpreted by the server. The text string is
    --  located on a plane defined by its position and direction
    --  vectors. The origin of the string is a point in modeling
    --  coordinates indicated by origin , and the string's
    --  direction is indicated by the direction vectors vector1
    --  and vector2 . Vector1 defines the positive x -axis of the
    --  text local coordinate system. Vector2 defines the
    --  positive y -axis of the text local coordinate system. *
    --  Depending on the setting of the text attribute ASF
    --  values, the text_color , text_precision , char_expansion
    --  , char_spacing , and text_font_index attributes are
    --  either obtained directly from the current text attribute
    --  values or from the text_bundle_index 'th entry in the
    --  renderer's text_bundle . The char_height , char_up_vector
    --  , text_path , and text_alignment attributes are also used
    --  when drawing the text primitive. An attempt is made to
    --  render the text string as accurately as possible with the
    --  font group named by the current text_font_index . The
    --  directions specified by char_up_vector and text_path will
    --  be relative to the text local coordinate system. During
    --  the rendering process, a string's position is transformed
    --  to a position in device coordinates. The string's color
    --  is transformed only by the depth-cueing computation (the
    --  light-source shading stage of the rendering pipeline
    --  affects only surfaces)and mapped to a device color. The
    --  text string is clipped depending on the current text
    --  precision value. If the text precision is String ,
    --  clipping is done in an implementation- dependent fashion.
    --  If the text precision is Char , clipping is done on at
    --  least a character-by-character basis. If the text
    --  precision is Stroke , clipping is performed at the
    --  clipping boundaries for each character. If either of
    --  the text direction vectors is zero length or if the
    --  vectors are parallel, the vectors <1,0,0> and <0,1,0>
    --  are used. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Text
        ( Buffer            : in out Output_Command_Buffer;
          Position          : in MC.Point_3;
          Direction_Vectors : in MC.Vector_Pair_3;
          Char_String       : in Mono_Encoding_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Text
    --
    --  <Description>
    --
    --  When processed, this command will cause a text primitive
    --  to be drawn. This primitive functions exactly as the 3D
    --  text primitive except that it has no direction vectors,
    --  the modeling coordinate position is specified using only
    --  x- and y- coordinates, and the z -coordinate is always
    --  assumed to be zero. No errors or defaults are defined. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Text
        ( Buffer            : in out Output_Command_Buffer;
          Position          : in MC.Point_3;
          Direction_Vectors : in MC.Vector_Pair_3;
          Set               : in Character_Set ;
          Char_String       : in String);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Text
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Text
        ( Buffer            : in out Output_Command_Buffer;
          Position          : in MC.Point_2;
          Char_String       : in Mono_Encoding_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Text
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Text
        ( Buffer            : in out Output_Command_Buffer;
          Position          : in MC.Point_2;
          Set               : in Character_Set ;
          Char_String       : in String);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Annotation_Text_Relative
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause an
    --  annotation text string to be rendered. The parameter
    --  string contains the text string to be rendered. A string
    --  is a list of data records, each of which contains
    --  char_set , char_set_width and encoding_state fields, as
    --  well as a list of the characters that actually make up
    --  that portion of the string. The char_set field is an
    --  index into the current font group. Font groups have
    --  individual fonts numbered starting at one, so a value of
    --  three would select the third font in the font group
    --  currently being used. If a char_set value is not
    --  available in the current font group, then the entire
    --  string will be rendered using the default font group. If
    --  a char_set value is not available in the default font
    --  group, then that portion of the string will be rendered
    --  in an implementation-dependent manner. The char_set_width
    --  indicates whether characters in the string are 8-, 16-,
    --  or 32-bit values. Characters in the string are byte- |
    --  swapped by the PEX server, if necessary. The
    --  encoding_state field is provided for use by clients and
    --  is not interpreted by the server. The origin of the
    --  string is a point in modeling coordinates indicated by
    --  origin . An offset value in normalized projection
    --  coordinates is specified by offset . The point at which
    --  the annotation text string is to be placed is called the
    --  annotation point, and is computed by adding offset to the
    --  transformed origin point. The z - component of the
    --  annotation point specifies the the x-y plane in
    --  normalized projection coordinate space on which the
    --  annotation text string will be placed. Depending on the
    --  setting of the text attribute ASF values, the text_color
    --  , text_precision , char_expansion , char_spacing , and
    --  text_font_index attributes are either obtained directly
    --  from the current text attribute values or from the
    --  text_bundle_index 'th entry in the renderer's text_bundle
    --  The atext_height , atext_path , atext_alignment ,
    --  atext_up_vector , and atext_style attributes are also
    --  used when rendering the text string. The annotation text
    --  string's color is transformed only by the depth-cueing
    --  computation (the light-source shading stage of the
    --  rendering pipeline affects only surfaces)and mapped to a
    --  device color. The entire annotation text primitive is
    --  clipped if origin is clipped. If origin is not clipped by
    --  modeling, view, or workstation clipping, the annotation
    --  text will be clipped according to text clipping rules and
    --  the connection line, if present, will be clipped
    --  according to polyline clipping rules, except that
    --  modeling clipping will be ignored. The current set of
    --  polyline attributes will be used to draw the connection
    --  line if it is to be drawn. No errors or defaults are
    --  defined.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Annotation_Text_Relative
        ( Buffer            : in out Output_Command_Buffer;
          Reference_Point   : in MC.Point_3;
          Annotation_Offset : in NPC.Vector_3;
          Char_String       : in Mono_Encoding_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Annotation_Text_Relative
    --
    --  <Description>
    --

    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Annotation_Text_Relative
        ( Buffer            : in out Output_Command_Buffer;
          Reference_Point   : in MC.Point_3;
          Annotation_Offset : in NPC.Vector_3;
          Set               : in Character_Set ;
          Char_String       : in String);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Annotation_Text_Relative
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Annotation_Text_Relative
        ( Buffer            : in out Output_Command_Buffer;
          Reference_Point   : in MC.Point_2;
          Annotation_Offset : in NPC.Vector_2;
          Char_String       : in Mono_Encoding_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Annotation_Text_Relative
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Annotation_Text_Relative
        ( Buffer            : in out Output_Command_Buffer;
          Reference_Point   : in MC.Point_2;
          Annotation_Offset : in NPC.Vector_2;
          Set               : in Character_Set ;
          Char_String       : in String);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Polyline
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  polyline to be rendered. The polyline is defined by the
    --  list of vertices that are specified in vertices , each of
    --  which is a coordinate in the modeling coordinate system.
    --  The vertices are joined together by line segments. The
    --  first vertex of a polyline is connected to the second,
    --  the second connected to the third, and so on. The last
    --  vertex is not connected to the first. Depending on the
    --  setting of the line attribute ASF values, line_color ,
    --  line_type , and line_width attributes are either obtained
    --  directly from the current line attribute values or from
    --  the line_bundle_index 'th entry in the renderer's
    --  line_bundle . Polylines have no geometric width, only
    --  length, so transformations will affect only the displayed
    --  length of a polyline. The polyline colors are transformed
    --  only by the depth-cueing computation (the light-source
    --  shading stage of the rendering pipeline affects only
    --  surfaces)and mapped to device colors. Polylines are not
    --  displayed if they are outside the currently defined
    --  clipping volume. Polylines that cross the clipping volume
    --  are clipped, and only the portion(s)inside the clipping
    --  volume is (are)displayed. At structure creation time, if
    --  the polyline has fewer than two vertices, it is stored in
    --  the structure, but when this output command is
    --  interpreted, it has no visual effect. In immediate mode,
    --  such a primitive is ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Polyline
        ( Buffer  : in out Output_Command_Buffer;
          Points  : in MC.Point_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Polyline
    --
    --  <Description>
    --
    --  When processed, this command will cause a polyline
    --  primitive to be drawn. This primitive functions exactly
    --  as the 3D polyline primitive except that modeling
    --  coordinate positions are specified using only x- and y-
    --  coordinates, and the z -coordinate is always assumed to
    --  be zero. At structure creation time, if the polyline has
    --  fewer than two vertices, it is stored in the structure,
    --  but when this output command is interpreted, it has no
    --  visual effect. In immediate mode, such a primitive is
    --  ignored. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Polyline
        ( Buffer  : in out Output_Command_Buffer;
          Points  : in MC.Point_List_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Polyline_Set_3d_With_Data
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  series of polylines to be rendered. The behavior of this
    --  primitive is identical to that of the 3D polyline
    --  primitive, except that multiple polylines can be drawn,
    --  and additional information can be specified at each
    --  polyline vertex. Color values that are passed will be of
    --  the type specified by color_type . The parameter
    --  vert_attributes indicates the attributes which are
    --  specified at each polyline vertex. The components of the
    --  vertex attributes bitmask are, in order: color COLOR If
    --  any of the attribute bits is set, the corresponding
    --  attributes must be present for each vertex, and they must
    --  be passed after the coordinate data for each vertex in
    --  the order that they appear in the list above. If color
    --  values are passed per vertex, they are considered to be
    --  part of the primitive and are used instead of the
    --  line_color attribute. In addition, the use of per-vertex
    --  colors is affected by the polyline_interp attribute,
    --  which is obtained directly from the polyline_interp value
    --  if the polyline_interp_asf attribute is set to Individual
    --  or from the line_bundle_index 'th entry in the renderer's
    --  line_bundle . The polyline_interp attribute defines how
    --  color values between the vertices are to be computed. At
    --  structure creation time, if any polyline in the set has
    --  fewer than two vertices, it is stored in the structure,
    --  but when this output command is interpreted, the polyline
    --  with insufficient data has no visual effect. In immediate
    --  mode, such a polyline is ignored.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Polyline_Set_3d_With_Data
        ( Buffer       : in out Output_Command_Buffer;
          Options      : in Line_Option_Mask ;
          Lines_Set    : in Array_Of_Vertices);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Nurb
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  non-uniform B-spline curve to be rendered. The order is
    --  specified as a positive integer. The spline shape is
    --  specified using a list of knots in the parametric
    --  coordinate space, plus a list of control points that are
    --  specified in modeling coordinates. In general, the number
    --  of control points must be at least as large as the order.
    --  The number of control points plus the spline order must
    --  equal the number of knots. The knots sequence must form a
    --  non-decreasing sequence of numbers. The type parameter
    --  specifies whether the curve is Rational or NonRational .
    --  If the type is Rational , then the point list must be
    --  provided as homogeneous modeling coordinates
    --  (COORD_4D)otherwise the the point list must be provided
    --  as non-homogeneous modeling coordinates (COORD_3D)The
    --  parameter range values tmin and tmax specify the range
    --  over which the B-spline curve is evaluated. Tmin must be
    --  less than tmax , tmin must be greater than or equal to
    --  the order 'th knot value, and tmax must be less than or
    --  equal to the (k+1- order )knot value, where k is the
    --  number of knots. Depending on the setting of the line
    --  attribute ASF values, line_color , line_type , line_width
    --  , and curve_approx attributes are either obtained
    --  directly from the current line attribute values or from
    --  the line_bundle_index 'th entry in the renderer's
    --  line_bundle. When an output command of this type is
    --  interpreted, if the curve order is not supported by the
    --  PEX server, the output command has no visual effect. In
    --  immediate mode, such a primitive is ignored. Several
    --  conditions may cause an OutputCommand error to be
    --  reported: type is not Rational or NonRational , order
    --  is less than one, there are fewer control points than the
    --  order, the number of knots and control points is not
    --  the same, the knots are not non-decreasing, tmin and tmax
    --  are inconsistent with the knots, or a Rational control
    --  point has a w coordinate that is less than or equal to
    --  zero.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Nurb
        ( Buffer  : in out Output_Command_Buffer;
          Curve   : in Non_Uniform_B_Spline_Curve);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  fill area primitive to be rendered. A fill area is
    --  defined by a list of vertices which are to be joined
    --  together to form a planar surface. (Fill areas are not
    --  strictly required to be planar, but strange shading
    --  artifacts can occur if a fill area is not planar or
    --  nearly so.)The first vertex of a fill area is connected
    --  to the second, the second connected to the third, and so
    --  on. The last vertex is implicitly connected to the first.
    --  During the rendering process, the fill area vertices are
    --  transformed to positions in device coordinates. The
    --  surface colors are transformed by the light source
    --  shading computation (which utilizes the interior style
    --  and the reflection model)and are further modified by the
    --  depth-cueing computation and mapped to device colors.
    --  Fill areas are not displayed if they are outside the
    --  currently-defined clipping volume. Fill areas that cross
    --  the clipping volume are clipped, and only the
    --  portion(s)inside the clipping volume is (are)displayed. A
    --  fill area can cross over itself to create a complex
    --  shape. The odd-even rule is used for determining the
    --  points that lie in the interior of a fill area. The shape
    --  parameter is passed as a hint as to the type of fill area
    --  that is defined by the vertices . A shape hint of Unknown
    --  means that nothing is known about the shape of the fill
    --  area. A shape of Complex means that the fill area edges
    --  may self-intersect. A shape of Nonconvex means that the
    --  edges do not self-intersect, but the fill area is not
    --  wholly convex. Convex means that all of the interior
    --  angles of the fill area are convex. Fill areas that are
    --  of a higher complexity than indicated by their shape hint
    --  are drawn in an implementation-dependent manner. PEX
    --  server extensions may ignore the shape hint and treat all
    --  fill area primitives as shape Unknown . The ignore_edges
    --  parameter specifies whether or not surface edge
    --  attributes are to be applied to the fill area primitives.
    --  If it is True , no surface edges will ever be drawn for
    --  the fill area . If False , surface edges will be drawn
    --  using the current surface edge attributes if the surface
    --  edge flag is On . Depending on the setting of the surface
    --  edge attribute ASF values, the surface_edges ,
    --  surface_edge_color , surface_edge_type , and
    --  surface_edge_width attributes are either obtained
    --  directly from the current surface edge attribute values
    --  or from the edge_bundle_index 'th entry in the renderer's
    --  edge_bundle . Depending on the setting of the surface
    --  attribute ASF values, the surface_color , interior_style
    --  , interior_style_index , surface_interp , and
    --  reflection_model attributes are either obtained directly
    --  from the current surface attribute values or from the
    --  interior_bundle_index 'th entry in the renderer's
    --  interior_bundle . If, when rendered, the fill area is
    --  determined to be front-facing with respect to the point
    --  of view, the surface_color (obtained as previously
    --  described)and reflection_attr attributes are used to
    --  compute the color(s)of the fill area. If the fill area is
    --  determined to be back-facing with respect to the point of
    --  view, the bf_surface_color and bf_reflection_attr
    --  attributes are used instead. Regardless of the
    --  orientation of the fill area, if the interior_style is
    --  Pattern , the pattern_size , pattern_ref_pt ,
    --  pattern_ref_vec1 , and pattern_ref_vec2 attributes may be
    --  used to pattern the fill area. At structure creation
    --  time, if the fill area has fewer than three vertices, it
    --  is stored in the structure, but when this output
    --  command is interpreted, it has no visual effect. In
    --  immediate mode, such a primitive is ignored. An
    --  OutputCommand error is reported if the shape or
    --  ignore_edges parameter is invalid.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area
        ( Buffer       : in out Output_Command_Buffer;
          Shape        : in Surface_Shape;
          Ignore_Edges : in Boolean;
          Points       : in MC.Point_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area
    --
    --  <Description>
    --
    --  When processed, this command will cause a fill area
    --  primitive to be drawn. This primitive functions exactly
    --  as the 3D fill area primitive except that modeling
    --  coordinate positions are specified using only x- and y-
    --  coordinates, and the z -coordinate is always assumed to
    --  be zero. At structure creation time, if the fill area
    --  has fewer than three vertices, it is stored in the
    --  structure, but when this output command is interpreted,
    --  it has no visual effect. In immediate mode, such a
    --  primitive is ignored. An OutputCommand error is
    --  reported if the shape or ignore_edges parameter is
    --  invalid. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area
        ( Buffer       : in out Output_Command_Buffer;
          Shape        : in Surface_Shape;
          Ignore_Edges : in Boolean;
          Points       : in MC.Point_List_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area_3d_With_Data
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  fill area to be rendered. The behavior of this primitive
    --  is identical to that of the 3D fill area primitive,
    --  except that additional information can be specified for
    --  the fill area itself and for each vertex. Color values
    --  that are passed will be of the type specified by
    --  color_type . The parameter facet_attributes indicates the
    --  attributes which are specified for the fill area. The
    --  components of the facet attributes bitmask are, in order:
    --  color COLOR normal VECTOR_3D If any of the attribute bits
    --  is set, the corresponding attribute must be present in
    --  the data that defines the fill area facet and they must
    --  be passed in the order that they appear in the list
    --  above. If a color value is passed per facet, it is taken
    --  to be the intrinsic color of the front face of the facet.
    --  If a normal is passed per facet, it is taken to be the
    --  normal to the facet. (Normals are assumed to be unit
    --  vectors.)The parameter vert_attributes specifies the
    --  attributes which are specified at each fill area vertex.
    --  The components of the vertex attributes bitmask are, in
    --  order: color COLOR normal VECTOR_3D If any of the
    --  attribute bits is set, the corresponding attributes must
    --  be present for each vertex, and they must be passed after
    --  the coordinate data for each vertex in the order that
    --  they appear in the list above. If color values are passed
    --  per vertex, they are considered to be part of the
    --  primitive and are used instead of the surface_color
    --  attribute. Vertex colors will be utilized rather than
    --  facet colors if both are provided. If normals are passed
    --  per vertex, they are taken to be the normals at the
    --  vertices of the fill area. In addition, the use of
    --  per-vertex colors is affected by the surface_interp
    --  attribute, which is obtained directly from the
    --  surface_interp value if the surface_interp_asf attribute
    --  is set to Individual or from the interior_bundle_index
    --  'th entry in the renderer's interior_bundle . The
    --  surface_interp attribute defines how color values between
    --  the vertices are to be computed. The shape parameter is
    --  passed as a hint as to the type of fill area that is
    --  defined by the vertices. A shape hint of Unknown means
    --  that nothing is known about the shape of the fill area. A
    --  shape of Complex means that the fill area edges may
    --  self-intersect. A shape of Nonconvex means that the edges
    --  do not self-intersect, but the fill area is not wholly
    --  convex. Convex means that all of the interior angles of
    --  the fill area are convex. Fill areas that are of a higher
    --  complexity than indicated by their shape hint are drawn
    --  in an implementation-dependent manner. PEX server
    --  extensions may ignore the shape hint and treat all fill
    --  area primitives as shape Unknown . The ignore_edges
    --  parameter specifies whether or not surface edge
    --  attributes are to be applied to the fill area primitives.
    --  If it is True , no surface edges will ever be drawn for
    --  the fill area . If False , surface edges will be drawn
    --  using the current surface edge attributes if the surface
    --  edge flag is On . At structure creation time, if the
    --  fill area has fewer than three vertices, it is stored in
    --  the structure, but when this output command is
    --  interpreted, it has no visual effect. In immediate mode,
    --  such a primitive is ignored. An OutputCommand error is
    --  reported if the shape or ignore_edges parameter is
    --  invalid.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area_3d_With_Data
        ( Buffer          : in out Output_Command_Buffer;
          Shape           : in Surface_Shape;
          Ignore_Edges    : in Boolean ;
          Facet_Mask      : in Facet_Option_Mask ;
          Vertex_Mask     : in Facet_Option_Mask ;
          Facet           : in Facet_Data );

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area_Set
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  fill area set primitive to be drawn. This type of
    --  primitive is essentially a set of fill area primitives.
    --  Together, this set of fill areas defines a polygon with
    --  islands or holes. The ignore_edges parameter will be
    --  applied to all of the fill areas in the fill area set.
    --  The shape parameter is passed as a hint as to the type of
    --  contours that comprise the fill area set. A shape hint of
    --  Unknown means that nothing is known about the shape of
    --  the constituent contours. A shape of Complex means that
    --  some contours of the fill area set may have edges that
    --  self-intersect. A shape of Nonconvex means that none of
    --  the contours of the fill area set have edges that
    --  self-intersect, but some may not be wholly convex. Convex
    --  means that all of the interior angles of all of the
    --  contours are convex. Contours that are of a * higher
    --  complexity than indicated by their shape hint are drawn
    --  in an implementation-dependent manner. PEX server
    --  extensions may ignore the shape hint and treat all
    --  constituent contours as shape Unknown . The contour_hint
    --  parameter provides further information as to the
    --  relationships between contours in the fill area set. If
    --  contour_hint is Disjoint , all contours will be spatially
    --  disjoint. No overlapping or intersection occurs between
    --  any contours in the fill area set. If contour_hint is
    --  Nested , contours will either be disjoint or wholly
    --  contained within another contour. No contour will have
    --  edges that intersect or are coincident with edges of any
    --  other contour. If contour_hint is Intersecting , separate
    --  contours may have edges that are coincident or overlap.
    --  If contour_hint is Unknown , nothing is known about the
    --  interrelationships between contours. Fill area sets with
    --  contours that have higher complexity interrelationships
    --  than that indicated by their contour hint are drawn in an
    --  implementation-dependent manner. PEX server extensions
    --  may ignore the contour hint and treat all fill area sets
    --  as Unknown . At structure creation time, if any of the
    --  contours of the fill area set has fewer than three
    --  vertices, or if there are no contours defined, the
    --  output command is stored in the structure, but when this
    --  output command is interpreted, it has no visual effect.
    --  In immediate mode, such a primitive is ignored. An
    --  OutputCommand error is reported if the shape or
    --  ignore_edges parameter is invalid. 
    -- 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area_Set
        ( Buffer       : in out Output_Command_Buffer;
          Shape        : in Surface_Shape;
          Ignore_Edges : in Boolean;
          Contour_Hint : in Contour_Hints;
          Point_Lists  : in MC.List_Of_Point_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area_Set
    --
    --  <Description>
    --
    --  When processed, this command will cause a fill area set
    --  primitive to be drawn. This primitive functions exactly
    --  as the 3D fill area set primitive except that modeling
    --  coordinate positions are specified using only x- and y-
    --  coordinates, and the z -coordinate is always assumed to
    --  be zero. At structure creation time, if any of the
    --  contours of the fill area set has fewer than three
    --  vertices, or if there are no contours defined, the
    --  output command is stored in the structure, but when this
    --  output command is interpreted, it has no visual effect.
    --  In immediate mode, such a primitive is ignored. An
    --  OutputCommand error is reported if the shape or
    --  ignore_edges parameter is invalid.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area_Set
        ( Buffer       : in out Output_Command_Buffer;
          Shape        : in Surface_Shape;
          Ignore_Edges : in Boolean;
          Contour_Hint : in Contour_Hints;
          Point_Lists  : in MC.List_Of_Point_List_2);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Area_Set_3d_With_Data
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  fill area set to be rendered. The behavior of this
    --  primitive is identical to that of the 3D fill area set
    --  primitive, except that additional information can be
    --  specified for of the fill area set, for each edge, and
    --  for each vertex. Color values that are passed will be of
    --  the type specified by color_type . The parameter
    --  facet_attributes indicates the attributes which are
    --  specified in the facet_data parameter. The components of
    --  the facet attributes bitmask are, in order: color COLOR
    --  normal VECTOR_3D If any of the attribute bits is set, the
    --  corresponding attributes must be present in the
    --  facet_data parameter and must be passed in the order that
    --  they appear in the list above. If a color value is passed
    --  as part of facet_data , it is taken to be the intrinsic
    --  color of the front face of the fill area set. The
    --  parameter vert_attributes specifies the attributes which
    --  are specified at each fill area set vertex. The
    --  components of the vertex attributes bitmask are, in
    --  order: color COLOR normal VECTOR_3D edges SWITCH If any
    --  of the attribute bits is set, the corresponding
    --  attributes must be present for each vertex, and they must
    --  be passed after the coordinate data for each vertex in
    --  the order that they appear in the list above. If color
    --  values are passed as part of facet_data or as part of the
    --  vertices list, they are considered to be part of the
    --  primitive and are used instead of the surface_color
    --  attribute. Vertex colors will be utilized rather than
    --  facet colors if both are provided. If normals are passed
    --  per vertex, they are taken to be the normals at the
    --  vertices of the fill area. If surface edge flags are
    --  specified per vertex, each flag specifies whether to draw
    --  the edge from the vertex with which the flag is specified
    --  to the next vertex. (E.g., for a facet with four
    --  vertices, the edge flag associated with vertex #1
    --  indicates whether to draw edge #1-#2, edge flag #2
    --  specifies edge #2- #3, edge flag #3 specifies edge #3-#4,
    --  and edge flag #4 specifies edge #4-#1.)Surface edges are
    --  always drawn with the surface edge color, never with per
    --  facet or per vertex colors. In addition, the use of
    --  per-vertex colors is affected by the surface_interp
    --  attribute, which is obtained directly from the
    --  surface_interp value if the surface_interp_asf attribute
    --  is set to Individual or from the interior_bundle_index
    --  'th entry in the renderer's interior_bundle . The
    --  surface_interp attribute defines how color values between
    --  the vertices are to be computed. At structure creation
    --  time, if any of the contours of the fill area set has
    --  fewer than three vertices, or if there are no contours
    --  defined, the output command is stored in the structure,
    --  but when this output command is interpreted, it has no
    --  visual effect. In immediate mode, such a primitive is
    --  ignored. An OutputCommand error is reported if the
    --  shape ignore_edges , or contour_hint parameter is
    --  invalid.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Fill_Area_Set_3d_With_Data
        ( Buffer          : in out Output_Command_Buffer;
          Shape           : in Surface_Shape;
          Ignore_Edges    : in Boolean ;
          Contour_Hint    : in Contour_Hints;
          Facet_Mask      : in Facet_Option_Mask ;
          Vertex_Mask     : in Area_Set_Option_Mask ;
          Facet_Options   : in Optional_Data ;
          Facet_Set       : in Array_Of_Vertices );

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Triangle_Strip
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  triangle strip primitive to be drawn. Color values that
    --  are passed will be of the type specified by color_type .
    --  The parameter facet_attributes indicates the attributes
    --  which are specified for each facet of the triangle strip.
    --  The components of the facet attributes bitmask are, in
    --  order: color COLOR normal VECTOR_3D If any of the
    --  attribute bits is set, the corresponding attribute must
    --  be present in facet_data , which is the data that defines
    --  each triangular facet. The attributes that are passed in
    --  this way must be passed in the order that they appear in
    --  the list above. If a color value is passed per facet, it
    --  is taken to be the intrinsic color of the front face of
    --  the facet. If a normal is passed per facet, it is taken
    --  to be the normal to the facet. (Normals are assumed to be
    --  unit vectors.)There will be n-2 entries in the facet_data
    --  list, where n is the number of entries in the vertices
    --  list. The parameter vert_attributes specifies the
    --  attributes which are specified at each triangle strip
    --  vertex. The components of the vertex attributes bitmask
    --  are, in order: color COLOR normal VECTOR_3D If any of the
    --  attribute bits is set, the corresponding attribute must
    --  be present for each vertex, and it must be passed after
    --  the coordinate data for each vertex. The attributes that
    --  are passed in this way must be passed in the order that
    --  they appear in the list above. If color values are passed
    --  per vertex, they are considered to be part of the
    --  primitive and are used instead of the surface_color
    --  attribute. Vertex colors will be utilized rather than
    --  facet colors if both are provided. If normals are passed
    --  per vertex, they are taken to be the normals at the
    --  vertices of the facet. The triangle strip is created from
    --  the vertex array. The strip is composed of n-2 triangles,
    --  where n is the number of vertices. The first triangle is
    --  formed from the first three vertices in the list, the
    --  second triangle is formed by the second through the
    --  fourth vertices in the list, etc., up to the last
    --  triangle, which is formed by the last three vertices in
    --  the list. All attributes that affect the representation
    --  of fill area sets also affect the representation of the
    --  triangle strip primitive. At structure creation time,
    --  if the triangle strip has fewer than three vertices, the
    --  output command is stored in the structure, but when
    --  this output command is interpreted, it has no visual
    --  effect. In immediate mode, such a primitive is ignored.
    -- 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Triangle_Strip
        ( Buffer           : in out Output_Command_Buffer;
          Facet_Mask       : in Facet_Option_Mask ;
          Vertex_Mask      : in Facet_Option_Mask ;
          Triangle_Options : in Optional_Data_Array ;
          Triangles        : in Vertex_Array );

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Quadrilateral_Mesh
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  quadrilateral mesh primitive to be rendered. The shape
    --  parameter is passed as a hint as to the type of
    --  quadrilaterals that comprise the primitive. A shape hint
    --  of Unknown means that nothing is known about the shape of
    --  the constituent quadrilaterals. A shape of Complex means
    --  that the some quadrilaterals may have edges that
    --  self-intersect. A shape of Nonconvex means that none of
    --  the quadrilaterals have edges that self-intersect, but
    --  some may not be wholly convex. Convex means that all of
    --  the interior angles of all of the quadrilaterals are
    --  convex. Quadrilaterals that are of a higher complexity
    --  than indicated by their shape hint are drawn in an
    --  implementation-dependent manner. Color values that are
    --  passed will be of the type specified by color_type . The
    --  parameter facet_attributes indicates the attributes which
    --  are specified for each facet of the quadrilateral mesh.
    --  The components of the facet attributes bitmask are, in
    --  order: color COLOR normal VECTOR_3D If any of the
    --  attribute bits is set, the corresponding attribute must
    --  be present in facet_data , which is the data that defines
    --  each quadrilateral facet. The attributes that are passed
    --  in this way must be passed in the order that they appear
    --  in the list above. If a color value is passed per facet,
    --  it is taken to be the intrinsic color of the front face
    --  of the facet. If a normal is passed per facet, it is
    --  taken to be the normal to the facet. (Normals are assumed
    --  to be unit vectors.)The parameter vert_attributes
    --  specifies the attributes which are specified at each
    --  quadrilateral mesh vertex. The components of the vertex
    --  attributes bitmask are, in order: color COLOR normal
    --  VECTOR_3D If any of the attribute bits is set, the
    --  corresponding attribute must be present for each vertex,
    --  and it must be passed after the coordinate data for each
    --  vertex. The attributes that are passed in this way must
    --  be passed in the order that they appear in the list
    --  above. If color values are passed per vertex, they are
    --  considered to be part of the primitive and are used
    --  instead of the surface_color attribute. Vertex colors
    --  will be utilized rather than facet colors if both are
    --  provided. If normals are passed per vertex, they are
    --  taken to be the normals at the vertices of the facet. The
    --  surface will be created from a vertex array that is
    --  stored in row major order (i.e., the column number varies
    --  fastest as vertices are stored in the array)The
    --  (ith,jth)(i+1th,jth)(i+1th,j+1th)and (ith,j+1th)vertices
    --  are connected to create a single facet. Adjacent vertices
    --  are interconnected until the entire facet network is
    --  processed. There are m_pts \264 n_pts entries in the
    --  vertices array, and there are (m_pts-1)\264
    --  (n_pts-1)entries in the facet_data array if any per-facet
    --  attributes are passed. It is allowable for the boundary
    --  of a single facet to not reside in a single plane. The
    --  treatment of the vertex attributes in this case is
    --  implementation-dependent. * All attributes that affect
    --  the representation of fill area sets also affect the
    --  representation the quadrilateral mesh primitive. At
    --  structure creation time, if either m_pts or n_pts is less
    --  than two, the output command is stored in the |
    --  structure, but when this output command is interpreted,
    --  it has no visual effect. In immediate mode, such a |
    --  primitive is ignored. An OutputCommand error is
    --  reported if the shape parameter is invalid.
    -- 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Quadrilateral_Mesh
        ( Buffer          : in out Output_Command_Buffer;
          Shape           : in Surface_Shape ;
          Facet_Mask      : in Facet_Option_Mask ;
          Vertex_Mask     : in Facet_Option_Mask ;
          Quad_Options    : in Optional_Data_Array ;
          Quads           : in Vertex_Matrix );

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Of_Fill_Area_Sets
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Of_Fill_Area_Sets
        ( Buffer          : in out Output_Command_Buffer;
          Shape           : in Surface_Shape;
          Contour_Hint    : in Contour_Hints;
          Single_Contour  : in Boolean ;
          Facet_Mask      : in Facet_Option_Mask ;
          Vertex_Mask     : in Facet_Option_Mask ;
          Facet_Options   : in Optional_Data_Array ;
          Vertices        : in Vertex_Array ;
          Edge_Switches   : in Edge_Flags ;
          Contours        : in Set_Of_Contour_Array);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Nurb_Surface
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Nurb_Surface
        ( Buffer          : in out Output_Command_Buffer;
          Surface         : in Non_Uniform_B_Spline_Surface ;
          Trimming_Loops  : in Trimming_Loop_Array );

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Cell_Array
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will cause a
    --  cell array primitive to be rendered. Color values that
    --  are passed will be of type Indexed . A cell array is a
    --  parallelogram of equal-sized cells, each of which is a
    --  parallelogram and has a single color. Each cell has a
    --  width defined by: width== dx sr
    --  ( point 
    --  1. x - point 
    --  2. x )2 + ( point 
    --  1. y - point 
    --  2. y )2 + ( point 
    --  1. z - point 
    --  2. z )2  and a height defined by: height== dy sr  ( point     --  1. x - point 
    --  3. x )2 + ( point 
    --  1. y - point 
    --  3. y )2 + ( point 
    --  1. z - point 
    --  3. z )2 
    --  The colors are specified in a one-dimensional array of
    --  size dx \264 dy . The color of each cell is specified by
    --  the index of the corresponding element in the colors
    --  array. Colors are stored in this array by rows, that is,
    --  the column number varies fastest as colors are stored
    --  into the array. The first color in the array is the color
    --  at the cell at the corner of point1 , and subsequent
    --  colors represent the colors of cells proceeding to to
    --  point2 . If any color index is not defined, color index
    --  one is used. If color index one is not defined, the
    --  default values in Appendix D are used. An
    --  OutputCommand error is reported if dx or dy is zero.
    -- 
    -- 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Cell_Array
        ( Buffer   : in out Output_Command_Buffer;
          Corner_P : in MC.Point_3;
          Corner_Q : in MC.Point_3;
          Corner_R : in MC.Point_3;
          Cells    : in Color_Index_Matrix);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Cell_Array
    --
    --  <Description>
    --
    --  When processed, this command will cause a cell array
    --  primitive to be drawn. This primitive functions exactly
    --  as the 3D cell array primitive except that modeling
    --  coordinate positions are specified using only x- and y-
    --  coordinates, and the z -coordinate is always assumed to
    --  be zero. In addition, the cell array is defined by two
    --  points which define a rectangle that is taken to be
    --  aligned with the modeling coordinate axes. If any color
    --  index is not defined, color index one is used. If color
    --  index one is not defined, the default values in
    --  Appendix D are used. An OutputCommand error is reported
    --  if dx or dy is zero. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Cell_Array
        ( Buffer   : in out Output_Command_Buffer;
          Corner_P : in MC.Point_2;
          Corner_Q : in MC.Point_2;
          Cells    : in Color_Index_Matrix);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Extended_Cell_Array
    --
    --  <Description>
    --
    --  When processed by a renderer, this command has the same
    --  effect as the "cell array 3D" primitive, except that the
    --  colors may be passed as indexed or direct color values,
    --  depending on the setting of color_type . If any color
    --  index is not defined, color index one is used. If color
    --  index one is not defined, the default values in
    --  Appendix D are used. An OutputCommand error is reported
    --  if dx or dy is zero. 
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Extended_Cell_Array
        ( Buffer   : in out Output_Command_Buffer;
          Corner_P : in MC.Point_3;
          Corner_Q : in MC.Point_3;
          Corner_R : in MC.Point_3;
          Cells    : in Color_Matrix);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Generalized_Gdp_3
    --
    --  <Description>
    --
    --  When processed by a renderer, the effect of this command
    --  is implementation-dependent. Because of floating point
    --  and color format discrepancies across a network
    --  interface, it is not anticipated that the GDP 3D will
    --  provide a useful extension mechanism, but it is provided
    --  for PHIGS compatibility purposes. If the specified GDP
    --  identifier is not supported, the output command is
    --  ignored. Note that byte-swapping and floating point
    --  conversion may not be done on this type of output
    --  command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    generic
        type Gdp_Data_Record is private ;

    procedure Generalized_Gdp_3
        ( Buffer   : in out Output_Command_Buffer;
          Gdp      : in Gdp_3_Id;
          Points   : in MC.Point_List_3;
          Gdp_Data : in Gdp_Data_Record);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Generalized_Gdp_2
    --
    --  <Description>
    --
    --  When processed by a renderer, the effect of this command
    --  is implementation-dependent. Because of floating point
    --  and color format discrepancies across a network
    --  interface, it is not anticipated that the GDP 2D will
    --  provide a useful extension mechanism, but it is provided
    --  for PHIGS compatibility purposes. If the specified GDP
    --  identifier is not supported, the output command is
    --  ignored. Note that byte-swapping and floating point
    --  conversion may not be done on this type of output
    --  command.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    generic
        type Gdp_Data_Record is private ;

    procedure Generalized_Gdp_2
        ( Buffer   : in out Output_Command_Buffer;
          Gdp      : in Gdp_Id;
          Points   : in MC.Point_List_2;
          Gdp_Data : in Gdp_Data_Record);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  No_Operation
    --
    --  <Description>
    --
    --  When processed by a renderer, this command will only update the
    --  current path. It is useful for client-side accumulate state
    --  traversals since it does not generate any output.
    --
    --  <Parameters>
    --
    --  Buffer   : Application Output Commands Buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure No_Operation ( Buffer  : in out Output_Command_Buffer) ;
end Pex_Output_Commands;
