---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--  
--
--   Package : Pex_Pick
--
--   Version : 
--
--   Subject : PEX Picking functions
--
--   Section 11.1 -- Pick Device Descriptors
--   Get_Pick_Device
--   Change_Pick_Device      
--
--   Section 11.2 -- Pick Measure Resource Management 
--   Create_Pick_Measure
--   Delete_Pick_Measure
--
--   Section 11.3 -- Pick Measure Inquiry
--   Get_Pick_Measure
--
--   Section 11.4 -- Pick Operations
--   Update_Pick_Measure
--
--   Renderer Picking
--
--   Begin_Pick_One
--   End_Pick_One
--   Pick_One
--
--   Begin_Pick_All
--   End_Pick_All
--   Pick_All
--
-- 
with X_Types;
with Pex_Types;
-- 
package Pex_Pick is 
-- 
    use X_Types;
    use Pex_Types;
-- 
--  
--   Section 11.1 -- Pick Device Descriptors
--
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Get_Pick_Device
    --
    --  <Description>
    --
    --  This request will return components of a pick descriptor
    --  for the PHIGS workstation resource specified by wks_id .
    --  The descriptor returned will be the currently-defined
    --  descriptor for the pick device of the type specified by
    --  dev_type . The item_mask specifies which components are
    --  to be inquired and returned. The specified attributes of
    --  the pick device descriptor will be returned in item_list
    --  Floating point values in item_list will be returned in
    --  the floating point format specified in fp_format .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Wks_Id     : The workstation.
    --  Device     : The pick device whose characteristics are wanted.
    --  Mask       : Used to select components to inquire about.
    --  Values     : Values returned.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Get_Pick_Device ( Display_Id : in Display;
                                Wks_Id     : in Phigs_Workstation;
                                Device     : in Pick_Device_Type;
                                Mask       : in Pick_Device_Bitmask;
                                Values     : in out Pick_Device_Values);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Change_Pick_Device
    --
    --  <Description>
    --
    --  This request will modify components of a pick descriptor
    --  for the PHIGS workstation resource specified by wks_id .
    --  The descriptor to be modified will be the
    --  currently-defined descriptor for the pick device of the
    --  type specified by dev_type . The item_mask specifies
    --  which components are to be changed. The specified
    --  attributes of the pick device descriptor will be set to
    --  the values contained in item_list . Floating point values
    --  in item_list will be in the floating point format
    --  specified in fp_format .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Wks_Id     : The workstation.
    --  Device     : The pick device.
    --  Mask       : Used to select components to change.
    --  Values     : Values passed for pick device modification.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Change_Pick_Device ( Display_Id : in Display;
                                   Wks_Id     : in Phigs_Workstation;
                                   Device     : in Pick_Device_Type;
                                   Mask       : in Pick_Device_Bitmask;
                                   Values     : in Pick_Device_Values);
  
--
-- Section 11.2 -- Pick Measure Resource Management
--

    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Create_Pick_Measure
    --
    --  <Description>
    --
    --  This request creates a pick measure resource of the type
    --  specified by Device . The resource ID of the pick
    --  measure is returned. The newly-created pick
    --  measure is initialized with the values contained in the
    --  appropriate pick device descriptor stored in the PHIGS
    --  workstation resource specified by wks_id . 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Wks_Id     : The workstation.
    --  Device     : The pick device.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    function Create_Pick_Measure ( Display_Id : in Display;
                                   Wks_Id     : in Phigs_Workstation;
                                   Device     : in Pick_Device_Type)
        return Pick_Measure;
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Free_Pick_Measure
    --
    --  <Description>
    --
    --  This request deletes the pick measure resource and frees
    --  the storage associated with it.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Measure_Id : The pick measure to delete.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Free_Pick_Measure ( Display_Id : in Display;
                                  Measure_Id : in Pick_Measure);
    
--
--  Section 11.3 -- Pick Measure Inquiry
--
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Get_Pick_Measure
    --
    --  <Description>
    --
    --  This request will return components of the pick measure
    --  specified by pm_id . The item_mask specifies which
    --  components are to be inquired and returned. The specified
    --  attributes of the pick measure will be returned in
    --  item_list .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Measure_Id : The pick measure to inquire about.
    --  Mask       : Used to select the components to inquire about.
    --  Values     : The returned values.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Get_Pick_Measure ( Display_Id : in Display;
                                 Measure_Id : in Pick_Measure;
                                 Mask       : in Pick_Measure_Bitmask;
                                 Values     : in out Pick_Measure_Values);
    
--
-- Section 11.4 -- Pick Operations
--
   
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Update_Pick_Measure
    --
    --  <Description>
    --
    --  This request will update the pick measure specified by
    --  pm_id using the information found in input_record . If
    --  this update causes a primitive to be picked, the pick
    --  measure's pick_status will be set to Ok and the
    --  picked_prim will be set to the path of the picked
    --  primitive. If no primitive was picked, the pick_status
    --  will be set to NoPick . The data in input_record is
    --  dependent on the type of pick device for which the pick
    --  measure resource was created, that is, the dev_type
    --  specified to PEXCreatePickMeasure . PEXGetPickMeasure can
    --  be used to return the results of a PEXUpdatePickMeasure
    --  request.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Measure_Id : The pick measure to modify.
    --  Values     : The datas to use for the update.
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Update_Pick_Measure ( Display_Id : in Display;
                                    Measure_Id : in Pick_Measure;
                                    Values     : in Pick_Data ) ;
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Begin_Pick_One
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Method       :
    --  Drawable_Ex  :
    --  Structure_Id :
    --  Data         :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Begin_Pick_One ( Display_Id   : in Display;
                               Renderer_Id  : in Renderer;
                               Method       : in Pick_One_Method;
                               Drawable_Ex  : in Drawable;
                               Structure_Id : in User_Structure ;
                               Data         : in Pick_Data);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  End_Pick_One
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Status       :
    --  Better_Exist :
    --  Path         :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure End_Pick_One ( Display_Id   : in Display;
                             Renderer_Id  : in Renderer;
                             Status       : out Pick_Status;
                             Better_Exist : out Boolean;
                             Path         : in out User_Pick_Path);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Pick_One
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Method       :
    --  Drawable_Ex  :
    --  Structure_Id :
    --  Data         :
    --  Status       :
    --  Better_Exist :
    --  Path         :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Pick_One ( Display_Id   : in Display;
                         Renderer_Id  : in Renderer;
                         Method       : in Pick_One_Method;
                         Drawable_Ex  : in Drawable;
                         Structure_Id : in Structure ;
                         Data         : in Pick_Data;
                         Status       : out Pick_Status;
                         Better_Exist : out Boolean;
                         Path         : in out Pick_Path);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Begin_Pick_All
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Method       :
    --  Drawable_Ex  :
    --  Structure_Id :
    --  Max_Hits     :
    --  Send_Event   :
    --  Data         :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Begin_Pick_All ( Display_Id   : in Display;
                               Renderer_Id  : in Renderer;
                               Method       : in Pick_All_Method;
                               Drawable_Ex  : in Drawable;
                               Structure_Id : in User_Structure ;
                               Max_Hits     : in X_Natural ;
                               Send_Event   : in Boolean ;
                               Data         : in Pick_Data);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  End_Pick_All
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Status       :
    --  Remaining    :
    --  Paths        :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure End_Pick_All ( Display_Id   : in Display;
                             Renderer_Id  : in Renderer;
                             Status       : out Pick_Status;
                             Remaining    : out Pick_All_State;
                             Paths        : in out User_Pick_Path_List);
    
    --=============================================================== =====
    --  <Subprogram Name>
    --
    --  Pick_All
    --
    --  <Description>
    --
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Renderer_Id  :
    --  Method       :
    --  Drawable_Ex  :
    --  Structure_Id :
    --  Max_Hits     :
    --  Data         :
    --  Status       :
    --  Remaining    :
    --  Paths        :
    --
    --  <Errors Raised>
    --
    --=============================================================== =====

    procedure Pick_All ( Display_Id   : in Display;
                         Renderer_Id  : in Renderer;
                         Method       : in Pick_All_Method;
                         Drawable_Ex  : in Drawable;
                         Max_Hits     : in X_Natural ;
                         Data         : in Pick_Data;
                         Status       : out Pick_Status;
                         Remaining    : out Pick_All_State;
                         Paths        : in out Pick_Path_List);
    
end Pex_Pick ;
