---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--
--   Package Pex_Pipeline
--
--   Version :
--
--   Subject : Pipeline Context Functions
--   
--   Section 5.1 -- Pipeline Context Resource Management
--   Create_Pipeline_Context
--   Copy_Pipeline_Context
--   Free_Pipeline_Context
--   
--   Section 5.2 -- Pipeline Context Inquiry
--   Get_Pipeline_Context
--  
--   Section 5.3 -- Pipeline Context Modification
--   Change_Pipeline_Context
--
-- 
with X_Types;
with Pex_Types;
-- 
package Pex_Pipeline is 
-- 
    use X_Types;
    use Pex_Types;
-- 
-- 
-- 
--   Section 5.1 -- Pipeline Context Resource Management
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Create_Pipeline_Context
    --
    --  <Description>
    --
    --  This request creates a pipeline context resource for the
    --  specified pc_id . The item_mask defines those pipeline
    --  context attributes that are to be explicitly set at the
    --  time the resource is created. The item_list contains the
    --  corresponding list of values used to modify the
    --  newly-created pipeline context. Floating point values in
    --  item_list will be in the floating point format specified
    --  in fp_format . Similarly, color values in item_list will
    --  be the color type specified by color_type .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Item_Mask  : Mask of values to set.
    --  Items      : Values to set.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    function Create_Pipeline_Context
        ( Display_Id : in Display;
          Item_Mask  : in Pipeline_Context_BitMask;
          Items      : in Pipeline_Context_Values)
        return Pipeline_Context;
  
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Copy_Pipeline_Context
    --
    --  <Description>
    --
    --  This request copies the source pipeline context src_pc_id
    --  to a destination pipeline context dest_pc_id . The
    --  dest_pc_id must already exist as a valid resource. The
    --  item_mask indicates which values in the pipeline context
    --  will be copied. 
    --
    --  <Parameters>
    --
    --  Display_Id   : The X Server connection.
    --  Src_Pipeline : The pipeline context to copy.
    --  Dst_Pipeline : The destination pipeline context.
    --  Item_Mask    : Mask of values to copy.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Copy_Pipeline_Context
        ( Display_Id   : in Display;
          Src_Pipeline : in Pipeline_Context;
          Dst_Pipeline : in Pipeline_Context;
          Item_Mask    : in Pipeline_Context_Bitmask);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Free_Pipeline_Context
    --
    --  <Description>
    --
    --  This request deletes the association between the resource
    --  ID and the pipeline context. The pipeline context storage
    --  will be freed when no other resource references it.
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  Pipeline_Id : The pipeline context to free.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Free_Pipeline_Context ( Display_Id   : in Display;
                                      Pipeline_Id  : in Pipeline_Context);

--
--  Section 5.2 -- Pipeline Context Inquiry
--

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Pipeline_Context
    --
    --  <Description>
    --
    --  This request will return components of the pipeline
    --  context specified by pc_id . The item_mask specifies
    --  which components are to be inquired and returned. The
    --  specified attributes of the pipeline context will be
    --  returned in item_list . Floating point values in
    --  item_list will be returned in the floating point format
    --  specified in fp_format . 
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  Pipeline_Id : The pipeline context.
    --  Item_Mask   : Mask of values to get.
    --  Items       : Values got.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Get_Pipeline_Context
        ( Display_Id  : in Display;
          Pipeline_Id : in Pipeline_Context;
          Item_Mask   : in Pipeline_Context_BitMask;
          Items       : in out Pipeline_Context_Values);
--
--  Section 5.3 -- Pipeline Context Modification
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Change_Pipeline_Context
    --
    --  <Description>
    --
    --  This request will modify components of the pipeline
    --  context specified by pc_id . The item_mask specifies
    --  which components are to be modified. The values for the
    --  attributes that are to be modified are contained in
    --  item_list . Floating point values in item_list will be in
    --  the floating point format specified in fp_format .
    --  Similarly, color values in item_list will be the color
    --  type specified by color_type . 
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  Pipeline_Id : The pipeline context to modify.
    --  Item_Mask   : Mask of values to change.
    --  Items       : Values to change.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Change_Pipeline_Context
        ( Display_Id  : in Display;
          Pipeline_Id : in Pipeline_Context;
          Item_Mask   : in Pipeline_Context_BitMask;
          Items       : in Pipeline_Context_Values);
    
end Pex_Pipeline;
