---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--
--  Package : Pex_Structures
--
--  Version :
--
--  Subject :  PEX Structures functions
--
--  Section 7.1 -- Structure Resource Management
--  Create_Structure
--  Copy_Structure
--  Destroy_Structures
--
--  Section 7.2 -- Structure Inquiry
--  Get_Structure_Information
--  Get_Element_Information
--  Get_Structures_In_Network
--  Get_Ancestors
--  Get_Descendants
--  Fetch_Elements
--
--  Section 7.3 -- Structure Resource Attribute Modification
--  Set_Editing_Mode
--  Set_Element_Pointer
--  Set_Element_Pointer_At_Label
--  Element_Search
--
--  Section 7.4 -- Structure Editing
--  Store_Elements
--  Delete_Elements
--  Delete_Elements_To_Label
--  Delete_Elements_Between_Labels
--  Copy_Elements
--  Change_Structure_References
--
--
with X_Types;
with Pex_Types;
--
package Pex_Structures is
--
    use X_Types;
    use Pex_Types;
--
--
--  Section 7.1 -- Structure Resource Management
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Create_Structure
    --
    --  <Description>
    --
    --  This request creates a structure resource with the
    --  returned s_id
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  function Create_Structure ( Display_Id : in Display) return Structure;

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Copy_Structure
    --
    --  <Description>
    --
    --  The structure elements in src_s_id are copied to
    --  dest_s_id . Any structure elements in dest_s_id are
    --  deleted prior to the copy operation. The destination
    --  structure must already exist as a valid resource.
    --  Src_s_id 's element pointer and editing mode are also
    --  copied to dest_s_id .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Src_Structure : The structure whose content will be copied.
    --  Dst_Structure : The destination structure.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Copy_Structure ( Display_Id    : in Display;
                             Src_Structure : in Structure;
                             Dst_Structure : in Structure);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Destroy_Structures
    --
    --  <Description>
    --
    --  This request destroys each of the structure resources
    --  that is specified in list and removes all references to
    --  it in the server. Any structure elements that reference a
    --  structure in list will be destroyed, and any structure in
    --  list that is posted to a PHIGS workstation resource will
    --  be unposted. Any paths in search contexts or pick
    --  measures that contain the resource ID of any structure in
    --  list may still be queried. Such paths may still contain
    --  the resource ID of a structure after it has been
    --  destroyed. However, any attempts to perform a
    --  PEXSearchNetwork or a PEXUpdatePickMeasure using such a
    --  path will result in a Path error being generated. Any
    --  structure that had structure elements removed due to a
    --  PEXDestroyStructures request will have their element
    --  pointer modified in the following way. If the element
    --  pointer is at a position before any of the elements that
    --  were deleted, the element pointer remains unchanged. If
    --  the element pointer was at the position of an element
    --  that was deleted, it will be set to the previous element.
    --  If the element pointer was after elements that were
    --  deleted, it will be updated so as to point at the same
    --  element it pointed to prior to the PEXDestroyStructures
    --  request.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structures : The structures to destroy.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Destroy_Structures ( Display_Id : in Display;
                                 Structures : in Structure_Array );
--
--  Section 7.2 -- Structure Inquiry
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Structure_Information
    --
    --  <Description>
    --
    --  This request returns information about the structure
    --  specified by s_id . Item_mask indicates which fields in
    --  the reply are required by the client. Items indicated by
    --  item_mask will have valid values returned, and the
    --  remaining values will be returned as undefined values.
    --  The current value of the structure's editing mode
    --  attribute will be returned in editing_mode . The current
    --  value of the structure's element pointer attribute will
    --  be returned in element_ptr . The number of structure
    --  elements in the specified structure will be returned in
    --  num_elements . If the structure is empty, num_elements
    --  will be zero. The total number of four-byte units
    --  necessary to store all elements of the structure (on the
    --  client side)will be returned in total_length . (This
    --  provides the client the information needed to allocate
    --  memory prior to doing a PEXFetchElements request.)If
    --  there are any "execute structure" elements in other
    --  structure resources that reference s_id , has_refs will
    --  be returned as True , otherwise it will be returned as
    --  False . The information returned will be computed as if
    --  the floating point type in fp_format were being used to
    --  return the data.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure to query about.
    --  Item_Mask    : The selection of items to query.
    --  Editing_Mode : Current editing mode.
    --  Element      : Current element position.
    --  Num_Elements : Number of contained elements.
    --  Total_Length : Size required for elements storage.
    --  Has_Refs     : True if structure is referenced.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Get_Structure_Information
      ( Display_Id     : in Display;
        Structure_Id   : in Structure;
        Item_Mask      : in Structure_Info_Mask_Type;
        Editing_Mode   : out Edit_Mode;
        Element        : out Element_Position;
        Num_Elements   : out Element_Position;
        Total_Length   : out X_Natural;
        Has_References : out Boolean );
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Element_Information
    --
    --  <Description>
    --
    --  This request returns information about elements in the
    --  structure specified by s_id . The offsets that define the
    --  range of elements about which information is to be
    --  obtained are computed using the positions in range in the
    --  following fashion. If whence is Beginning , the computed
    --  offset is just the value specified by offset . If whence
    --  is Current , the offset is computed by adding offset to
    --  s_id 's element pointer. If whence is End , the offset is
    --  computed by adding offset to the number of elements in
    --  the structure. Offset can be a negative number. If either
    --  computed offset is less than zero, it will be set to zero
    --  before obtaining the element information. If either
    --  computed offset is greater than the number of elements in
    --  the structure, it will be set to the offset of the last
    --  structure element in the structure. The request returns
    --  each element's ELEMENT_TYPE field, and the length of the
    --  element in units of four bytes. The sum of the lengths of
    --  all elements in the structure will be equal to the length
    --  that can be inquired with the PEXGetStructureInfo
    --  request. The information returned will be computed as if
    --  the floating point type in fp_format were being used to
    --  return the data. No information will be returned for
    --  inquiries on element offset zero. The element pointer
    --  attribute of s_id is not affected by this request.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure
    --  Between      : Range of elements to query.
    --  Elements     : The elements list returned.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Get_Element_Information ( Display_Id   : in Display;
                                      Structure_Id : in Structure;
                                      Between      : in Element_Range;
                                      Elements     : out Element_Info_List);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Structures_In_Network
    --
    --  <Description>
    --
    --  This request returns a list of unique structure resource
    --  IDs that are referenced in the structure network rooted
    --  at s_id . If which is All , all of the structure
    --  resources referenced in the structure network rooted at
    --  s_id will be returned in the list structures . If which
    --  is NoCrossRefs , only the IDs of structures in the
    --  network that aren't referenced outside the specified
    --  structure network are returned. S_id will always be
    --  returned in the list, unless s_id is an invalid structure
    --  ID, in which case the returned list will be empty.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Which        : All_Structures or No_Cross_Refs.
    --  Structures   : The list of structures returned.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Get_Structures_In_Network ( Display_Id   : in Display;
                                        Structure_Id : in Structure;
                                        Which        : in Network_Filter;
                                        Structures   : out Structure_List);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Ancestors
    --
    --  <Description>
    --
    --  This request returns unique paths in the structure
    --  hierarchy that lead to s_id . Paths are returned as a
    --  list of structure id/element offset pairs that are in the
    --  order from the root structure down to s_id . For full
    --  paths, the last entry in each path would have the
    --  structure ID equal to s_id and the offset equal to zero.
    --  The path_depth indicates the maximum length (number of
    --  structure id/element offset pairs)of each path, except
    --  that path_depth== 0 means that the full path is to be
    --  returned. The path_part indicates whether the head or the
    --  tail of each path is to be returned. If path_part==
    --  TopPart , only the first path_depth structure id/element
    --  offset pairs in each path will be returned. If path_part
    -- == BottomPart , only the last path_depth structure
    --  id/element offset pairs in each path will be returned.
    --  Only unique paths will be returned (i.e., there will be
    --  no duplicates in the list of returned paths)For instance,
    --  specifying path_depth== 0 and path_part = TopPart would
    --  cause all paths leading to s_id to be returned.
    --  Specifying path_depth== 1 and path_part = TopPart would
    --  cause all the structures at the top of each structure
    --  network containing s_id to be returned. Specifying
    --  path_depth== 1 and path_part = BottomPart would allow you
    --  to determine whether or not s_id is referenced.
    --  Specifying path_depth== 2 and path_part = BottomPart
    --  would allow you to determine the number of immediate
    --  ancestors for s_id as well as returning their resource
    --  IDs.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Order        : Top_First or Bottom_First
    --  Depth        : The desired depth.
    --  Paths        : The returned paths list.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Get_Ancestors ( Display_Id   : in Display;
                            Structure_Id : in Structure;
                            Order        : in Path_Order;
                            Depth        : in Path_Depth;
                            Paths        : out Element_Path_List);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Descendants
    --
    --  <Description>
    --
    --  This request returns unique paths in the structure
    --  hierarchy from s_id to leaf nodes in the hierarchy. Paths
    --  are returned as a list of structure id/element offset
    --  pairs that are in the order from s_id down to the leaf
    --  nodes. For full paths, the first entry in each path would
    --  have the structure ID equal to s_id and the offset equal
    --  to the offset of the element that references a descendant
    --  in the hierarchy, and the last entry would have the
    --  structure ID of a leaf node in the structure hierarchy
    --  and an offset of zero. The path_depth indicates the
    --  maximum length (number of structure id/element offset
    --  pairs)of each path, except that path_depth== 0 means that
    --  the full path is to be returned. The path_part indicates
    --  whether the head or the tail of each path is to be
    --  returned. If path_part== TopPart , only the first
    --  path_depth structure id/element offset pairs in each path
    --  will be returned. If path_part== BottomPart , only the
    --  last path_depth structure id/element offset pairs in each
    --  path will be returned. Only unique paths will be returned
    --  (i.e., there will be no duplicates in the list of
    --  returned paths)For instance, specifying path_depth== 0
    --  and path_part== TopPart would cause all paths leading
    --  from s_id to leaf nodes to be returned. Specifying
    --  path_depth== 1 and path_part = TopPart would cause all
    --  the direct references to other structures in s_id to be
    --  returned. Specifying path_depth== 1 and path_part =
    --  BottomPart could be used to determine the leaf nodes for
    --  the structure hierarchy rooted at s_id .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Order        :
    --  Depth        :
    --  Paths        : The returned paths list.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Get_Descendants ( Display_Id   : in Display;
                              Structure_Id : in Structure;
                              Order        : in Path_Order;
                              Depth        : in Path_Depth;
                              Paths        : out Element_Path_List);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Fetch_Elements
    --
    --  <Description>
    --
    --  This request returns a range of structure elements in
    --  the structure specified by s_id . The offsets that define
    --  the range of elements to be fetched are computed using the
    --  positions in range in the following fashion. If whence is
    --  Beginning , the computed offset is just the value specified by
    --  offset . If whence is Current , the offset is computed by adding
    --  offset to s_id 's element pointer. If whence is End , the offset
    --  is computed by adding offset to the number of elements in the
    --  structure. Offset can be a negative number. If either computed
    --  offset is less than zero, it will be set to zero before the
    --  fetch occurs. If either computed offset is greater than the
    --  number of elements in the source structure, it will be set to
    --  the offset of the last structure element in the source
    --  structure. The element pointer attribute of s_id is not affected
    --  by the fetch operation. The data for each structure element will
    --  be returned in the same format as described Section 3 - Output
    --  Commands section. Floating point values in elements will be
    --  returned in the floating point format specified in fp_format (if
    --  it is supported by the PEX implementation). No information will
    --  be returned for inquiries on element offset zero.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Between      : The element range to query about.
    --  Element_Data : The elements contents
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Fetch_Elements ( Display_Id   : in Display;
                             Structure_Id : in Structure;
                             Between      : in Element_Range;
                             Element_Data : in out Output_Command_Buffer);
--
--  Section 7.3 -- Structure Resource Attribute Modification
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Editing_Mode
    --
    --  <Description>
    --
    --  This request modifies s_id 's editing mode attribute. The editing
    --  mode attribute specifies how editing operations will affect the
    --  structure. If the editing mode is set to StructureInsert ,
    --  subsequent requests to create structure elements will cause
    --  elements to be inserted into the structure. Elements will be
    --  inserted into the structure after the structure element
    --  specified by the element pointer. The element pointer will then
    --  be incremented by the number of elements inserted. If the mode
    --  is StructureReplace , output requests used to create structure
    --  elements will cause structure elements to replace elements
    --  starting at the location specified by the element pointer.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Editing_Mode : Insert or Replace.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Set_Editing_Mode ( Display_Id   : in Display;
                               Structure_Id : in Structure;
                               Editing_Mode : in Edit_Mode);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Element_Pointer
    --
    --  <Description>
    --
    --  This request sets the element pointer attribute of the structure
    --  specified by s_id . The element pointer attribute of the
    --  structure will be set to the position specified by position ,
    --  which contains a whence and offset pair. If whence is Beginning
    --  , the element pointer is set to the specified offset . If whence
    --  is Current , the new element pointer value is computed by adding
    --  offset to the current value of the element pointer. If whence is
    --  End , the new element pointer value is computed by adding the
    --  specified offset to the offset of the last element in the
    --  structure. Offset can be a negative number. If the resultant
    --  value of the element pointer is less than zero, the element
    --  pointer is set to zero. If the resultant value of the element
    --  pointer is greater than the number of elements in the structure,
    --  then the element pointer is set to the offset of the last
    --  element in the structure.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Offset       : The new element position.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Set_Element_Pointer ( Display_Id   : in Display;
                                  Structure_Id : in Structure;
                                  Offset       : in Element_Location);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Element_Pointer_At_Label
    --
    --  <Description>
    --
    --  This request sets the element pointer attribute of the structure
    --  specified by s_id . A search is conducted for the next
    --  occurrence of a "label" structure element containing label . The
    --  search for the label starts at the current element pointer plus
    --  one, and proceeds in the forward direction. If a "label"
    --  structure element containing label is found, the element pointer
    --  for the structure is set to the offset of the located label plus
    --  offset . If the resultant value of the element pointer is less
    --  than zero, the element pointer is set to zero. If the resultant
    --  value of the element pointer is greater than the number of
    --  elements in the structure, then the element pointer is set to
    --  the offset of the last element in the structure. This is a
    --  non-descending search (i.e., the search does not include any
    --  structures referenced by "execute structure" elements)If no
    --  occurrence of the specified label is found, an error is
    --  generated and the structure's element pointer is left unchanged.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Label        : Label to find within structure.
    --  Offset       : Offset to position element pointer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Set_Element_Pointer_At_Label ( Display_Id   : in Display;
                                           Structure_Id : in Structure;
                                           Label        : in Label_Id;
                                           Offset       : in Element_Position);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Element_Search
    --
    --  <Description>
    --
    --  This request conducts a search for the first occurrence of the
    --  specified element type in the structure specified by s_id . The
    --  offset at which to begin searching is computed using the whence
    --  and offset found in position . If whence is Beginning , the
    --  search will begin at the element specified by offset . If whence
    --  is Current , the specified offset is added to the current value
    --  of s_id 's element pointer to determine the element at which to
    --  begin the search. If whence is End , the element at which to
    --  begin the search is computed by adding the specified offset to
    --  the offset of the last element in the structure. Offset can be a
    --  negative number. If the computed offset is less than zero, the
    --  search will begin at the position preceding the first element in
    --  the structure. If the computed offset is greater than the number
    --  of elements in the structure, then the search will begin at the
    --  last element in the structure. The search always includes the
    --  starting element. An element will be selected if its element
    --  type is not contained in excl and its element type is included
    --  in incl . An element type of All causes all element types to
    --  match. If a structure element type is in both the inclusion and
    --  exclusion sets, it will be excluded. The search terminates if a
    --  match is found or if the limits of the structure are reached.
    --  The search progresses from the start point and proceeds either
    --  forward in the structure or backward, depending on direction .
    --  This is a non-descending search (i.e., the search does not
    --  include any structures referenced by "execute structure"
    --  elements)If the search succeeds in finding a match, a status of
    --  Found is returned, and the offset of the matching element is
    --  returned in found_offset . If the search is unsuccessful, a
    --  status of NotFound is returned, and a value of zero is returned
    --  in found_offset . The element pointer attribute of s_id is not
    --  modified as a result of this request.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id   : The structure.
    --  Start          : Offset to start search.
    --  Direction      : Search direction.
    --  Inclusion_Set  : Kinds of elements to include.
    --  Exclusion_Set  : Kinds of elements to exclude.
    --  Search_Status  : Status of search returned.
    --  Found_Position : Offset found.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Element_Search ( Display_Id     : in Display;
                             Structure_Id   : in Structure;
                             Start          : in Element_Location;
                             Direction      : in Search_Direction;
                             Inclusion_Set  : in Element_Type_Set;
                             Exclusion_Set  : in Element_Type_Set;
                             Search_Status  : out Search_Status_Indicator;
                             Found_Position : out Element_Position);
--
--  Section 7.4 -- Structure Editing
--
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Store_Elements
    --
    --  <Description>
    --
    --  This request causes elements to be stored in the structure whose
    --  resource ID is specified by s_id . If the editing_mode attribute
    --  of the structure is set to StructureInsert , each entry in
    --  elements is used to create a structure element in the structure
    --  after the element specified by the structure's element_ptr
    --  attribute. Output commands in the request are processed until
    --  one is found to be in error, or until the entire list has been
    --  processed. The erroneous output command and all others following
    --  it in the request are ignored (they will not be added to the
    --  structure)and an OutputCommand error is returned to the client.
    --  The element pointer is incremented by one for each such
    --  insertion. Therefore, at the conclusion of the request, the
    --  element pointer is left pointing at the last inserted element.
    --  If the editing_mode attribute is set to StructureReplace , a
    --  range of elements is first deleted from the structure as if
    --  PEXDeleteElements were called with a position1 whence/offset of
    --  { Current , 0} and a position2 whence/offset of { Current , n -
    --  1} where n is the number of elements to be deleted. (If element
    --  zero is included as part of the range to be deleted, all
    --  elements in the range except for element zero will be
    --  deleted.)If there are fewer elements between the structure's
    --  element pointer and the end of the structure than there are
    --  output commands in elements , then only the elements between the
    --  element pointer and the end of the structure will be deleted.
    --  The element pointer will be left pointing at the element
    --  preceding the deleted range. After the deletion, the output
    --  commands specified in elements are inserted at the element
    --  pointer position as described above. Floating point values in
    --  elements will be in the floating point format specified in
    --  fp_format . The formats for the various types of output commands
    --  can be found in the "Output Commands" section.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id    : The structure.
    --  Output_Commands : The output commands to store.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Store_Elements ( Display_Id      : in Display;
                             Structure_Id    : in Structure;
                             Output_Commands : in Output_Command_Buffer);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Delete_Elements
    --
    --  <Description>
    --
    --  This request deletes elements in range from the structure whose
    --  ID is specified in s_id . The range of elements to be deleted is
    --  computed using the whence and offset values of position1 and
    --  position2 . The offsets that define the range of elements to be
    --  deleted are each computed using the positions in range in the
    --  following fashion. If whence is Beginning , the computed offset
    --  is just the value specified by offset . If whence is Current ,
    --  the offset is computed by adding offset to s_id 's element
    --  pointer. If whence is End , the offset is computed by adding
    --  offset to the number of elements in the structure. Offset can be
    --  a negative number. If either computed offset is less than zero,
    --  it will be set to zero before the deletion occurs. If either
    --  computed offset is greater than the number of elements in the
    --  structure, it will be set to the offset of the last structure
    --  element. It is not necessary for the first offset to be less
    --  than the second. The deletion is inclusive, meaning that the
    --  elements at the boundary of the deletion range are also deleted.
    --  It does not matter which of the two offsets determines the lower
    --  end of the range to be deleted and which determines the higher
    --  end of the range. Deleting element zero is effectively a noop.
    --  After the deletion operation, the structure's element pointer
    --  will be set to the element preceding the range of deleted
    --  elements.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Between      : Element offset range to delete.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Delete_Elements ( Display_Id   : in Display;
                              Structure_Id : in Structure;
                              Between      : in Element_Range);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Delete_Elements_To_Label
    --
    --  <Description>
    --
    --  This request deletes all elements between a computed offset and a
    --  specified label in the structure whose ID is specified in s_id .
    --  The offset is computed using position in the following fashion.
    --  If whence is Beginning , the computed offset is just the value
    --  specified by offset . If whence is Current , the offset is
    --  computed by adding offset to s_id 's element pointer. If whence
    --  is End , the offset is computed by adding offset to the number
    --  of elements in the structure. Offset can be a negative number.
    --  If the computed offset is less than zero, it will be set to zero
    --  before the deletion occurs. If the computed offset is greater
    --  than the number of elements in the structure, it will be set to
    --  the offset of the last structure element. Elements are deleted
    --  starting at the element immediately after the computed offset up
    --  to the next occurrence of the label . The label itself is not
    --  deleted. The structure's element pointer is set to point at the
    --  element immediately preceding the range of deleted elements. If
    --  the specified label is not found, no deletion occurs and the
    --  structure's element pointer is left unchanged.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  From         : Element offset origin for deletion.
    --  Label        : Label extemity for deletion.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Delete_Elements_To_Label ( Display_Id   : in Display;
                                       Structure_Id : in Structure;
                                       From         : in Element_Location;
                                       Label        : in Label_Id);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Delete_Elements_Between_Labels
    --
    --  <Description>
    --
    --  This request deletes all elements between label1 and label2 in
    --  the structure whose ID is specified in s_id . A search for
    --  label1 is first performed starting at the element whose offset
    --  is specified by the structure's element pointer. A search for
    --  label2 is then performed, starting at the element after label1 .
    --  The range of elements between the two labels is then deleted.
    --  The two label elements themselves are not deleted. The
    --  structure's element pointer is set to point at the element
    --  immediately preceding the range of deleted elements (the element
    --  containing label1 )If either of the two specified labels is not
    --  found between starting point of the search and the end of the
    --  structure, no deletion occurs and the structure's element
    --  pointer is left unchanged.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Structure_Id : The structure.
    --  Label_Id1    : Origin of deletion.
    --  Label_Id2    : End of deletion.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Delete_Elements_Between_Labels( Display_Id   : in Display;
                                            Structure_Id : in Structure;
                                            Label_Id1    : in Label_Id;
                                            Label_Id2    : in Label_Id);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Copy_Elements
    --
    --  <Description>
    --
    --  This request copies elements in src_range from the structure
    --  specified in src_s_id to the dest_position in the structure
    --  specified in dest_s_id . The range of elements to be copied is
    --  computed using the whence and offset values of position1 and
    --  position2 in src_range . The offsets that define the range of
    --  elements to be copied are each computed using the positions in
    --  src_range in the following fashion. If whence is Beginning , the
    --  computed offset is just the value specified by offset . If
    --  whence is Current , the offset is computed by adding offset to
    --  src_s_id 's element pointer. If whence is End , the offset is
    --  computed by adding offset to the number of elements in the
    --  structure. Offset can be a negative number. If either computed
    --  offset is less than zero, it will be set to zero before the copy
    --  occurs. If either computed offset is greater than the number of
    --  elements in the source structure, it will be set to the offset
    --  of the last structure element in the source structure. It does
    --  not matter which of the two offsets determines the lower end
    --  of the range to be copied and which determines the higher end of
    --  the range (i.e., the element order will not be reversed if
    --  offset1 is greater than offset2 )The position to which the
    --  elements will be copied is computed using dest_position in the
    --  following fashion. If whence is Beginning , the computed
    --  destination offset is just the value specified by offset . If
    --  whence is Current , the offset is computed by adding offset to
    --  dest_s_id 's element pointer. If whence is End , the offset is
    --  computed by adding offset to the number of elements in the
    --  destination structure. Offset can be a negative number. * It is
    --  permissible for src_s_id and dest_s_id to be the same. If this
    --  is the case, the copy operation performs as if the indicated
    --  range is copied to a temporary location, and then inserted
    --  relative to the destination position. After the copy operation,
    --  the destination structure's element pointer is updated to point
    --  at the last element inserted in the destination structure. The
    --  editing mode attribute of dest_s_id is ignored during this
    --  request; copied elements are always inserted into the
    --  destination structure, never used to replace existing structure
    --  elements.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Src_Structure : The structure whose content will be copied.
    --  Between       : Element offset range to copy.
    --  Dst_Structure : The destination structure.
    --  Dst_Position  : Offset location for inserting copy.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Copy_Elements ( Display_Id    : in Display;
                            Src_Structure : in Structure;
                            Between       : in Element_Range;
                            Dst_Structure : in Structure;
                            Dst_Position  : in Element_Location);
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Change_Structure_References
    --
    --  <Description>
    --
    --  This request changes any "execute structure" elements in the
    --  server that reference the structure specified by old_s_id into
    --  "execute structure" elements that reference the structure
    --  specified by new_s_id . Any references to the structure new_s_id
    --  that existed before this request are not affected. On all PHIGS
    --  workstation resources where new_s_id is already posted, it
    --  remains posted and old_s_id , if posted there, is unposted. On
    --  all PHIGS workstation resources where new_s_id is not already
    --  posted and old_s_id is posted, new_s_id is posted with the same
    --  priority as old_s_id and old_s_id is unposted. If there were
    --  references to the structure resource specified by old_s_id and
    --  the structure resource specified by new_s_id does not exist, an
    --  error is returned and no action is taken.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Old_Structure : The ancient structure id to reference.
    --  New_Structure : The new structure id to reference.
    --
    --  <Errors Raised>
    --
    --=====================================================================

  procedure Change_Structure_References ( Display_Id    : in Display;
                                          Old_Structure : in Structure;
                                          New_Structure : in Structure);
end Pex_Structures ;
