---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
-- Copyright 1992 J.C. Mahieux and B. 
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--
with System ;
with X_Types ;
with Pex_Constants ;
with Pex_Coordinate_System ;
package Pex_Types is
    use X_Types ;
    use Pex_Constants ;
    
    Pex_Closed   : exception; -- PEX server not existing or not yet connected
    
    Buffer_Full         : exception ;
    --
    type Index_Count is range 0 .. 255 ;
    --
    --
    -- Modeling Coordinate System
    --

    type MC_Type is digits Pex_Precision;

    package MC is new Pex_Coordinate_System (MC_Type);

    type Transformation_Matrix is array (1 .. 4, 1..4) of Mc_Type ;
    type Transformation_Matrix_2 is array ( 1..3, 1..3) of MC_Type;

    --
    -- World Coordinate System
    --

    type WC_Type is digits Pex_Precision;

    package WC is new Pex_Coordinate_System (WC_Type);

    --
    -- View Reference Coordinate Coordinate System
    --

    type VRC_Type is digits Pex_Precision;

    package VRC is new Pex_Coordinate_System (VRC_Type);

    --
    -- Normalized Projection Coordinate System
    --

    type NPC_Type is digits Pex_Precision;

    package NPC is new Pex_Coordinate_System (NPC_Type);

    --
    -- Device Coordinate System
    --

    subtype DC_Type is Pixels;
    type Z_DC_Type is digits Pex_Precision;
    
    type Dc_Distance is digits Pex_Precision;

    package DC is
        
        type Point_3 is
        record
            X : DC_Type;
            Y : DC_Type;
            Z : Z_DC_Type;
        end record;

        for Point_3 use
        record
            X   at 0 * X_32_Bits range  0 .. 15;
            Y   at 0 * X_32_Bits range 16 .. 31;
            Z   at 1 * X_32_Bits range  0 .. Fp_End;
        end record;

        type Point_2 is
        record
            X : DC_Type;
            Y : DC_Type;
        end record;

        for Point_2 use
        record
            X   at 0 * X_32_Bits range  0 .. 15;
            Y   at 0 * X_32_Bits range 16 .. 31;
        end record;

        type Point_List_3 is array (Positive range <>) of Point_3;

        type Point_List_2 is array (Positive range <>) of Point_2;

        type Access_Point_List_3 is access Point_List_3;

        type Access_Point_List_2 is access Point_List_2;

        type List_Of_Point_List_3 is array (Positive range <>) of
            Access_Point_List_3;

        type List_Of_Point_List_2 is array (Positive range <>) of
            Access_Point_List_2;

        type Access_List_Of_Point_List_3 is access List_Of_Point_List_3;

        type Access_List_Of_Point_List_2 is access List_Of_Point_List_2;

        type Vector_3 is new Point_3;

        type Vector_Pair_3 is array (1..2) of Vector_3;

        type Vector_2 is new Point_2;

        type Vector_Pair_2 is array (1..2) of Vector_2;

        type Rectangular_Region_3 is 
        record
            Xmin : DC_Type;
            Xmax : DC_Type;
            Ymin : DC_Type;
            Ymax : DC_Type;
            Zmin : Z_DC_Type;
            Zmax : Z_DC_Type;
        end record;

        for Rectangular_Region_3 use
        record
            Xmin   at 0 * X_32_Bits range  0 .. 15;
            Ymin   at 0 * X_32_Bits range 16 .. 31;
            Zmin   at 1 * X_32_Bits range  0 .. Fp_End;
            Xmax   at (1+Fp) * X_32_Bits range  0 .. 15;
            Ymax   at (1+Fp) * X_32_Bits range 16 .. 31;
            Zmax   at (2+Fp) * X_32_Bits range  0 .. Fp_End;
        end record;

        type Rectangular_Region_2 is 
        record
            Xmin : DC_Type;
            Xmax : DC_Type;
            Ymin : DC_Type;
            Ymax : DC_Type;
        end record;

        for Rectangular_Region_2 use
        record
            Xmin   at 0 * X_32_Bits range  0 .. 15;
            Ymin   at 0 * X_32_Bits range 16 .. 31;
            Xmax   at 1 * X_32_Bits range  0 .. 15;
            Ymax   at 1 * X_32_Bits range 16 .. 31;
        end record;
        
        type Rectangular_Region_2_Array is array (X_Natural range <>) of
            Rectangular_Region_2 ;
        
        type Half_Space_3 is 
        record
            Reference_Position : Point_3;
            Acceptance_Normal  : Vector_3;
        end record;

        type Half_Space_2 is 
        record
            Reference_Position : Point_2;
            Acceptance_Normal  : Vector_2;
        end record;

        type Half_Space_List_3 is array (Positive range <>) of Half_Space_3;

        type Half_Space_List_2 is array (Positive range <>) of Half_Space_2;

        type Access_Half_Space_List_3 is access Half_Space_List_3;

        type Access_Half_Space_List_2 is access Half_Space_List_2;

    end DC;
    --
    type Bytes is access Byte_Array ;
    type Data_Record is
    record
        Size : X_Natural ;
        Data : Bytes ;
    end record ;
    
    type Longs is access Long_Array ;
    type Application_Data_Record is
    record
        Size : X_Natural ;
        Data : Longs ;
    end record ;
    
    type Gse_Data_Record is new Application_Data_Record ;
    --
    type Name is range 0 .. 2 ** 31 - 1 ;

    type Name_Array is array (X_Natural range <>) of Name ;
    type Name_List (Size : X_Natural := 0) is
    record
        Names : Name_Array (1 .. Size) ;
    end record ;
    
    --
    -- X Resource entities
    --
    type Font is new Resource ;
    type Name_Set is new Resource ;
    type Phigs_Workstation is new Resource ;
    type Pick_Measure is new Resource;
    type Pipeline_Context is new Resource ;
    type Renderer is new Resource ;
    type Search_Context is new Resource;
    type Structure is new Resource ;
    
    type User_Structure is new X_Long_Integer  ;
    
    
    -- Pex Requests
    type Request_Type is
        ( Get_Extension_Info,            Get_Enumerated_Type_Info,
          Get_Imp_Dep_Constants,         Create_Lookup_Table,
          Copy_Lookup_Table,             Free_Lookup_Table,
          Get_Table_Info,                Get_Predefined_Entries,
          Get_Defined_Indices,           Get_Table_Entry,
          Get_Table_Entries,             Set_Table_Entries,
          Delete_Table_Entries,          Create_Pipeline_Context,
          Copy_Pipeline_Context,         Free_Pipeline_Context,
          Get_Pipeline_Context,          Change_Pipeline_Context,
          Create_Renderer,               Free_Renderer,
          Change_Renderer,               Get_Renderer_Attributes,
          Get_Renderer_Dynamics,         Begin_Rendering,
          End_Rendering,                 Begin_Structure,
          End_Structure,                 Render_Output_Commands,
          Render_Network,                Create_Structure,
          Copy_Structure,                Destroy_Structures,
          Get_Structure_Info,            Get_Element_Info,
          Get_Structures_In_Network,     Get_Ancestors,
          Get_Descendants,               Fetch_Elements,
          Set_Editing_Mode,              Set_Element_Pointer,
          Set_Element_Pointer_At_Label,  Element_Search,
          Store_Elements,                Delete_Elements,
          Delete_Elements_To_Label,      Delete_Elements_Between_Labels,
          Copy_Elements,                 Change_Structure_References,
          Create_Name_Set,               Copy_Name_Set,
          Free_Name_Set,                 Get_Name_Set,
          Change_Name_Set,               Create_Search_Context,
          Copy_Search_Context,           Free_Search_Context,
          Get_Search_Context,            Change_Search_Context,
          Search_Network,                Create_Phigs_Wks,
          Free_Phigs_Wks,                Get_Wks_Info,
          Get_Dynamics,                  Get_View_Rep,
          Redraw_All_Structures,         Update_Workstation,
          Redraw_Clip_Region,            Execute_Deferred_Actions,
          Set_View_Priority,             Set_Display_Update_Mode,
          Map_Dc_To_Wc,                  Map_Wc_To_Dc,
          Set_View_Rep,                  Set_Wks_Window,
          Set_Wks_Viewport,              Set_Hlhsr_Mode,
          Set_Wks_Buffer_Mode,           Post_Structure,
          Unpost_Structure,              Unpost_All_Structures,
          Get_Wks_Postings,              Get_Pick_Device,
          Change_Pick_Device,            Create_Pick_Measure,
          Free_Pick_Measure,             Get_Pick_Measure,
          Update_Pick_Measure,           Open_Font,
          Close_Font,                    Query_Font,
          List_Fonts,                    List_Fonts_With_Info,
          Query_Text_Extents,            Match_Renderer_Targets,
          Escape,                        Escape_With_Reply,
          Render_Elements,               Accumulate_State,
          Begin_Pick_One,                End_Pick_One,
          Pick_One,                      Begin_Pick_All,
          End_Pick_All,                  Pick_All ) ;
    
    for Request_Type use
      (    1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
          14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
          27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
          40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
          53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
          66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
          79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
          92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104);
    
    -- Types for implementation information
    --
    type Implementation_Specific_Value is range -2**31 .. -1 ;
    
    type Implementation_Specific_Array is array (X_Natural range <>)
        of Implementation_Specific_Value ;
    
    type Implementation_Specific_List (Size : X_Natural := 0) is
    record
        Values : Implementation_Specific_Array (1 .. Size) ;
    end record ;
    
    type Implementation_Level is
        ( Immediate_Rendering_Subset, Phigs_Workstation_Subset,
          Structure_Rendering_Subset, Vendor_Specific_Subset,
          Full_Pex_Implementation ) ;
    
    subtype Subset_Info_Mask is Boolean_Array (0 .. 31) ;
    
    Immediate_Rendering_Subset_Bit : constant := 0 ;
    Phigs_Workstation_Subset_Bit   : constant := 1 ;
    Structure_Rendering_Subset_Bit : constant := 2 ;
    
    type Enumerated_Value is private ;
    type Enumerated_Array is array (X_Natural range <>) of Enumerated_Value ;
    type Enumerated_Type is access Enumerated_Array ;
    type Enumerated_Type_Array is array (X_Natural range <>)
        of Enumerated_Type ;
    type Enumerated_Type_List (Size : X_Natural := 0) is
    record
        Types : Enumerated_Type_Array (1 .. Size) ;
    end record ;
    
    type String_Pointer is access String ;
    type String_Array is array (X_Natural range <>) of String_Pointer ;
    type String_List is access String_Array ;
    type String_List_Array is array (X_Natural range <>) of String_List ;
    type Set_Of_String_Lists (Size : X_Natural := 0) is
    record
        Lists : String_List_Array (1 .. Size) ;
    end record ;
    
    type Enumerated_Definition is
    record
        Value    : Enumerated_Value ;
        Mnemonic : String_Pointer ;
    end record ;
    type Enumerated_Definition_Array is array (X_Natural range <>)
        of Enumerated_Definition ;
    type Enumerated_Type_Definition is access Enumerated_Definition_Array ;
    type Enumerated_Type_Definition_Array is array (X_Natural range <>)
        of Enumerated_Type_Definition ;
    type Enumerated_Type_Definition_List (Size : X_Natural := 0) is
    record
        Types : Enumerated_Type_Definition_Array (1 .. Size) ;
    end record ;
    
    type Enumerated_Types is private ;
    type Enumerated_Types_Array is array (X_Natural range <>)
        of Enumerated_Types ;
    Enum_Marker_Type                   : constant Enumerated_Types ;
    Enum_A_Text_Style                  : constant Enumerated_Types ;
    Enum_Interior_Style                : constant Enumerated_Types ;
    Enum_Hatch_Style                   : constant Enumerated_Types ;
    Enum_Line_Type                     : constant Enumerated_Types ;
    Enum_Surface_Edge_Type             : constant Enumerated_Types ;
    Enum_Pick_Device_Type              : constant Enumerated_Types ;
    Enum_Polyline_Interpolation_Method : constant Enumerated_Types ;
    Enum_Curve_Approx_Method           : constant Enumerated_Types ;
    Enum_Reflection_Model              : constant Enumerated_Types ;
    Enum_Surface_Interpolation_Method  : constant Enumerated_Types ;
    Enum_Surface_Approx_Method         : constant Enumerated_Types ;
    Enum_Model_Clip_Operator           : constant Enumerated_Types ;
    Enum_Light_Type                    : constant Enumerated_Types ;
    Enum_Color_Type                    : constant Enumerated_Types ;
    Enum_Float_Format                  : constant Enumerated_Types ;
    Enum_Hlhsr_Mode                    : constant Enumerated_Types ;
    Enum_Prompt_Echo_Type              : constant Enumerated_Types ;
    Enum_Display_Update_Mode           : constant Enumerated_Types ;
    Enum_Color_Approx_Type             : constant Enumerated_Types ;
    Enum_Color_Approx_Model            : constant Enumerated_Types ;
    Enum_GDP                           : constant Enumerated_Types ;
    Enum_GDP3                          : constant Enumerated_Types ;
    Enum_GSE                           : constant Enumerated_Types ;
    Enum_Trim_Curve_Approx_Method      : constant Enumerated_Types ;
    Enum_Rendering_Color_Model         : constant Enumerated_Types ;
    Enum_Para_Surf_Characteristics     : constant Enumerated_Types ;
    Enum_Escape                        : constant Enumerated_Types ;
    Enum_Pick_One_Method               : constant Enumerated_Types ;
    Enum_Pick_All_Method               : constant Enumerated_Types ;
    
    
    -- Pseudo enumerated types
    type Marker_Type is private ;
    Marker_Dot              : constant Marker_Type ;
    Marker_Cross            : constant Marker_Type ;
    Marker_Asterisk         : constant Marker_Type ;
    Marker_Circle           : constant Marker_Type ;
    Marker_X                : constant Marker_Type ;
    
    type Annotation_Style is private ;
    Annotation_Not_Connected    : constant Annotation_Style ;
    Annotation_Connected        : constant Annotation_Style ;
    
    type Interior_Style is ( Hollow, Solid, Pattern, Hatch, Empty ) ;
    for Interior_Style use ( Hollow => 1, Solid => 2, Pattern => 3,
                             Hatch  => 4, Empty => 5 ) ;
    
    type Hatch_Style is private ;
    
    type Line_Type is private ;
    Line_Type_Solid         : constant Line_Type ;
    Line_Type_Dashed        : constant Line_Type ;
    Line_Type_Dotted        : constant Line_Type ;
    Line_Type_Dash_Dot      : constant Line_Type ;
    
    type Surface_Edge_Type is private ;
    Surface_Edge_Solid      : constant Surface_Edge_Type ;
    Surface_Edge_Dashed     : constant Surface_Edge_Type ;
    Surface_Edge_Dotted     : constant Surface_Edge_Type ;
    Surface_Edge_Dash_Dot   : constant Surface_Edge_Type ;
    
    type Pick_Device_Type is private ;
    Pick_Device_DC_Hit_Box      : constant Pick_Device_Type ;
    Pick_Device_NPC_Hit_Volume  : constant Pick_Device_Type ;
    
    type Polyline_Interpolation_Method is private ;
    Polyline_Interpolation_None    : constant Polyline_Interpolation_Method ;
    Polyline_Interpolation_Color   : constant Polyline_Interpolation_Method ;
    
    type Approximation_Method is private ;
    Approx_Imp_Dep                    : constant Approximation_Method ;
    Approx_Constant_Between_Knots     : constant Approximation_Method ;
    Approx_Wcs_Chordal_Size           : constant Approximation_Method ;
    Approx_Npc_Chordal_Size           : constant Approximation_Method ;
    Approx_Dc_Chordal_Size            : constant Approximation_Method ;
    Curve_Approx_Wcs_Chordal_Dev      : constant Approximation_Method ;
    Curve_Approx_Npc_Chordal_Dev      : constant Approximation_Method ;
    Curve_Approx_Dc_Chordal_Dev       : constant Approximation_Method ;
    Surface_Approx_Wcs_Planar_Dev     : constant Approximation_Method ;
    Surface_Approx_Npc_Planar_Dev     : constant Approximation_Method ;
    Surface_Approx_Dc_Planar_Dev      : constant Approximation_Method ;
    Approx_Wcs_Relative               : constant Approximation_Method ;
    Approx_Npc_Relative               : constant Approximation_Method ;
    Approx_Dc_Relative                : constant Approximation_Method ;
    
    subtype Curve_Approximation_Method is Approximation_Method ;
    subtype Surface_Approximation_Method is Approximation_Method ;
    
    type Trim_Curve_Approx_Method is private ;
    Trim_Approx_Imp_Dep                : constant Trim_Curve_Approx_Method ;
    Trim_Approx_Constant_Between_Knots : constant Trim_Curve_Approx_Method ;
    
    type Reflection_Model is private ;
    Reflection_No_Shading   : constant Reflection_Model ;
    Reflection_Ambient      : constant Reflection_Model ;
    Reflection_Diffuse      : constant Reflection_Model ;
    Reflection_Specular     : constant Reflection_Model ;
    
    type Surface_Interpolation_Method is private ;
    Surface_Interpolation_None        : constant Surface_Interpolation_Method;
    Surface_Interpolation_Color       : constant Surface_Interpolation_Method;
    Surface_Interpolation_Dot_Product : constant Surface_Interpolation_Method;
    Surface_Interpolation_Normal      : constant Surface_Interpolation_Method;
    
    type Model_Clip_Operator is private ;
    Model_Clip_Replace              : constant Model_Clip_Operator ;
    Model_Clip_Intersection         : constant Model_Clip_Operator ;
    
    type Light_Type is private ;
    Light_Ambient                   : constant Light_Type ;
    Light_Wcs_Vector                : constant Light_Type ;
    Light_Wcs_Point                 : constant Light_Type ;
    Light_Wcs_Spot                  : constant Light_Type ;
    
    type Color_Type is private ;
    Indexed_Color       : constant Color_Type ;
    Rgb_Float_Color     : constant Color_Type ;
    Cie_Float_Color     : constant Color_Type ;
    Hsv_Float_Color     : constant Color_Type ;
    Hls_Float_Color     : constant Color_Type ;
    Rgb_8_Color         : constant Color_Type ;
    Rgb_16_Color        : constant Color_Type ;
    
    type Floating_Point_Format is private ;
    Ieee_754_32             : constant Floating_Point_Format ;
    Dec_F_Floating          : constant Floating_Point_Format ;
    Ieee_754_64             : constant Floating_Point_Format ;
    Dec_D_Floating          : constant Floating_Point_Format ;
    
    type Hlhsr_Mode is private ;
    Hlhsr_Off               : constant Hlhsr_Mode ;
    Hlhsr_Z_Buffer          : constant Hlhsr_Mode ;
    Hlhsr_Painters          : constant Hlhsr_Mode ;
    Hlhsr_Scanline          : constant Hlhsr_Mode ;
    Hlhsr_Hidden_Line_Only  : constant Hlhsr_Mode ;
    Hlhsr_Z_Buffer_Id       : constant Hlhsr_Mode ;
    
    type Prompt_Echo_Type is private ;
    Echo_Primitive  : constant Prompt_Echo_Type ;
    Echo_Structure  : constant Prompt_Echo_Type ;
    Echo_Network    : constant Prompt_Echo_Type ;
    
    type Display_Update_Method is private ;
    Visualize_Each          : constant Display_Update_Method ;
    Visualize_Easy          : constant Display_Update_Method ;
    Visualize_None          : constant Display_Update_Method ;
    Simulate_Some           : constant Display_Update_Method ;
    Visualize_Whenever      : constant Display_Update_Method ;
    
    type Color_Approximation_Type is private ;
    Color_Space          : constant Color_Approximation_Type ;
    Color_Range          : constant Color_Approximation_Type ;
    
    type Color_Approximation_Method is private ;
    Color_Approx_RGB      : constant Color_Approximation_Method ;
    Color_Approx_CIE      : constant Color_Approximation_Method ;
    Color_Approx_HSV      : constant Color_Approximation_Method ;
    Color_Approx_HLS      : constant Color_Approximation_Method ;
    Color_Approx_YIQ      : constant Color_Approximation_Method ;
    
    type Rendering_Color_Model is private ;
    Rdr_Color_Model_Imp_Dep : constant Rendering_Color_Model ;
    Rdr_Color_Model_RGB    : constant Rendering_Color_Model ;
    Rdr_Color_Model_CIE    : constant Rendering_Color_Model ;
    Rdr_Color_Model_HSV    : constant Rendering_Color_Model ;
    Rdr_Color_Model_HLS    : constant Rendering_Color_Model ;
    
    type Parametric_Surface_Characteristics is private ;
    PSC_None            : constant Parametric_Surface_Characteristics ;
    PSC_Imp_Dep         : constant Parametric_Surface_Characteristics ;
    PSC_Iso_Curves      : constant Parametric_Surface_Characteristics ;
    PSC_Mc_Level_Curves : constant Parametric_Surface_Characteristics ;
    PSC_Wc_Level_Curves : constant Parametric_Surface_Characteristics ;
    
    type Isoparametric_Curves is private ;
    Uniform_Placement            : constant Isoparametric_Curves ;
    Non_Uniform_Placement        : constant Isoparametric_Curves ;
    
    type Escape_Request is private ;
    Set_Echo_Color      : constant Escape_Request ;
    
    type Pick_One_Method is private ;
    Last_Picked      : constant Pick_One_Method ;
    Closest_Z        : constant Pick_One_Method ;
    Any_Visible      : constant Pick_One_Method ;
    Visible_Closest  : constant Pick_One_Method ;
    
    type Pick_All_Method is private ;
    Retain_All      : constant Pick_All_Method ;
    Retain_Visible  : constant Pick_All_Method ;
    
    --
    -- Types for output commands
    type Line_Width is digits Pex_Precision;
    type Marker_Size is digits Pex_Precision;
    type Pattern_Size is new Mc.Point_2 ;
    type Char_Expansion is digits Pex_Precision;
    type Char_Spacing is digits Pex_Precision;
    type Edge_Width is digits Pex_Precision;
    type Reflection_Coefficient is digits Pex_Precision;
    type Lookup_Table_Index is range 0 .. 65_535;
    for Lookup_Table_Index'size use 16;
    --
    subtype Line_Bundle_Index is Lookup_Table_Index ;
    subtype Marker_Bundle_Index is Lookup_Table_Index ;
    subtype Text_Bundle_Index is Lookup_Table_Index ;
    subtype Interior_Bundle_Index is Lookup_Table_Index ;
    subtype Edge_Bundle_Index is Lookup_Table_Index ;
    subtype Pattern_Index is Lookup_Table_Index ;
    subtype Text_Font_Index is Lookup_Table_Index ;
    subtype Color_Index is Lookup_Table_Index ;
    subtype View_Index is Lookup_Table_Index ;
    subtype Light_Index is Lookup_Table_Index ;
    subtype Depth_Cue_Index is Lookup_Table_Index ;
    subtype Color_Approximation_Index is Lookup_Table_Index ;
    
    type Surface_Interior ( Style : Interior_Style := Empty ) is
    record
        case Style is
        when Pattern => Index : Pattern_Index ;
        when Hatch   => Tint  : Hatch_Style ;
        when others  => null ;
        end case ;
    end record ;
    
    type Predefined_Color_Type is ( Indexed, Rgb_Float, Cie_Float, Hsv_Float,
                                    Hls_Float, Rgb_8, Rgb_16, Specific) ;
    
    type Rgb_Float_Value is digits Pex_Precision ;
    type Color_Float_Value is digits Pex_Precision ;
    type Hue_Float_Value is digits Pex_Precision ;
    type Saturation_Float_Value is digits Pex_Precision ;
    type Lightness_Float_Value is digits Pex_Precision ;
    type Rgb_Byte_Value is range 0 .. 255 ;
    for Rgb_Byte_Value'size use 8 ;
    type Rgb_Word_Value is range 0 .. 65_535 ;
    for Rgb_Word_Value'size use 16 ;
    
    type Color_Limit is range 0 .. 65_535 ;
    type Color_Multiplier is new X_Long_Integer ;
    Type Color_Weight is digits Pex_Precision ;
    
    type Rgb_Float_Representation is
    record
        Red    : Rgb_Float_Value ;
        Green  : Rgb_Float_Value ;
        Blue   : Rgb_Float_Value ;
    end record ;
    
    type Rgb_Float_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Rgb_Float_Representation ;
    
    type Cie_Float_Representation is
    record
        X  : Color_Float_Value ;
        Y  : Color_Float_Value ;
        Z  : Color_Float_Value ;
    end record ;
    
    type Cie_Float_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Cie_Float_Representation ;
    
    type Hsv_Float_Representation is
    record
        Hue         : Hue_Float_Value ;
        Saturation  : Saturation_Float_Value ;
        Value       : Color_Float_Value ;
    end record ;
    
    type Hsv_Float_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Hsv_Float_Representation ;
    
    type Hls_Float_Representation is
    record
        Hue         : Hue_Float_Value ;
        Lightness   : Lightness_Float_Value ;
        Saturation  : Saturation_Float_Value ;
    end record ;
    
    type Hls_Float_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Hls_Float_Representation ;
    
    type Rgb_Byte_Representation is
    record
        Red    : Rgb_Byte_Value ;
        Green  : Rgb_Byte_Value ;
        Blue   : Rgb_Byte_Value ;
    end record ;
    
    for Rgb_Byte_Representation use
    record at mod X_32_Bits ;
        Red    at 0 * X_32_Bits range  0 ..  7 ;
        Green  at 0 * X_32_Bits range  8 .. 15 ;
        Blue   at 0 * X_32_Bits range 16 .. 23 ;
    end record ;
    
    type Rgb_Byte_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Rgb_Byte_Representation ;
    
    type Rgb_Word_Representation is
    record
        Red    : Rgb_Word_Value ;
        Green  : Rgb_Word_Value ;
        Blue   : Rgb_Word_Value ;
    end record ;
    
    for Rgb_Word_Representation use
    record at mod X_32_Bits ;
        Red    at 0 * X_32_Bits range  0 .. 15 ;
        Green  at 0 * X_32_Bits range 16 .. 31 ;
        Blue   at 1 * X_32_Bits range  0 .. 15 ;
    end record ;
    
    type Rgb_Word_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Rgb_Word_Representation ;
    
    type Specific_Representation is
    record
        A  : Color_Float_Value ;
        B  : Color_Float_Value ;
        C  : Color_Float_Value ;
    end record ;
    
    type Specific_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Specific_Representation ;
    
    type Color_Representation (Kind : Predefined_Color_Type := Indexed) is
    record
        case Kind is
        when Indexed   => Index  : Color_Index ;
        when Rgb_Float => Rgb    : Rgb_Float_Representation ;
        when Cie_Float => Cie    : Cie_Float_Representation ;
        when Hsv_Float => Hsv    : Hsv_Float_Representation ;
        when Hls_Float => Hls    : Hls_Float_Representation ;
        when Rgb_8     => Rgb_8  : Rgb_Byte_Representation ;
        when Rgb_16    => Rgb_16 : Rgb_Word_Representation ;
        when Specific =>
            The_Type : Color_Type ;
            Specific : Specific_Representation ;
        end case ;
    end record ;
    
    type Color_Index_Matrix is array
        ( X_Natural range <>, X_Natural range <> ) of Color_Index ;
    pragma Pack (Color_Index_Matrix) ;
    
    type Color_Matrix is array
        ( X_Natural range <>, X_Natural range <> ) of Color_Representation ;
    
    type Reflection_Attributes is
    record
        Ambient                : Reflection_Coefficient;
        Diffuse                : Reflection_Coefficient;
        Specular               : Reflection_Coefficient;
        Specular_Concentration : Reflection_Coefficient;
        Transmission           : Reflection_Coefficient;
        Specular_Color         : Color_Representation;
    end record;
    
    type Attenuation_Factor is digits Pex_Precision ;
    type Angle is digits Pex_Precision ;
    
    type Attenuation_Type is
    record
        Factor_1 : Attenuation_Factor ;
        Factor_2 : Attenuation_Factor ;
    end record ;
    
    type Curve_Approximation is
    record
        Method    : Curve_Approximation_Method ;
        Tolerance : Mc_Type ;
    end record ;
    
    type Surface_Approximation is
    record
        Method      : Surface_Approximation_Method ;
        U_Tolerance : Mc_Type ;
        V_Tolerance : Mc_Type ;
    end record ;
    
    type Nurb_Order is range 1 .. 10 ;
    type Trim_Order is range 2 .. 10 ;
    --
    --
    type Label_Id is new X_Long_Integer ;
    type Pick_Id is new X_Long_Integer;
    --
    --
    type Gse_Id is new X_Long_Integer;
    --
    --
    type Hlhsr_Id is new X_Long_Integer;

    type Gdp_3_Id is new X_Long_Integer;
    type Gdp_Id is new X_Long_Integer;


    subtype Aspect_Mask is Boolean_Array (0 .. 29) ;
    
    Marker_Type_Asf_Bit : constant := 0 ;
    Marker_Type_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Marker_Type_Asf_Bit => True, others => False ) ;
    
    Marker_Scale_Asf_Bit : constant := 1 ;
    Marker_Scale_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Marker_Scale_Asf_Bit => True, others => False ) ;
    
    Marker_Colour_Asf_Bit : constant := 2 ;
    Marker_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Marker_Colour_Asf_Bit => True, others => False ) ;
    
    Text_Font_Index_Asf_Bit : constant := 3 ;
    Text_Font_Index_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Text_Font_Index_Asf_Bit => True, others => False ) ;
    
    Text_Prec_Asf_Bit : constant := 4 ;
    Text_Prec_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Text_Prec_Asf_Bit => True, others => False ) ;
    
    Char_Expansion_Asf_Bit : constant := 5 ;
    Char_Expansion_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Char_Expansion_Asf_Bit => True, others => False ) ;
    
    Char_Spacing_Asf_Bit : constant := 6 ;
    Char_Spacing_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Char_Spacing_Asf_Bit => True, others => False ) ;
    
    Text_Colour_Asf_Bit : constant := 7 ;
    Text_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Text_Colour_Asf_Bit => True, others => False ) ;
    
    Line_Type_Asf_Bit : constant := 8 ;
    Line_Type_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Line_Type_Asf_Bit => True, others => False ) ;
    
    Line_Width_Asf_Bit : constant := 9 ;
    Line_Width_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Line_Width_Asf_Bit => True, others => False ) ;
    
    Line_Colour_Asf_Bit : constant := 10 ;
    Line_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Line_Colour_Asf_Bit => True, others => False ) ;
    
    Curve_Approx_Asf_Bit : constant := 11 ;
    Curve_Approx_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Curve_Approx_Asf_Bit => True, others => False ) ;
    
    Polyline_Interp_Asf_Bit : constant := 12 ;
    Polyline_Interp_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Polyline_Interp_Asf_Bit => True, others => False ) ;
    
    Interior_Style_Asf_Bit : constant := 13 ;
    Interior_Style_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Interior_Style_Asf_Bit => True, others => False ) ;
    
    Interior_Style_Index_Asf_Bit : constant := 14 ;
    Interior_Style_Index_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Interior_Style_Index_Asf_Bit => True, others => False ) ;
    
    Surface_Colour_Asf_Bit : constant := 15 ;
    Surface_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Colour_Asf_Bit => True, others => False ) ;
    
    Surface_Interp_Asf_Bit : constant := 16 ;
    Surface_Interp_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Interp_Asf_Bit => True, others => False ) ;
    
    Reflection_Model_Asf_Bit : constant := 17 ;
    Reflection_Model_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Reflection_Model_Asf_Bit => True, others => False ) ;
    
    Reflection_Attr_Asf_Bit : constant := 18 ;
    Reflection_Attr_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Reflection_Attr_Asf_Bit => True, others => False ) ;
    
    Bf_Interior_Style_Asf_Bit : constant := 19 ;
    Bf_Interior_Style_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Interior_Style_Asf_Bit => True, others => False ) ;
    
    Bf_Interior_Style_Index_Asf_Bit : constant := 20 ;
    Bf_Interior_Style_Index_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Interior_Style_Index_Asf_Bit => True, others => False ) ;
    
    Bf_Surface_Colour_Asf_Bit : constant := 21 ;
    Bf_Surface_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Surface_Colour_Asf_Bit => True, others => False ) ;
    
    Bf_Surface_Interp_Asf_Bit : constant := 22 ;
    Bf_Surface_Interp_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Surface_Interp_Asf_Bit => True, others => False ) ;
    
    Bf_Reflection_Model_Asf_Bit : constant := 23 ;
    Bf_Reflection_Model_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Reflection_Model_Asf_Bit => True, others => False ) ;
    
    Bf_Reflection_Attr_Asf_Bit : constant := 24 ;
    Bf_Reflection_Attr_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Bf_Reflection_Attr_Asf_Bit => True, others => False ) ;
    
    Surface_Approx_Asf_Bit : constant := 25 ;
    Surface_Approx_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Approx_Asf_Bit => True, others => False ) ;
    
    Surface_Edges_Asf_Bit : constant := 26 ;
    Surface_Edges_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Edges_Asf_Bit => True, others => False ) ;
    
    Surface_Edge_Type_Asf_Bit : constant := 27 ;
    Surface_Edge_Type_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Edge_Type_Asf_Bit => True, others => False ) ;
    
    Surface_Edge_Width_Asf_Bit : constant := 28 ;
    Surface_Edge_Width_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Edge_Width_Asf_Bit => True, others => False ) ;
    
    Surface_Edge_Colour_Asf_Bit : constant := 29 ;
    Surface_Edge_Colour_Asf : constant Aspect_Mask :=
        Aspect_Mask'( Surface_Edge_Colour_Asf_Bit => True, others => False ) ;
    
    type ASF is (Bundled, Individual);
    for ASF use (0, 1);
    
    type Asf_Values is array (Aspect_Mask'range) of ASF ;
    pragma Pack (Asf_Values) ;
    
    type Composition_Type is (Pre_Concatenate, Post_Concatenate, Comp_Replace);
    for Composition_Type use (0, 1, 2);

    type Culling_Mode is (Culling_None, Back_Faces, Front_Faces);
    for Culling_Mode use (0, 1, 2);

    -- Curve Type  and Surface Type 
    type Curve_Or_Surface_Type is (Rational, Non_Rational);
    for Curve_Or_Surface_Type use (0, 1);

    -- Get_Enumerated_Type return format 
    type Enumerated_Type_Returned is (ET_Index, ET_Mnemonic, ET_Both);
    for Enumerated_Type_Returned use (0, 1, 2);

    type Off_On is (Off, On);
    for Off_On use (0, 1);

    type Edge_Flag is new Off_On;
    type Edge_Flags is array (X_Natural range <>) of Edge_Flag ;

    type Surface_Shape is (Complex, Nonconvex, Convex, Unknown_Shape);
    for Surface_Shape use (0, 1, 2, 3);
    for Surface_Shape'size use 16;

    type Contour_Hints is (Disjoint, Nested, Intersecting, Unknown_Contour);
    for Contour_Hints use (0, 1, 2, 3);
    for Contour_Hints'size use 8;

    -- Status in Get_Table_Entry 
    type Table_Entry_Status is (Default_Entry, Defined_Entry);
    for Table_Entry_Status use (0, 1);

    -- Value type in Get_Table_Entr{y|ies} 
    type Returned_Value_Type is (Set, Realized);
    for Returned_Value_Type use (0, 1);

    -- Constants for Path and Vertical and Horizontal alignment 
    type Text_Path is (Right, Left, Up, Down);
    for Text_Path use (0, 1, 2, 3);
    for Text_Path'size use 16;

    type Vertical_Alignment is (Normal, Top, Cap, Half, Base, Bottom);
    for Vertical_Alignment use (0, 1, 2, 3, 4, 5);
    for Vertical_Alignment'size use 16;

    type Horizontal_Alignment is (Normal, Left, Center, Right);
    for Horizontal_Alignment use (0, 1, 2, 3);
    for Horizontal_Alignment'size use 16;

    type Text_Alignment is 
    record
        Horizontal : Horizontal_Alignment;
        Vertical   : Vertical_Alignment;
    end record;

    -- Text precision 
    type Text_Precision is
        (String_Precision, Char_Precision, Stroke_Precision);
    for Text_Precision use (0, 1, 2);
    for Text_Precision'size use 16;

    type Character_Set_Width is (Byte_Set, Word_Set, Long_Set, Ada_Set) ;
    for Character_Set_Width use (0, 1, 2, 4);

    type Character_Set is range 0 .. 32767 ;
    
    subtype Byte_Character is X_Character ;
    type Word_Character is range 0 .. 65_535 ;
    type Long_Character is new X_Long_Integer ;
    
    subtype Mono_Natural is Natural range 0 .. 80 ;
    type Byte_String is array (Mono_Natural range <>) of Byte_Character ;
    type Word_String is array (Mono_Natural range <>) of Word_Character ;
    type Long_String is array (Mono_Natural range <>) of Long_Character ;
    
    type Ada_Characters ( Size : Mono_Natural := 0 ) is
    record
        Chars : String (1 .. Size) ;
    end record ;
    
    type Byte_Characters ( Size : Mono_Natural := 0 ) is
    record
        Chars : Byte_String (1 .. Size) ;
    end record ;

    type Word_Characters ( Size : Mono_Natural := 0 ) is
    record
        Chars : Word_String (1 .. Size) ;
    end record ;

    type Long_Characters ( Size : Mono_Natural := 0 ) is
    record
        Chars : Long_String (1 .. Size) ;
    end record ;
    
    type Encoding_State is range 0 .. 255 ;
    for Encoding_State'size use 8 ;
    
    type Mono_Encoding (Width : Character_Set_Width := Byte_Set) is
    record
        Set    : Character_Set ;
        State  : Encoding_State ;
        case Width is
        when Byte_Set  => Byte_Chars  : Byte_Characters ;
        when Word_Set  => Word_Chars  : Word_Characters ;
        when Long_Set  => Long_Chars  : Long_Characters ;
        when Ada_Set   => Ada_Chars   : Ada_Characters ;
        end case ;
    end record ;
    
    type Mono_Encoding_Array is array (X_Natural range <>) of Mono_Encoding ;
    type Mono_Encoding_List is access Mono_Encoding_Array ;
    type Set_Of_Mono_Encoding_List is array (X_Natural range <>) of
        Mono_Encoding_List ;
    
    type Extent_Info is
    record
        Rectangle     : Mc.Rectangular_Region_2 ;
        Concatenation : Mc.Vector_2 ;
    end record ;
    
    type Extent_Info_Array is array (X_Natural range <>) of Extent_Info ;
    type Extent_Info_List ( Size : X_Natural := 0 ) is
    record
        Extents : Extent_Info_Array (1 .. Size) ;
    end record ;
    
    type Optional_Data is
    record
        Color  : Color_Representation ;
        Normal : Mc.Vector_3 ;
        Edges  : Edge_Flag ;
    end record ;
    
    type Optional_Data_Array is array (X_Natural range <>) of Optional_Data ;
    
    type Vertex is
    record
        Point    : Mc.Point_3 ;
        Options  : Optional_Data ;
    end record ;
    
    type Vertex_Index is range 0 .. 65_535 ;
    for Vertex_Index'size use 16 ;
    type Vertex_Array is array (Vertex_Index range <>) of Vertex ;
    type Vertices is access Vertex_Array ;
    
    type Array_Of_Vertices is array (X_Natural range <>) of Vertices ;
    
    type Vertex_Matrix is array ( X_Natural range <>,
                                  X_Natural range <>) of Vertex ;
    
    type Facet_Data is
    record
        Facet_Options : Optional_Data ;
        Vertex_List   : Vertices ;
    end record ;
    
    type Vertex_Index_Array is array (X_Natural range <>) of Vertex_Index ;
    type Contour is access Vertex_Index_Array ;
    type Contour_Array is array (X_Natural range <>) of Contour ;
    type Set_Of_Contour is access Contour_Array ;
    type Set_Of_Contour_Array is array (X_Natural range <>) of Set_Of_Contour;
    type Set_Of_Contour_Sets is access Set_Of_Contour_Array ;
    
    type Option is (Color_Option, Normal_Option, Edge_Option) ;
    type Option_Mask is array (Option range <>) of Boolean ;
    subtype Line_Option_Mask is Option_Mask (Color_Option .. Color_Option) ;
    subtype Facet_Option_Mask is Option_Mask (Color_Option .. Normal_Option) ;
    subtype Area_Set_Option_Mask is Option_Mask (Color_Option .. Edge_Option);
    subtype Edge_Option_Mask is Option_Mask (Edge_Option .. Edge_Option) ;
    subtype All_Options_Mask is Option_Mask (Option) ;
    
    type Predefined_Parametric_Characteristics is
        ( None, Imp_Dep, Iso_Curves, Mc_Level_Curves,
          Wc_Level_Curves, Specific) ;
    
    type Parametric_Characteristics_Data
        (Kind : Predefined_Parametric_Characteristics := None) is
    record
        case Kind is
        when None | Imp_Dep => null ;
        when Iso_Curves =>
            Placement : Isoparametric_Curves ;
            U_Number  : X_Natural ;
            V_Number  : X_Natural ;
        when Mc_Level_Curves =>
            Mc_Origin         : Mc.Point_3 ;
            Mc_Vector         : Mc.Vector_3 ;
            Mc_Intersections  : Mc.Access_Point_List_3 ;
        when Wc_Level_Curves =>
            Wc_Origin         : Wc.Point_3 ;
            Wc_Vector         : Wc.Vector_3 ;
            Wc_Intersections  : Wc.Access_Point_List_3 ;
        when Specific =>
            Characteristics : Parametric_Surface_Characteristics ;
            Data            : Data_Record ;
        end case ;
    end record ;
    
    type Knot_Value is digits Pex_Precision range 0.0 .. 1.0 ;
    type Knot_Array is array (Short_Natural range <>) of Knot_Value ;
    type Knot_List is access Knot_Array ;
    
    type Knot_Range is
    record
        Min : Knot_Value ;
        Max : Knot_Value ;
    end record ;
    
    type Non_Uniform_B_Spline_Curve
        ( Kind       : Curve_Or_Surface_Type := Rational ;
          Num_Knots  : Short_Natural := 8 ;
          Length     : Short_Natural := 0 ) is
    record
        Order    : Nurb_Order ;
        Bounds   : Knot_Range ;
        Knots    : Knot_Array (1 .. Num_Knots) ;
        case Kind is
        when Rational =>
            Controls_4 : Mc.Point_List_4 (1 .. Length);
        when Non_Rational =>
            Controls_3 : Mc.Point_List_3 (1 .. Length);
        end case ;
    end record ;
    
    type Mc_Matrix_3 is array ( X_Natural range <>,
                               X_Natural range <> ) of Mc.Point_3 ;
    
    type Mc_Matrix_4 is array ( X_Natural range <>,
                               X_Natural range <> ) of Mc.Point_4 ;
    
    type Trim_Curve (Kind : Curve_Or_Surface_Type := Rational) is
    record
        Visibility : Edge_Flag ;
        Order      : Nurb_Order ;
        Method     : Trim_Curve_Approx_Method ;
        Tolerance  : Mc_Type ;
        Bounds     : Knot_Range ;
        Knots      : Knot_List ;
        case Kind is
        when Rational =>
            Controls_3 : Mc.Access_Point_List_3 ;
        when Non_Rational =>
            Controls_2 : Mc.Access_Point_List_2 ;
        end case ;
    end record ;
    
    type Trim_Curve_Array is array (X_Natural range <>) of Trim_Curve ;
    
    type Trimming_Loop is access Trim_Curve_Array ;
    
    type Trimming_Loop_Array is array (X_Natural range <>) of Trimming_Loop ;
    
    type Trimming_List ( Size : Short_Natural := 0 ) is
    record
        Loops  : Trimming_Loop_Array (1 .. Size) ;
    end record ;
    
    type Non_Uniform_B_Spline_Surface
        ( Kind         : Curve_Or_Surface_Type := Rational ;
          Num_U_Knots  : Short_Natural := 8 ;
          Num_V_Knots  : Short_Natural := 8 ;
          U_Length     : Short_Natural := 0 ;
          V_Length     : Short_Natural := 0 ) is
    record
        U_Order    : Nurb_Order ;
        V_Order    : Nurb_Order ;
        U_Knots    : Knot_Array (1 .. Num_U_Knots) ;
        V_Knots    : Knot_Array (1 .. Num_U_Knots) ;
        case Kind is
        when Rational =>
            Controls_4 : Mc_Matrix_4 (1 .. U_Length, 1 .. V_Length);
        when Non_Rational =>
            Controls_3 : Mc_Matrix_3 (1 .. U_Length, 1 .. V_Length);
        end case ;
    end record ;
    
    type Visual_State is (Correct, Deferred, Simulated) ;
    for Visual_State use (0, 1, 2);

    type Display_State is (Empty,  Not_Empty ) ;
    for Display_State use (0, 1);
    
    type Pick_All_State is (More_Hits, No_More_Hits, May_Be_More_Hits) ;
    
    type Pick_Status is (No_Pick, Ok, Aborted) ;

    type Pick_Echo_Switch is (No_Echo, Echo, Unecho) ;

    type Pick_Path_Order is (Top_First, Bottom_First) ;

    --  for Get_Structures_In_Network 
    type Structure_Flag is (All_Structures, Orphans) ;

    --  for Get_Ancestors
    type Path_Part is (Top_Part, Bottom_Part) ;

    type Priority is (Higher, Lower) ;

    type Clip_Status is (Clip, No_Clip) ;

    type Font_Array is array (Character_Set range <>) of Font ;
    type Font_List is access Font_Array ;
    
    --
    -- Types for lookup table requests
    --
    --
    type Lookup_Table_Type is ( Line_Bundle, Marker_Bundle, Text_Bundle, 
                                Interior_Bundle, Edge_Bundle, Pattern, 
                                Text_Font, Color, View, Light,
                                Depth_Cue, Color_Approximation);
    for Lookup_Table_Type use ( 1, 2, 3,  4,  5,  6,
                                7, 8, 9, 10, 11, 12);
    for Lookup_Table_Type'size use 16;
    --
    type Lookup_Table ( Kind : Lookup_Table_Type) is private;

    subtype Marker_Bundle_Lut is Lookup_Table ( Kind => Marker_Bundle );
    subtype Text_Bundle_Lut is Lookup_Table ( Kind => Text_Bundle );
    subtype Line_Bundle_Lut is Lookup_Table ( Kind => Line_Bundle );
    subtype Interior_Bundle_Lut is Lookup_Table ( Kind => Interior_Bundle );
    subtype Edge_Bundle_Lut is Lookup_Table ( Kind => Edge_Bundle );
    subtype Color_Lut is Lookup_Table ( Kind => Color );
    subtype View_Lut is Lookup_Table ( Kind => View );
    subtype Light_Lut is Lookup_Table ( Kind => Light );
    subtype Depth_Cue_Lut is Lookup_Table ( Kind => Depth_Cue );
    subtype Color_Approximation_Lut is Lookup_Table
        ( Kind => Color_Approximation );
    subtype Pattern_Lut is Lookup_Table ( Kind => Pattern );
    subtype Text_Font_Lut is Lookup_Table ( Kind => Text_Font );
    
    Null_Marker_Bundle_Lut       : constant Marker_Bundle_Lut ;
    Null_Text_Bundle_Lut         : constant Text_Bundle_Lut ;
    Null_Line_Bundle_Lut         : constant Line_Bundle_Lut ;
    Null_Interior_Bundle_Lut     : constant Interior_Bundle_Lut ;
    Null_Edge_Bundle_Lut         : constant Edge_Bundle_Lut ;
    Null_Color_Lut               : constant Color_Lut ;
    Null_View_Lut                : constant View_Lut ;
    Null_Light_Lut               : constant Light_Lut ;
    Null_Depth_Cue_Lut           : constant Depth_Cue_Lut ;
    Null_Color_Approximation_Lut : constant Color_Approximation_Lut ;
    Null_Pattern_Lut             : constant Pattern_Lut ;
    Null_Text_Font_Lut           : constant Text_Font_Lut ;
    
    --
    type Lookup_Table_Index_Array is array ( Index_Count range <>) of
        Lookup_Table_Index;
        
    type Lookup_Table_Index_List ( Size : Index_Count := 0 ) is
    record
        Indices : Lookup_Table_Index_Array (1 .. Size) ;
    end record ;
    
    type Light_Lut_Array is array (Lookup_Table_Index range <>) of Light_Lut;
    
    subtype Light_List is Lookup_Table_Index_List ;
    subtype View_List is Lookup_Table_Index_List ;
    
    --
    --
    type Lookup_Table_Entry_Status is (Default, Defined );
    for Lookup_Table_Entry_Status use (0, 1);
    for Lookup_Table_Entry_Status'size use 16;
    --
    type Lookup_Table_Value_Type is (Set, Realized);
    for Lookup_Table_Value_Type use (0, 1);
    for Lookup_Table_Value_Type'size use 16;
    
    --
    -- types for lookup tables (from MIT C code)
    --
    type Index_Matrix is array (X_Natural range <>, X_Natural range <>) of
        Lookup_Table_Index ;
    
    type Table_Info is
    record
        Definable_Entries : X_Natural ;
        Num_Predefined    : X_Natural ;
        Predefined_Min    : X_Natural ;
        Predefined_Max    : X_Natural ;
    end record;
    --  
    type Line_Bundle_Entry is
    record
        Type_Index    : Line_Type ;
        Interpolation : Polyline_Interpolation_Method ;
        Approximation : Curve_Approximation_Method ;
        Width         : Line_Width ;
        Color         : Color_Representation ;
    end record;
    --  
    type Marker_Bundle_Entry is
    record
        Type_Index    : Marker_Type ;
        Size          : Marker_Size ;
        Color         : Color_Representation ;
    end record;
    --  
    type Text_Bundle_Entry is
    record
        Index           : Text_Font_Index ;
        Precision       : Text_Precision ;
        Expansion       : Char_Expansion ;
        Spacing         : Char_Spacing ;
        Color           : Color_Representation ;
    end record;
    --  
    --  
    type Interior_Bundle_Entry is
    record
        Interior            : Surface_Interior ;
        Reflection          : Reflection_Model ;
        Interpolation       : Surface_Interpolation_Method ;
        Approximation       : Surface_Approximation ;
        Back_Interior       : Surface_Interior ;
        Back_Reflection     : Reflection_Model ;
        Back_Interpolation  : Surface_Interpolation_Method ;
        Back_Approximation  : Surface_Approximation ;
        Color               : Color_Representation ;
        Attributes          : Reflection_Attributes ;
        Back_Color          : Color_Representation ;
        Back_Attributes     : Reflection_Attributes ;
    end record;
    --  
    type Edge_Bundle_Entry is
    record
        Switch      : Edge_Flag ;
        Type_Index  : Surface_Edge_Type ;
        Width       : Edge_Width ;
        Color       : Color_Representation ;
    end record;
    --  
    type Pattern_Entry_Record ( Kind   : Predefined_Color_Type ;
                                Num_X  : X_Natural ;
                                Num_Y  : X_Natural ) is
    record
        case Kind is
        when Indexed   => Indices   : Index_Matrix (1..Num_X, 1..Num_Y) ;
        when Rgb_Float => Rgbs      : Rgb_Float_Matrix (1..Num_X, 1..Num_Y) ;
        when Cie_Float => Cies      : Cie_Float_Matrix (1..Num_X, 1..Num_Y) ;
        when Hsv_Float => Hsvs      : Hsv_Float_Matrix (1..Num_X, 1..Num_Y) ;
        when Hls_Float => Hlss      : Hls_Float_Matrix (1..Num_X, 1..Num_Y) ;
        when Rgb_8     => Rgb_8s    : Rgb_Byte_Matrix (1..Num_X, 1..Num_Y) ;
        when Rgb_16    => Rgb_16s   : Rgb_Word_Matrix (1..Num_X, 1..Num_Y) ;
        when Specific  =>
            The_Type    : Color_Type ;
            Specifics : Specific_Matrix (1..Num_X, 1..Num_Y) ;
        end case ;
    end record;
    type Pattern_Entry is access Pattern_Entry_Record ;
    --
    -- 
    subtype Color_Entry is Color_Representation ;
    --  
    type Text_Font_Entry is
    record
        Fonts  : Font_List ;
    end record;
    --
    subtype Clipping_Indicator is Boolean_Array (0 .. 2);
    
    Clip_Xy_Bit : constant := 0;
    Xy_Clipping : Clipping_Indicator :=
        Clipping_Indicator'(Clip_Xy_Bit => True,
                            others      => False);
    
    Clip_Back_Bit : constant := 1;
    Back_Clipping : Clipping_Indicator :=
        Clipping_Indicator'(Clip_Back_Bit => True,
                            others        => False);
    
    Clip_Front_Bit : constant := 2;
    Front_Clipping : Clipping_Indicator :=
        Clipping_Indicator'(Clip_Front_Bit => True,
                            others         => False);

    type View_Entry is
    record
        Clipping_Flags  : Clipping_Indicator ;
        Clipping_Limits : Npc.Rectangular_Region_3;
        Orientation     : Transformation_Matrix;
        Mapping         : Transformation_Matrix;
    end record ;
    
    for View_Entry use
    record
        Clipping_Flags      at  1 * X_32_Bits range  0 ..  2;
        Clipping_Limits     at  2 * X_32_Bits range  0 .. (6*Fp_Size) -1;
        Orientation    at  (2 +  6*Fp) * X_32_Bits range  0 .. (16*Fp_Size)-1;
        Mapping        at  (2 + 22*Fp) * X_32_Bits range  0 .. (16*Fp_Size)-1;
    end record;
    
    --  
    type Light_Entry is
    record
        Type_Index      : Light_Type ;
        Direction       : Mc.Vector_3 ;
        Point           : Mc.Point_3 ;
        Concentration   : Reflection_Coefficient ;
        Spread_Angle    : Angle ;
        Attenuation     : Attenuation_Type ;
        Light           : Color_Representation ;
    end record;
    --  
    type Depth_Cue_Entry is
    record
        Mode          : Off_On ;
        Front_Plane   : Mc_Type ;
        Back_Plane    : Mc_Type ;
        Front_Scaling : Mc_Type ;
        Back_Scaling  : Mc_Type ;
        Color         : Color_Representation ;
    end record;
    --  
    type Color_Approximation_Entry is
    record
        Approximation : Color_Approximation_Type ;
        Model         : Color_Approximation_Method ;
        Max_1         : Color_Limit ;
        Max_2         : Color_Limit ;
        Max_3         : Color_Limit ;
        Dither        : Off_On ;
        Mult_1        : Color_Multiplier ;
        Mult_2        : Color_Multiplier ;
        Mult_3        : Color_Multiplier ;
        Weight_1      : Color_Weight ;
        Weight_2      : Color_Weight ;
        Weight_3      : Color_Weight ;
        Base_Pixel    : Pixel_Value ;
    end record;
    
    --
    type Lookup_Table_Entry (Kind : Lookup_Table_Type := View) is
    record
        case Kind is
        when Line_Bundle         => Line          : Line_Bundle_Entry ;
        when Marker_Bundle       => Marker        : Marker_Bundle_Entry ;
        when Text_Bundle         => Text          : Text_Bundle_Entry ;
        when Interior_Bundle     => Interior      : Interior_Bundle_Entry ;
        when Edge_Bundle         => Edge          : Edge_Bundle_Entry ;
        when Pattern             => Pattern       : Pattern_Entry ;
        when Text_Font           => Font          : Text_Font_Entry ;
        when Color               => Color         : Color_Entry ;
        when View                => View          : View_Entry ;
        when Light               => Light         : Light_Entry ;
        when Depth_Cue           => Depth_Cue     : Depth_Cue_Entry ;
        when Color_Approximation => Approximation : Color_Approximation_Entry;
        end case ;
    end record;

    type Lookup_Table_Entry_Array is array ( Lookup_Table_Index range <>) of
        Lookup_Table_Entry;

    type Lookup_Table_Entry_List ( Size : Lookup_Table_Index := 0 ) is
    record
        Entries : Lookup_Table_Entry_Array(1 .. Size) ;
    end record ;

    -- Name Sets structures
    --
    type Name_Set_Array is array (X_Natural range <>) of Name_Set ;
    --
    type Name_Set_Action is (Add, Remove, Replace);
    for Name_Set_Action use (0, 1, 2);
    for Name_Set_Action'size use 16;
    --
    type Name_Set_Filter is
    record
        Inclusion : Name_Set;
        Exclusion : Name_Set;
    end record;
    
    type Name_Set_Filter_Array is array (X_Natural range <>) of
        Name_Set_Filter ;
    type Name_Set_Filter_Access is access Name_Set_Filter_Array ;
    type Name_Set_Filter_List is
    record
        Size : X_Natural ;
        List : Name_Set_Filter_Access ;
    end record ;
    
    -- Pipeline contexts
    
    subtype Pipeline_Context_Bitmask is Boolean_Array (0 .. 63) ;

    Marker_Type_Bit : constant := 0 ;
    Marker_Type_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Marker_Type_Bit => True,
                                   others => False ) ;

    Marker_Scale_Bit : constant := 1 ;
    Marker_Scale_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Marker_Scale_Bit => True,
                                   others => False ) ;

    Marker_Color_Bit : constant := 2 ;
    Marker_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Marker_Color_Bit => True,
                                   others => False ) ;

    Marker_Bundle_Index_Bit : constant := 3 ;
    Marker_Bundle_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Marker_Bundle_Index_Bit => True,
                                   others => False ) ;

    Text_Font_Bit : constant := 4 ;
    Text_Font_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Font_Bit => True,
                                   others => False ) ;

    Text_Precision_Bit : constant := 5 ;
    Text_Precision_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Precision_Bit => True,
                                   others => False ) ;

    Char_Expansion_Bit : constant := 6 ;
    Char_Expansion_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Char_Expansion_Bit => True,
                                   others => False ) ;

    Char_Spacing_Bit : constant := 7 ;
    Char_Spacing_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Char_Spacing_Bit => True,
                                   others => False ) ;

    Text_Color_Bit : constant := 8 ;
    Text_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Color_Bit => True,
                                   others => False ) ;

    Char_Height_Bit : constant := 9 ;
    Char_Height_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Char_Height_Bit => True,
                                   others => False ) ;

    Char_Up_Vector_Bit : constant := 10 ;
    Char_Up_Vector_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Char_Up_Vector_Bit => True,
                                   others => False ) ;

    Text_Path_Bit : constant := 11 ;
    Text_Path_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Path_Bit => True,
                                   others => False ) ;

    Text_Alignment_Bit : constant := 12 ;
    Text_Alignment_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Alignment_Bit => True,
                                   others => False ) ;

    Atext_Height_Bit : constant := 13 ;
    Atext_Height_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Atext_Height_Bit => True,
                                   others => False ) ;

    Atext_Up_Vector_Bit : constant := 14 ;
    Atext_Up_Vector_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Atext_Up_Vector_Bit => True,
                                   others => False ) ;

    Atext_Path_Bit : constant := 15 ;
    Atext_Path_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Atext_Path_Bit => True,
                                   others => False ) ;

    Atext_Alignment_Bit : constant := 16 ;
    Atext_Alignment_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Atext_Alignment_Bit => True,
                                   others => False ) ;

    Atext_Style_Bit : constant := 17 ;
    Atext_Style_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Atext_Style_Bit => True,
                                   others => False ) ;

    Text_Bundle_Index_Bit : constant := 18 ;
    Text_Bundle_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Text_Bundle_Index_Bit => True,
                                   others => False ) ;

    Line_Type_Bit : constant := 19 ;
    Line_Type_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Line_Type_Bit => True,
                                   others => False ) ;

    Line_Width_Bit : constant := 20 ;
    Line_Width_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Line_Width_Bit => True,
                                   others => False ) ;

    Line_Color_Bit : constant := 21 ;
    Line_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Line_Color_Bit => True,
                                   others => False ) ;

    Curve_Approximation_Bit : constant := 22 ;
    Curve_Approximation_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Curve_Approximation_Bit => True,
                                   others => False ) ;

    Polyline_Interpolation_Bit : constant := 23 ;
    Polyline_Interpolation_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Polyline_Interpolation_Bit => True,
                                   others => False ) ;

    Line_Bundle_Index_Bit : constant := 24 ;
    Line_Bundle_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Line_Bundle_Index_Bit => True,
                                   others => False ) ;

    Interior_Style_Bit : constant := 25 ;
    Interior_Style_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Interior_Style_Bit => True,
                                   others => False ) ;

    Interior_Style_Index_Bit : constant := 26 ;
    Interior_Style_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Interior_Style_Index_Bit => True,
                                   others => False ) ;

    Surface_Color_Bit : constant := 27 ;
    Surface_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Color_Bit => True,
                                   others => False ) ;

    Surface_Refl_Attr_Bit : constant := 28 ;
    Surface_Refl_Attr_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Refl_Attr_Bit => True,
                                   others => False ) ;

    Surface_Refl_Model_Bit : constant := 29 ;
    Surface_Refl_Model_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Refl_Model_Bit => True,
                                   others => False ) ;

    Surface_Interpolation_Bit : constant := 30 ;
    Surface_Interpolation_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Interpolation_Bit => True,
                                   others => False ) ;

    Bf_Interior_Style_Bit : constant := 31 ;
    Bf_Interior_Style_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Interior_Style_Bit => True,
                                   others => False ) ;

    Bf_Interior_Style_Index_Bit : constant := 32 ;
    Bf_Interior_Style_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Interior_Style_Index_Bit => True,
                                   others => False ) ;

    Bf_Surface_Color_Bit : constant := 33 ;
    Bf_Surface_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Surface_Color_Bit => True,
                                   others => False ) ;

    Bf_Surface_Refl_Attr_Bit : constant := 34 ;
    Bf_Surface_Refl_Attr_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Surface_Refl_Attr_Bit => True,
                                   others => False ) ;

    Bf_Surface_Refl_Model_Bit : constant := 35 ;
    Bf_Surface_Refl_Model_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Surface_Refl_Model_Bit => True,
                                   others => False ) ;

    Bf_Surface_Interpolation_Bit : constant := 36 ;
    Bf_Surface_Interpolation_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Bf_Surface_Interpolation_Bit => True,
                                   others => False ) ;

    Surface_Approximation_Bit : constant := 37 ;
    Surface_Approximation_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Approximation_Bit => True,
                                   others => False ) ;

    Culling_Mode_Bit : constant := 38 ;
    Culling_Mode_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Culling_Mode_Bit => True,
                                   others => False ) ;

    Distinguish_Flag_Bit : constant := 39 ;
    Distinguish_Flag_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Distinguish_Flag_Bit => True,
                                   others => False ) ;

    Pattern_Size_Bit : constant := 40 ;
    Pattern_Size_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Pattern_Size_Bit => True,
                                   others => False ) ;

    Pattern_Ref_Pt_Bit : constant := 41 ;
    Pattern_Ref_Pt_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Pattern_Ref_Pt_Bit => True,
                                   others => False ) ;
    
    Pattern_Ref_Vec1_Bit : constant := 42 ;
    Pattern_Ref_Vec1_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Pattern_Ref_Vec1_Bit => True,
                                   others => False ) ;
    
    Pattern_Ref_Vec2_Bit : constant := 43 ;
    Pattern_Ref_Vec2_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Pattern_Ref_Vec2_Bit => True,
                                   others => False ) ;
    
    Interior_Bundle_Index_Bit : constant := 44 ;
    Interior_Bundle_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Interior_Bundle_Index_Bit => True,
                                   others => False ) ;
    
    Surface_Edge_Flag_Bit : constant := 45 ;
    Surface_Edge_Flag_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Edge_Flag_Bit => True,
                                   others => False ) ;
    
    Surface_Edge_Type_Bit : constant := 46 ;
    Surface_Edge_Type_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Edge_Type_Bit => True,
                                   others => False ) ;
    
    Surface_Edge_Width_Bit : constant := 47 ;
    Surface_Edge_Width_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Edge_Width_Bit => True,
                                   others => False ) ;
    
    Surface_Edge_Color_Bit : constant := 48 ;
    Surface_Edge_Color_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Surface_Edge_Color_Bit => True,
                                   others => False ) ;
    
    Edge_Bundle_Index_Bit : constant := 49 ;
    Edge_Bundle_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Edge_Bundle_Index_Bit => True,
                                   others => False ) ;
    
    Local_Transform_Bit : constant := 50 ;
    Local_Transform_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Local_Transform_Bit => True,
                                   others => False ) ;
    
    Global_Transform_Bit : constant := 51 ;
    Global_Transform_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Global_Transform_Bit => True,
                                   others => False ) ;
    
    Model_Clip_Bit : constant := 52 ;
    Model_Clip_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Model_Clip_Bit => True,
                                   others => False ) ;
    
    Model_Clip_Volume_Bit : constant := 53 ;
    Model_Clip_Volume_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Model_Clip_Volume_Bit => True,
                                   others => False ) ;
    
    View_Index_Bit : constant := 54 ;
    View_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( View_Index_Bit => True,
                                   others => False ) ;
    
    Light_State_Bit : constant := 55 ;
    Light_State_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Light_State_Bit => True,
                                   others => False ) ;
    
    Depth_Cue_Index_Bit : constant := 56 ;
    Depth_Cue_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Depth_Cue_Index_Bit => True,
                                   others => False ) ;
    
    Set_Asf_Values_Bit : constant := 57 ;
    Set_Asf_Values_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Set_Asf_Values_Bit => True,
                                   others => False ) ;
    
    Pick_Id_Bit : constant := 58 ;
    Pick_Id_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Pick_Id_Bit => True,
                                   others => False ) ;
    
    Hlhsr_Identifier_Bit : constant := 59 ;
    Hlhsr_Identifier_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Hlhsr_Identifier_Bit => True,
                                   others => False ) ;
    
    Name_Set_Bit : constant := 60 ;
    Name_Set_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Name_Set_Bit => True,
                                   others       => False ) ;
    
    Color_Approx_Index_Bit : constant := 61 ;
    Color_Approx_Index_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Color_Approx_Index_Bit => True,
                                   others                 => False ) ;
    
    Rendering_Color_Model_Bit : constant := 62 ;
    Rendering_Color_Model_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Rendering_Color_Model_Bit => True,
                                   others                    => False ) ;
    
    Para_Surf_Characteristics_Bit : constant := 63 ;
    Para_Surf_Characteristics_Mask : constant Pipeline_Context_Bitmask :=
        Pipeline_Context_Bitmask'( Para_Surf_Characteristics_Bit => True,
                                   others                        => False ) ;
    
    type Model_Clip_Volume is new Mc.Access_Half_Space_List_3 ;
    
    type Pipeline_Context_Values is
    record
        Pc_Marker_Type                 : Marker_Type ;
        Pc_Marker_Size                 : Marker_Size ;
        Pc_Marker_Color                : Color_Representation ;
        Pc_Marker_Bundle_Index         : Marker_Bundle_Index ;
        Pc_Text_Font                   : Text_Font_Index ;
        Pc_Text_Precision              : Text_Precision ;
        Pc_Char_Expansion              : Char_Expansion ;
        Pc_Char_Spacing                : Char_Spacing ;
        Pc_Text_Color                  : Color_Representation ;
        Pc_Char_Height                 : Mc.Magnitude ;
        Pc_Char_Up_Vector              : Mc.Vector_2 ;
        Pc_Text_Path                   : Text_Path ;
        Pc_Text_Alignment              : Text_Alignment ;
        Pc_Atext_Height                : Mc.Magnitude ;
        Pc_Atext_Up_Vector             : Mc.Vector_2 ;
        Pc_Atext_Path                  : Text_Path ;
        Pc_Atext_Alignment             : Text_Alignment ;
        Pc_Atext_Style                 : Annotation_Style ;
        Pc_Text_Bundle_Index           : Text_Bundle_Index ;
        Pc_Line_Type                   : Line_Type ;
        Pc_Line_Width                  : Line_Width ;
        Pc_Line_Color                  : Color_Representation ;
        Pc_Curve_Approximation         : Curve_Approximation ;
        Pc_Polyline_Interpolation      : Polyline_Interpolation_Method ;
        Pc_Line_Bundle_Index           : Line_Bundle_Index ;
        Pc_Interior                    : Surface_Interior ;
        Pc_Surface_Color               : Color_Representation ;
        Pc_Surface_Refl_Attr           : Reflection_Attributes ;
        Pc_Surface_Refl_Model          : Reflection_Model ;
        Pc_Surface_Interpolation       : Surface_Interpolation_Method ;
        Pc_Bf_Interior                 : Surface_Interior ;
        Pc_Bf_Surface_Color            : Color_Representation ;
        Pc_Bf_Surface_Refl_Attr        : Reflection_Attributes ;
        Pc_Bf_Surface_Refl_Model       : Reflection_Model ;
        Pc_Bf_Surface_Interpolation    : Surface_Interpolation_Method ;
        Pc_Surface_Approximation       : Surface_Approximation ;
        Pc_Culling_Mode                : Culling_Mode ;
        Pc_Distinguish_Flag            : Boolean ;
        Pc_Pattern_Size                : Pattern_Size ;
        Pc_Pattern_Ref_Pt              : Mc.Point_3 ;
        Pc_Pattern_Ref_Vec1            : Mc.Vector_3 ;
        Pc_Pattern_Ref_Vec2            : Mc.Vector_3 ;
        Pc_Interior_Bundle_Index       : Interior_Bundle_Index ;
        Pc_Surface_Edge_Flag           : Edge_Flag ;
        Pc_Surface_Edge_Type           : Surface_Edge_Type ;
        Pc_Surface_Edge_Width          : Edge_Width ;
        Pc_Surface_Edge_Color          : Color_Representation ;
        Pc_Edge_Bundle_Index           : Edge_Bundle_Index ;
        Pc_Local_Transform             : Transformation_Matrix ;
        Pc_Global_Transform            : Transformation_Matrix ;
        Pc_Model_Clip                  : Clip_Status ;
        Pc_Model_Clip_Volume           : Model_Clip_Volume ;
        Pc_View_Index                  : View_Index ;
        Pc_Light_State                 : Light_List ;
        Pc_Depth_Cue_Index             : Depth_Cue_Index ;
        Pc_Asf_Mask                    : Aspect_Mask ;
        Pc_Asf_Values                  : Asf_Values ;
        Pc_Pick_Id                     : Pick_Id ;
        Pc_Hlhsr_Identifier            : Hlhsr_Id ;
        Pc_Name_Set                    : Name_Set ;
        Pc_Color_Approx_Index          : Color_Approximation_Index ;
        Pc_Rendering_Color_Model       : Rendering_Color_Model ;
        Pc_Para_Surf_Characteristics   : Parametric_Characteristics_Data ;
    end record ;
    
    -- Renderers
    
    type Renderer_State is (Idle, Rendering, Picking) ;
    
    type Match_Draw_Type is ( Dont_Care, Match_Window,
                              Match_Pixmap, Match_Buffer ) ;
    type Renderer_Target is
    record
        Depth     : Depth_Type ;
        Match     : Match_Draw_Type ;
        Matching  : Visual ;
    end record ;
    
    type Renderer_Targets is array (X_Positive range <>) of Renderer_Target ;
    
    subtype Renderer_Bitmask is Boolean_Array (0 .. 29) ;
    
    Rd_Pipeline_Context_Bit : constant := 0 ;
    Rd_Pipeline_Context_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Pipeline_Context_Bit => True,
                           others => False ) ;
    
    Rd_Current_Path_Bit : constant := 1 ;
    Rd_Current_Path_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Current_Path_Bit => True,
                           others => False ) ;
    
    Rd_Marker_Bundle_Bit : constant := 2 ;
    Rd_Marker_Bundle_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Marker_Bundle_Bit => True,
                           others => False ) ;
    
    Rd_Text_Bundle_Bit : constant := 3 ;
    Rd_Text_Bundle_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Text_Bundle_Bit => True,
                           others => False ) ;
    
    Rd_Line_Bundle_Bit : constant := 4 ;
    Rd_Line_Bundle_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Line_Bundle_Bit => True,
                           others => False ) ;
    
    Rd_Interior_Bundle_Bit : constant := 5 ;
    Rd_Interior_Bundle_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Interior_Bundle_Bit => True,
                           others => False ) ;
    
    Rd_Edge_Bundle_Bit : constant := 6 ;
    Rd_Edge_Bundle_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Edge_Bundle_Bit => True,
                           others => False ) ;
    
    Rd_View_Table_Bit : constant := 7 ;
    Rd_View_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_View_Table_Bit => True,
                           others => False ) ;
    
    Rd_Color_Table_Bit : constant := 8 ;
    Rd_Color_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Color_Table_Bit => True,
                           others => False ) ;
    
    Rd_Depth_Cue_Table_Bit : constant := 9 ;
    Rd_Depth_Cue_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Depth_Cue_Table_Bit => True,
                           others => False ) ;
    
    Rd_Light_Table_Bit : constant := 10 ;
    Rd_Light_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Light_Table_Bit => True,
                           others => False ) ;
    
    Rd_Color_Approx_Table_Bit : constant := 11 ;
    Rd_Color_Approx_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Color_Approx_Table_Bit => True,
                           others => False ) ;
    
    Rd_Pattern_Table_Bit : constant := 12 ;
    Rd_Pattern_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Pattern_Table_Bit => True,
                           others => False ) ;
    
    Rd_Text_Font_Table_Bit : constant := 13 ;
    Rd_Text_Font_Table_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Text_Font_Table_Bit => True,
                           others => False ) ;
    
    Rd_Highlight_Incl_Bit : constant := 14 ;
    Rd_Highlight_Incl_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Highlight_Incl_Bit => True,
                           others => False ) ;
    
    Rd_Highlight_Excl_Bit : constant := 15 ;
    Rd_Highlight_Excl_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Highlight_Excl_Bit => True,
                           others => False ) ;
    
    Rd_Invisibility_Incl_Bit : constant := 16 ;
    Rd_Invisibility_Incl_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Invisibility_Incl_Bit => True,
                           others => False ) ;
    
    Rd_Invisibility_Excl_Bit : constant := 17 ;
    Rd_Invisibility_Excl_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Invisibility_Excl_Bit => True,
                           others => False ) ;
    
    Rd_Renderer_State_Bit : constant := 18 ;
    Rd_Renderer_State_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Renderer_State_Bit => True,
                           others => False ) ;
    
    Rd_Hlhsr_Mode_Bit : constant := 19 ;
    Rd_Hlhsr_Mode_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Hlhsr_Mode_Bit => True,
                           others => False ) ;
    
    Rd_Npc_Subvolume_Bit : constant := 20 ;
    Rd_Npc_Subvolume_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Npc_Subvolume_Bit => True,
                           others => False ) ;
    
    Rd_Viewport_Bit : constant := 21 ;
    Rd_Viewport_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Viewport_Bit => True,
                           others => False ) ;
    
    Rd_Clip_List_Bit : constant := 22 ;
    Rd_Clip_List_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Clip_List_Bit => True,
                           others => False ) ;
    
    Rd_Pick_Inclusion_Bit : constant := 23 ;
    Rd_Pick_Inclusion_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Pick_Inclusion_Bit => True,
                           others => False ) ;
    
    Rd_Pick_Exclusion_Bit : constant := 24 ;
    Rd_Pick_Exclusion_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Pick_Exclusion_Bit => True,
                           others => False ) ;
    
    Rd_Pick_Start_Path_Bit : constant := 25 ;
    Rd_Pick_Start_Path_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Pick_Start_Path_Bit => True,
                           others => False ) ;
    
    Rd_Background_Bit : constant := 26 ;
    Rd_Background_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Background_Bit => True,
                           others => False ) ;
    
    Rd_Clear_I_Bit : constant := 27 ;
    Rd_Clear_I_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Clear_I_Bit => True,
                           others => False ) ;
    
    Rd_Clear_Z_Bit : constant := 28 ;
    Rd_Clear_Z_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Clear_Z_Bit => True,
                           others => False ) ;
    
    Rd_Echo_Mode_Bit : constant := 29 ;
    Rd_Echo_Mode_Mask : constant Renderer_Bitmask :=
        Renderer_Bitmask'( Rd_Echo_Mode_Bit => True,
                           others => False ) ;
    
    type Element_Reference_Array ;
    type Element_Path is access Element_Reference_Array ;
    
    type Device_Rectangle_List is access Dc.Rectangular_Region_2_Array ;
    
    type Workstation_Viewport is
    record
        Viewport     : Dc.Rectangular_Region_3 ;
        Use_Drawable : Boolean ;
    end record ;
    
    type Renderer_Values is
    record
        Context                     : Pipeline_Context ;
        Current_Path                : Element_Path ;
        Marker_Bundle               : Marker_Bundle_Lut;
        Text_Bundle                 : Text_Bundle_Lut;
        Line_Bundle                 : Line_Bundle_Lut;
        Interior_Bundle             : Interior_Bundle_Lut;
        Edge_Bundle                 : Edge_Bundle_Lut;
        View_Table                  : View_Lut ;
        Color_Table                 : Color_Lut;
        Depth_Cue_Table             : Depth_Cue_Lut;
        Light_Table                 : Light_Lut;
        Color_Approx_Table          : Color_Approximation_Lut;
        Pattern_Table               : Pattern_Lut;
        Text_Font_Table             : Text_Font_Lut;
        Highlight_Incl              : Name_Set ;
        Highlight_Excl              : Name_Set ;
        Invisibility_Incl           : Name_Set ;
        Invisibility_Excl           : Name_Set ;
        State                       : Renderer_State ;
        Hlhsr                       : Hlhsr_Mode ;
        Npc_Subvolume               : Npc.Rectangular_Region_3 ;
        Viewport                    : Workstation_Viewport ;
        Clip_List                   : Device_Rectangle_List ;
        Pick_Inclusion              : Name_Set ;
        Pick_Exclusion              : Name_Set ;
        Pick_Start_Path             : Element_Path ;
        Background                  : Color_Representation ;
        Clear_Image                 : Boolean ;
        Clear_Z_Buffer              : Boolean ;
        Echo_Mode                   : Pick_Echo_Switch ;
    end record ;
    
    subtype Renderer_Tables_Bitmask is Boolean_Array (0 .. 23) ;
    
    Dyn_Marker_Bundle_Bit : constant := 0 ;
    Dyn_Marker_Bundle_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Marker_Bundle_Bit => True,
                                  others => False ) ;
    
    Dyn_Text_Bundle_Bit : constant := 1 ;
    Dyn_Text_Bundle_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Text_Bundle_Bit => True,
                                  others => False ) ;
    
    Dyn_Line_Bundle_Bit : constant := 2 ;
    Dyn_Line_Bundle_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Line_Bundle_Bit => True,
                                  others => False ) ;
    
    Dyn_Interior_Bundle_Bit : constant := 3 ;
    Dyn_Interior_Bundle_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Interior_Bundle_Bit => True,
                                  others => False ) ;
    
    Dyn_Edge_Bundle_Bit : constant := 4 ;
    Dyn_Edge_Bundle_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Edge_Bundle_Bit => True,
                                  others => False ) ;
    
    Dyn_View_Table_Bit : constant := 5 ;
    Dyn_View_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_View_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Color_Table_Bit : constant := 6 ;
    Dyn_Color_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Color_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Depth_Cue_Table_Bit : constant := 7 ;
    Dyn_Depth_Cue_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Depth_Cue_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Light_Table_Bit : constant := 8 ;
    Dyn_Light_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Light_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Color_Approx_Table_Bit : constant := 9 ;
    Dyn_Color_Approx_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Color_Approx_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Pattern_Table_Bit : constant := 10 ;
    Dyn_Pattern_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Pattern_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Text_Font_Table_Bit : constant := 11 ;
    Dyn_Text_Font_Table_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Text_Font_Table_Bit => True,
                                  others => False ) ;
    
    Dyn_Marker_Bundle_Contents_Bit : constant := 12 ;
    Dyn_Marker_Bundle_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Marker_Bundle_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Text_Bundle_Contents_Bit : constant := 13 ;
    Dyn_Text_Bundle_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Text_Bundle_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Line_Bundle_Contents_Bit : constant := 14 ;
    Dyn_Line_Bundle_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Line_Bundle_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Interior_Bundle_Contents_Bit : constant := 15 ;
    Dyn_Interior_Bundle_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Interior_Bundle_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Edge_Bundle_Contents_Bit : constant := 16 ;
    Dyn_Edge_Bundle_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Edge_Bundle_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_View_Table_Contents_Bit : constant := 17 ;
    Dyn_View_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_View_Table_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Color_Table_Contents_Bit : constant := 18 ;
    Dyn_Color_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Color_Table_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Depth_Cue_Table_Contents_Bit : constant := 19 ;
    Dyn_Depth_Cue_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Depth_Cue_Table_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Light_Table_Contents_Bit : constant := 20 ;
    Dyn_Light_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Light_Table_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Color_Approx_Contents_Bit : constant := 21 ;
    Dyn_Color_Approx_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Color_Approx_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Pattern_Table_Contents_Bit : constant := 22 ;
    Dyn_Pattern_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Pattern_Table_Contents_Bit => True,
                                  others => False ) ;
    
    Dyn_Text_Font_Table_Contents_Bit : constant := 23 ;
    Dyn_Text_Font_Table_Contents_Mask : constant Renderer_Tables_Bitmask :=
        Renderer_Tables_Bitmask'( Dyn_Text_Font_Table_Contents_Bit => True,
                                  others => False ) ;
     
    subtype Renderer_Name_Set_Bitmask is Boolean_Array (0 .. 14) ;
    
    Dyn_Highlight_Name_Set_Bit : constant := 0 ;
    Dyn_Highlight_Name_Set_Mask : constant Renderer_Name_Set_Bitmask :=
        Renderer_Name_Set_Bitmask'( Dyn_Highlight_Name_Set_Bit => True,
                                   others => False ) ;
    
    Dyn_Invisibility_Name_Set_Bit : constant := 1 ;
    Dyn_Invisibility_Name_Set_Mask : constant Renderer_Name_Set_Bitmask :=
        Renderer_Name_Set_Bitmask'( Dyn_Invisibility_Name_Set_Bit => True,
                                   others => False ) ;
    
    Dyn_Pick_Name_Set_Bit : constant := 2 ;
    Dyn_Pick_Name_Set_Mask : constant Renderer_Name_Set_Bitmask :=
        Renderer_Name_Set_Bitmask'( Dyn_Pick_Name_Set_Bit => True,
                                   others => False ) ;
    
    Dyn_Highlight_Name_Set_Contents_Bit : constant := 12 ;
    Dyn_Highlight_Name_Set_Contents_Mask : constant Renderer_Name_Set_Bitmask
        := Renderer_Name_Set_Bitmask'
        ( Dyn_Highlight_Name_Set_Contents_Bit => True,
          others => False ) ;
    
    Dyn_Invisibility_Name_Set_Contents_Bit : constant := 13 ;
    Dyn_Invisibility_Name_Set_Contents_Mask : constant
        Renderer_Name_Set_Bitmask := Renderer_Name_Set_Bitmask'
        ( Dyn_Invisibility_Name_Set_Contents_Bit => True,
          others => False ) ;
    
    Dyn_Pick_Name_Set_Contents_Bit : constant := 14 ;
    Dyn_Pick_Name_Set_Contents_Mask : constant
        Renderer_Name_Set_Bitmask := Renderer_Name_Set_Bitmask'
        ( Dyn_Pick_Name_Set_Contents_Bit => True,
          others => False ) ;
    
    subtype Renderer_Attributes_Bitmask is Boolean_Array (0 .. 4) ;
    
    Dyn_Hlhsr_Mode_Bit : constant := 0 ;
    Dyn_Hlhsr_Mode_Mask : constant Renderer_Attributes_Bitmask :=
        Renderer_Attributes_Bitmask'( Dyn_Hlhsr_Mode_Bit => True,
                                      others => False ) ;
    
    Dyn_Npc_Subvolume_Bit : constant := 1 ;
    Dyn_Npc_Subvolume_Mask : constant Renderer_Attributes_Bitmask :=
        Renderer_Attributes_Bitmask'( Dyn_Npc_Subvolume_Bit => True,
                                      others => False ) ;
    
    Dyn_Viewport_Bit : constant := 2 ;
    Dyn_Viewport_Mask : constant Renderer_Attributes_Bitmask :=
        Renderer_Attributes_Bitmask'( Dyn_Viewport_Bit => True,
                                      others => False ) ;
    
    Dyn_Clip_List_Bit : constant := 3 ;
    Dyn_Clip_List_Mask : constant Renderer_Attributes_Bitmask :=
        Renderer_Attributes_Bitmask'( Dyn_Clip_List_Bit => True,
                                      others => False ) ;
    
    Dyn_Echo_Mode_Bit : constant := 4 ;
    Dyn_Echo_Mode_Mask : constant Renderer_Attributes_Bitmask :=
        Renderer_Attributes_Bitmask'( Dyn_Echo_Mode_Bit => True,
                                      others => False ) ;
    
    -- Structures
    type Structure_Array is array ( X_Natural range <>) of Structure;
    
    type Structure_List ( Size : X_Natural := 0 ) is
    record
        Structures : Structure_Array (1 .. Size) ;
    end record ;
    
    --   
    type Display_Priority is digits Pex_Precision ;
    
    type Structure_Info is
    record
        Id       : Structure ;
        Priority : Display_Priority ;
    end record ;    
    
    type Structure_Info_Array is array ( X_Natural range <>) of
        Structure_Info;
    type Structure_Infos is access Structure_Info_Array ;
    type Structure_Info_List is
    record
        Size  : X_Natural := 0 ;
        Infos : Structure_Infos ;
    end record ;
    
    --
    type Element_Type is
        ( All_Elements,
          Set_Marker_Type,
          Set_Marker_Scale,
          Set_Marker_Color_Index,
          Set_Marker_Color,
          Set_Marker_Bundle_Index,
          Set_Text_Font,
          Set_Text_Precision,
          Set_Character_Expansion,
          Set_Character_Spacing,
          Set_Text_Color_Index,
          Set_Text_Color,
          Set_Character_Height,
          Set_Character_Up_Vector,
          Set_Text_Path,
          Set_Text_Alignment,
          Set_Annotation_Text_Height,
          Set_Annotation_Text_Up_Vector,
          Set_Annotation_Text_Path,
          Set_Annotation_Text_Alignment,
          Set_Annotation_Text_Style,
          Set_Text_Bundle_Index,
          Set_Line_Type,
          Set_Line_Width,
          Set_Line_Color_Index,
          Set_Line_Color,
          Set_Curve_Approximation_Method,
          Set_Polyline_Interpolation_Method,
          Set_Line_Bundle_Index,
          Set_Surface_Interior_Style,
          Set_Surface_Interior_Style_Index,
          Set_Surface_Color_Index,
          Set_Surface_Color,
          Set_Surface_Reflection_Attributes,
          Set_Surface_Reflection_Model,
          Set_Surface_Interpolation_Method,
          Set_Backface_Surface_Interior_Style,
          Set_Backface_Surface_Interior_Style_Index,
          Set_Backface_Surface_Color_Index,
          Set_Backface_Surface_Color,
          Set_Backface_Surface_Reflection_Attributes,
          Set_Backface_Surface_Reflection_Model,
          Set_Backface_Surface_Interpolation_Method,
          Set_Surface_Approximation_Method,
          Set_Facet_Culling_Mode,
          Set_Facet_Distinguish_Flag,
          Set_Pattern_Size,
          Set_Pattern_Reference_Point,
          Set_Pattern_Attributes,
          Set_Interior_Bundle_Index,
          Set_Surface_Edge_Flag,
          Set_Surface_Edge_Type,
          Set_Surface_Edge_Width,
          Set_Surface_Edge_Color_Index,
          Set_Surface_Edge_Color,
          Set_Edge_Bundle_Index,
          Set_Individual_Asf,
          Set_Local_Transform_3D,
          Set_Local_Transform_2D,
          Set_Global_Transform_3D,
          Set_Global_Transform_2D,
          Set_Model_Clip,
          Set_Model_Clip_Volume_3D,
          Set_Model_Clip_Volume_2D,
          Restore_Model_Clip_Volume,
          Set_View_Index,
          Set_Light_Source_State,
          Set_Depth_Cue_Index,
          Set_Pick_Id,
          Set_Hlhsr_Identifier,
          Set_Color_Approximation_Index,
          Set_Rendering_Color_Model,
          Set_Parametric_Surface_Characteristics,
          Add_Names_To_Name_Set,
          Remove_Names_From_Name_Set,
          Execute_Structure,
          Label,
          Application_Data,
          Gse,
          Marker_3D,
          Marker_2D,
          Text_3D,
          Text_2D,
          Annotation_Text_3D,
          Annotation_Text_2D,
          Polyline_3D,
          Polyline_2D,
          Polyline_Set_3D_With_Data,
          Nurb_Curve,
          Fill_Area_3D,
          Fill_Area_2D,
          Fill_Area_3D_With_Data,
          Fill_Area_Set_3D,
          Fill_Area_Set_2D,
          Fill_Area_Set_3D_With_Data,
          Triangle_Strip,
          Quadrilateral_Mesh,
          Set_Of_Fill_Area_Sets,
          Nurb_Surface,
          Cell_Array_3D,
          Cell_Array_2D,
          Extended_Cell_Array_3D,
          Gdp_3D,
          Gdp_2D,
          No_Operation );

    for Element_Type use
      (  0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
        13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
        26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
        39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
        52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
        65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
        78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
        91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104 );
    
    for Element_Type'size use 16;
    --
    type Element_Position is new X_Long_Integer;
    for Element_Position'size use 32;
    
    type Whence_Type is (Beginning, Current, Ending);
    for Whence_Type use (0, 1, 2);
    for Whence_Type'size use 16;
    
    type Element_Type_Set is array (Element_Type) of Boolean ;
    
    type Element_Location is
    record
        Whence   : Whence_Type ;
        Position : Element_Position ;
    end record;
    
    for Element_Location use
    record
        Whence    at 0 * X_32_Bits range 0 .. 15;
        Position  at 1 * X_32_Bits range 0 .. 31;
    end record;
    --
    type Element_Range is 
    record
        Start   : Element_Location ;
        Finish  : Element_Location ;
    end record;
    
    for Element_Range use 
    record
        Start    at 0 * X_32_Bits range 0 .. 63;
        Finish   at 2 * X_32_Bits range 0 .. 63;
    end record;
    --
    type Output_Command_Buffer is private ;
    --
    type Path_Order is (Top_First, Bottom_First);
    for Path_Order use (0, 1);
    for Path_Order'size use 16;
    --
    type Path_Depth is range 0 .. 65_535 ;
    for Path_Depth'size use 32;
    --
    -- Defines a mode for structure editing
    type Edit_Mode is (Insert, Replace);
    for Edit_Mode use (0, 1);
    for Edit_Mode'size use 16;
    --
    type Search_Direction is (Forward, Backward);
    for Search_Direction use (0, 1);
    for Search_Direction'size use 32;
    --
    type Search_Status_Indicator is (Failure, Success);
    for Search_Status_Indicator use (1, 2);
    for Search_Status_Indicator'size use 16;
    --
    type Network_Filter is (All_Structures, No_Cross_Refs);
    for Network_Filter use (0, 1);
    for Network_Filter'size use 16;
    --
    subtype Structure_Info_Mask_Type is Boolean_Array (0 .. 4);

    Edit_Mode_Bit : constant := 0 ;
    Editing_Mode             : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(Edit_Mode_Bit  => True, others => False);
    
    Position_Bit : constant := 1 ;
    Current_Position        : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(Position_Bit   => True, others => False);
    
    Number_Bit : constant := 2 ;
    Number_Of_Elements    : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(Number_Bit  => True, others => False);
    
    Length_Bit : constant := 3 ;
    Total_Length              : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(Length_Bit => True, others => False);
    
    Has_Refs_Bit : constant := 4 ;
    Has_Refs                : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(Has_Refs_Bit  => True, others => False);
    
    All_Items               : constant Structure_Info_Mask_Type :=
        Structure_Info_Mask_Type'(others  => True);

--
    type Element_Info is
    record
        Kind   : Element_Type ;
        Length : X_Natural ;
    end record ;
    
    for Element_Info use
    record
        Kind   at 0 * X_32_Bits range  0 .. 15;
        Length at 0 * X_32_Bits range 16 .. 31;
    end record ;
    
    type Element_Info_Array is array
        (Element_Position range <>) of Element_Info ;
    
    type Element_Info_List ( Size : Element_Position := 0 ) is
    record
        Infos : Element_Info_Array (1 .. Size) ;
    end record ;
--
    type Element_Reference is
    record
        Structure_Id : Structure ;
        Offset       : Element_Position ;
    end record ;
    
    type Element_Reference_Array is array (X_Natural range <>) of
        Element_Reference ;
    
    type Element_Path_Array is array (X_Natural range <>) of Element_Path ;
    
    type Element_Path_List ( Size : X_Natural := 0) is
    record
        Paths : Element_Path_Array (1 .. Size) ;
    end record ;
    
    -- Search Context
    
    subtype Search_Context_Bitmask is Boolean_Array (0 .. 6) ;
    
    Search_Position_Bit : constant := 0 ;
    Search_Position_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Position_Bit => True,
                                 others => False ) ;
    
    Search_Distance_Bit : constant := 1 ;
    Search_Distance_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Distance_Bit => True,
                                 others => False ) ;
    
    Search_Ceiling_Bit : constant := 2 ;
    Search_Ceiling_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Ceiling_Bit => True,
                                 others => False ) ;
    
    Search_Model_Clip_Flag_Bit : constant := 3 ;
    Search_Model_Clip_Flag_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Model_Clip_Flag_Bit => True,
                                 others => False ) ;
    
    Search_Start_Path_Bit : constant := 4 ;
    Search_Start_Path_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Start_Path_Bit => True,
                                 others => False ) ;
    
    Search_Normal_List_Bit : constant := 5 ;
    Search_Normal_List_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Normal_List_Bit => True,
                                 others => False ) ;
    
    Search_Inverted_List_Bit : constant := 6 ;
    Search_Inverted_List_Mask : constant Search_Context_Bitmask :=
        Search_Context_Bitmask'( Search_Inverted_List_Bit => True,
                                 others => False ) ;
    
    type Search_Context_Values is
    record
        Position           : Mc.Point_3 ;
        Distance           : Mc_Type ;
        Ceiling            : Element_Position ;
        Model_Clip_Flag    : Boolean ;
        Start_Path         : Element_Path ;
        Normal_List        : Name_Set_Filter_List ;
        Inverted_List      : Name_Set_Filter_List ;
    end record ;
    
    --
    -- Workstations
    type Buffer_Mode is (Single, Double);
    for Buffer_Mode use (0, 1);
    for Buffer_Mode'size use 16;
    --   
    type Relative_Priority is (Higher, Lower);
    for Relative_Priority use (0, 1);
    for Relative_Priority'size use 16;
    --   
    type View_Update is (Not_Pending, Pending);
    for View_Update use (0, 1);
    for View_Update'size use 16;
    --   
    type Phigs_Workstation_Array is array (X_Natural range <>) of
        Phigs_Workstation ;
    type Phigs_Workstation_List ( Size : X_Natural := 0 ) is
    record
        Stations : Phigs_Workstation_Array (1 .. Size) ;
    end record ;
    
    --   
    type Workstation_Lookup_tables is
    record
        Marker_Bundle       : Marker_Bundle_Lut;
        Text_Bundle         : Text_Bundle_Lut;
        Line_Bundle         : Line_Bundle_Lut;
        Interior_Bundle     : Interior_Bundle_Lut;
        Edge_Bundle         : Edge_Bundle_Lut;
        Color_Table         : Color_Lut;
        Depth_Cue_Table     : Depth_Cue_Lut;
        Light_Table         : Light_Lut;
        Color_Approx_Table  : Color_Approximation_Lut;
        Pattern_Table       : Pattern_Lut;
        Text_Font_Table     : Text_Font_Lut;
    end record;
    
    subtype Workstation_Bitmask is Boolean_Array (0 .. 33) ;
    
    Display_Update_Bit : constant := 0 ;
    Display_Update_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Display_Update_Bit => True,
                              others => False ) ;
    
    Visual_State_Bit : constant := 1 ;
    Visual_State_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Visual_State_Bit => True,
                              others => False ) ;
    
    Display_Surface_Bit : constant := 2 ;
    Display_Surface_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Display_Surface_Bit => True,
                              others => False ) ;
    
    View_Update_Bit : constant := 3 ;
    View_Update_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( View_Update_Bit => True,
                              others => False ) ;
    
    Defined_Views_Bit : constant := 4 ;
    Defined_Views_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Defined_Views_Bit => True,
                              others => False ) ;
    
    Wks_Update_Bit : constant := 5 ;
    Wks_Update_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Wks_Update_Bit => True,
                              others => False ) ;
    
    Req_Npc_Subvolume_Bit : constant := 6 ;
    Req_Npc_Subvolume_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Req_Npc_Subvolume_Bit => True,
                              others => False ) ;
    
    Cur_Npc_Subvolume_Bit : constant := 7 ;
    Cur_Npc_Subvolume_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Cur_Npc_Subvolume_Bit => True,
                              others => False ) ;
    
    Req_Wks_Viewport_Bit : constant := 8 ;
    Req_Wks_Viewport_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Req_Wks_Viewport_Bit => True,
                              others => False ) ;
    
    Cur_Wks_Viewport_Bit : constant := 9 ;
    Cur_Wks_Viewport_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Cur_Wks_Viewport_Bit => True,
                              others => False ) ;
    
    Hlhsr_Update_Bit : constant := 10 ;
    Hlhsr_Update_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Hlhsr_Update_Bit => True,
                              others => False ) ;
    
    Req_Hlhsr_Mode_Bit : constant := 11 ;
    Req_Hlhsr_Mode_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Req_Hlhsr_Mode_Bit => True,
                              others => False ) ;
    
    Cur_Hlhsr_Mode_Bit : constant := 12 ;
    Cur_Hlhsr_Mode_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Cur_Hlhsr_Mode_Bit => True,
                              others => False ) ;
    
    Drawable_Bit : constant := 13 ;
    Drawable_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Drawable_Bit => True,
                              others => False ) ;
    
    Marker_Bundle_Bit : constant := 14 ;
    Marker_Bundle_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Marker_Bundle_Bit => True,
                              others => False ) ;
    
    Text_Bundle_Bit : constant := 15 ;
    Text_Bundle_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Text_Bundle_Bit => True,
                              others => False ) ;
    
    Line_Bundle_Bit : constant := 16 ;
    Line_Bundle_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Line_Bundle_Bit => True,
                              others => False ) ;
    
    Interior_Bundle_Bit : constant := 17 ;
    Interior_Bundle_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Interior_Bundle_Bit => True,
                              others => False ) ;
    
    Edge_Bundle_Bit : constant := 18 ;
    Edge_Bundle_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Edge_Bundle_Bit => True,
                              others => False ) ;
    
    Color_Table_Bit : constant := 19 ;
    Color_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Color_Table_Bit => True,
                              others => False ) ;
    
    Depth_Cue_Table_Bit : constant := 20 ;
    Depth_Cue_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Depth_Cue_Table_Bit => True,
                              others => False ) ;
    
    Light_Table_Bit : constant := 21 ;
    Light_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Light_Table_Bit => True,
                              others => False ) ;
    
    Color_Approx_Table_Bit : constant := 22 ;
    Color_Approx_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Color_Approx_Table_Bit => True,
                              others => False ) ;
    
    Pattern_Table_Bit : constant := 23 ;
    Pattern_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Pattern_Table_Bit => True,
                              others => False ) ;
    
    Text_Font_Table_Bit : constant := 24 ;
    Text_Font_Table_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Text_Font_Table_Bit => True,
                              others => False ) ;
    
    Highlight_Incl_Bit : constant := 25 ;
    Highlight_Incl_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Highlight_Incl_Bit => True,
                              others => False ) ;
    
    Highlight_Excl_Bit : constant := 26 ;
    Highlight_Excl_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Highlight_Excl_Bit => True,
                              others => False ) ;
    
    Invisibility_Incl_Bit : constant := 27 ;
    Invisibility_Incl_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Invisibility_Incl_Bit => True,
                              others => False ) ;
    
    Invisibility_Excl_Bit : constant := 28 ;
    Invisibility_Excl_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Invisibility_Excl_Bit => True,
                              others => False ) ;
    
    Posted_Structures_Bit : constant := 29 ;
    Posted_Structures_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Posted_Structures_Bit => True,
                              others => False ) ;
    
    Num_Priorities_Bit : constant := 30 ;
    Num_Priorities_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Num_Priorities_Bit => True,
                              others => False ) ;
    
    Buffer_Update_Bit : constant := 31 ;
    Buffer_Update_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Buffer_Update_Bit => True,
                              others => False ) ;
    
    Req_Buffer_Mode_Bit : constant := 32 ;
    Req_Buffer_Mode_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Req_Buffer_Mode_Bit => True,
                              others => False ) ;
    
    Cur_Buffer_Mode_Bit : constant := 33 ;
    Cur_Buffer_Mode_Mask : constant Workstation_Bitmask :=
        Workstation_Bitmask'( Cur_Buffer_Mode_Bit => True,
                              others => False ) ;
    
    type Workstation_Values is
    record
        Ws_Display_Update     : Display_Update_Method ;
        Ws_Visual_State       : Visual_State ;
        Ws_Display_Surface    : Display_State ;
        Ws_View_Update        : View_Update ;
        Ws_Defined_Views      : View_List ;
        Ws_Wks_Update         : View_Update ;
        Ws_Req_Npc_Subvolume  : Npc.Rectangular_region_3 ;
        Ws_Cur_Npc_Subvolume  : Npc.Rectangular_region_3 ;
        Ws_Req_Wks_Viewport   : Workstation_Viewport ;
        Ws_Cur_Wks_Viewport   : Workstation_Viewport ;
        Ws_Hlhsr_Update       : View_Update ;
        Ws_Req_Hlhsr_Mode     : Hlhsr_Mode ;
        Ws_Cur_Hlhsr_Mode     : Hlhsr_Mode ;
        Ws_Drawable           : Drawable ;
        Ws_Marker_Bundle      : Marker_Bundle_Lut ;
        Ws_Text_Bundle        : Text_Bundle_Lut ;
        Ws_Line_Bundle        : Line_Bundle_Lut ;
        Ws_Interior_Bundle    : Interior_Bundle_Lut ;
        Ws_Edge_Bundle        : Edge_Bundle_Lut ;
        Ws_Color_Table        : Color_Lut ;
        Ws_Depth_Cue_Table    : Depth_Cue_Lut ;
        Ws_Light_Table        : Light_Lut ;
        Ws_Color_Approx_Table : Color_Approximation_Lut ;
        Ws_Pattern_Table      : Pattern_Lut ;
        Ws_Text_Font_Table    : Text_Font_Lut ;
        Ws_Highlight_Incl     : Name_Set ;
        Ws_Highlight_Excl     : Name_Set ;
        Ws_Invisibility_Incl  : Name_Set ;
        Ws_Invisibility_Excl  : Name_Set ;
        Ws_Posted_Structures  : Structure_Info_List ;
        Ws_Num_Priorities     : X_Natural ;
        Ws_Buffer_Update      : View_Update ;
        Ws_Req_Buffer_Mode    : Buffer_Mode ;
        Ws_Cur_Buffer_Mode    : Buffer_Mode ;
    end record ;
    
    type Dynamics is
        ( View_Rep,         Marker_Bundle,        Text_Bundle,
          Line_Bundle,      Interior_Bundle,      Edge_Bundle,
          Color_Table,      Pattern_Table,        Wks_Transform,
          Highlight_Filter, Invisibility_Filter,  Hlhsr_Mode_Dyn,
          Structure_Modify, Post_Structure,       Unpost_Structure,
          Delete_Structure, Reference_Modify,     Buffer_Modify,
          Light_Table,      Depth_Cue_Table,      Color_Approx_Table ) ;
    
    type Dynamic_Modification is (Imm, Irg, Cbs) ;
    for Dynamic_Modification'size use 8 ;
    
    type Dynamics_Info is array (Dynamics) of Dynamic_Modification ;
    pragma Pack (Dynamics_Info) ;
    
    type Projection_Type is ( Parallel, Perspective ) ;
    
    type View_Representation is
    record
        Index           : View_Index; 
        Clipping_Flags  : Clipping_Indicator;
        Clipping_Limits : NPC.Rectangular_Region_3;
        Orientation     : Transformation_Matrix;
        Mapping         : Transformation_Matrix;
    end record;
    
    for View_Representation use
    record
        Index               at  0 * X_32_Bits range  0 .. 15;
        Clipping_Flags      at  1 * X_32_Bits range  0 ..  2;
        Clipping_Limits     at  2 * X_32_Bits range  0 .. (6*Fp_Size) -1;
        Orientation    at  (2 +  6*Fp) * X_32_Bits range  0 .. (16*Fp_Size)-1;
        Mapping        at  (2 + 22*Fp) * X_32_Bits range  0 .. (16*Fp_Size)-1;
    end record;
    
    -- Picking

    subtype Pick_Device_Bitmask is Boolean_Array (0 .. 8) ;
    
    Pick_Status_Bit : constant := 0 ;
    Pick_Status_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Status_Bit => True,
                              others => False ) ;
    
    Pick_Path_Bit : constant := 1 ;
    Pick_Path_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Path_Bit => True,
                              others => False ) ;
    
    Pick_Path_Order_Bit : constant := 2 ;
    Pick_Path_Order_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Path_Order_Bit => True,
                              others => False ) ;
    
    Pick_Incl_Bit : constant := 3 ;
    Pick_Incl_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Incl_Bit => True,
                              others => False ) ;
    
    Pick_Excl_Bit : constant := 4 ;
    Pick_Excl_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Excl_Bit => True,
                              others => False ) ;
    
    Pick_Data_Rec_Bit : constant := 5 ;
    Pick_Data_Rec_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Data_Rec_Bit => True,
                              others => False ) ;
    
    Pick_Prompt_Echo_Type_Bit : constant := 6 ;
    Pick_Prompt_Echo_Type_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Prompt_Echo_Type_Bit => True,
                              others => False ) ;
    
    Pick_Echo_Volume_Bit : constant := 7 ;
    Pick_Echo_Volume_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Echo_Volume_Bit => True,
                              others => False ) ;
    
    Pick_Echo_Switch_Bit : constant := 8 ;
    Pick_Echo_Switch_Mask : constant Pick_Device_Bitmask :=
        Pick_Device_Bitmask'( Pick_Echo_Switch_Bit => True,
                              others => False ) ;
    
    type Pick_Element_Reference is
    record
        Structure_Id : Structure ;
        Offset       : Element_Position ;
        Picked_Id    : Pick_Id ;
    end record ;
    
    type Pick_Element_Reference_Array is array (X_Natural range <>) of
        Pick_Element_Reference ;
    
    type Pick_Path is access Pick_Element_Reference_Array ;
    
    type Pick_Path_Array is array (X_Natural range <>) of Pick_Path ;
    
    type Pick_Path_List is access Pick_Path_Array ;
    
    type User_Pick_Element_Reference is
    record
        Structure_Id : User_Structure ;
        Offset       : Element_Position ;
        Picked_Id    : Pick_Id ;
    end record ;
    
    type User_Pick_Element_Reference_Array is array (X_Natural range <>) of
        User_Pick_Element_Reference ;
    
    type User_Pick_Path is access User_Pick_Element_Reference_Array ;
    
    type User_Pick_Path_Array is array (X_Natural range <>) of User_Pick_Path;
    
    type User_Pick_Path_List is access User_Pick_Path_Array ;
    
    type Pick_Data_Type is (DC_Hit_Box, NPC_Hit_Volume, Specific) ;
    
    type Pick_Data_Record (Size : Short_Natural := 0) is
    record
        Bytes : Byte_Array (1 .. Size) ;
    end record ;
    
    type Pick_Data (Kind : Pick_Data_Type := DC_Hit_Box) is
    record
        case Kind is
        when Dc_Hit_Box =>
            Position : Dc.Point_2;
            Distance : Dc_Distance ;
            
        when Npc_Hit_Volume =>
            Volume : Npc.Rectangular_Region_3 ;
            
        when Specific =>
            Data : Pick_Data_Record ;
        end case ;
    end record ;
    
    type Pick_Device_Values is
    record
        Status            : Pick_Status ;
        Path              : Pick_Path ;
        Path_Order        : Pick_Path_Order ;
        Include           : Name_Set ;
        Exclude           : Name_Set ;
        Data              : Pick_Data_Record ;
        Prompt_Echo       : Prompt_Echo_Type ;
        Echo_Volume       : Workstation_Viewport ;
        Echo_Switch       : Pick_Echo_Switch ;
    end record ;
    
    subtype Pick_Measure_Bitmask is Boolean_Array (0 .. 1) ;
    
    Measure_Status_Bit : constant := 0 ;
    Measure_Status_Mask : constant Pick_Measure_Bitmask :=
        Pick_Measure_Bitmask'( Measure_Status_Bit => True,
                               others => False ) ;
    
    Measure_Path_Bit : constant := 1 ;
    Measure_Path_Mask : constant Pick_Measure_Bitmask :=
        Pick_Measure_Bitmask'( Measure_Path_Bit => True,
                               others => False ) ;
    
    type Pick_Measure_Values is
    record
        Status      : Pick_Status ;
        Path        : Pick_Path ;
    end record ;
    
    type Implementation_Constant_Name is private ;
    
    Dithering_Supported_Name        : constant Implementation_Constant_Name ;
    Max_Edge_Width_Name             : constant Implementation_Constant_Name ;
    Max_Line_Width_Name             : constant Implementation_Constant_Name ;
    Max_Marker_Size_Name            : constant Implementation_Constant_Name ;
    Max_Model_Clip_Planes_Name      : constant Implementation_Constant_Name ;
    Max_Name_Set_Names_Name         : constant Implementation_Constant_Name ;
    Max_Non_Ambient_Lights_Name     : constant Implementation_Constant_Name ;
    Max_Nurb_Order_Name             : constant Implementation_Constant_Name ;
    Max_Trim_Curve_Order_Name       : constant Implementation_Constant_Name ;
    Min_Edge_Width_Name             : constant Implementation_Constant_Name ;
    Min_Line_Width_Name             : constant Implementation_Constant_Name ;
    Min_Marker_Size_Name            : constant Implementation_Constant_Name ;
    Nominal_Edge_Width_Name         : constant Implementation_Constant_Name ;
    Nominal_Line_Width_Name         : constant Implementation_Constant_Name ;
    Nominal_Marker_Size_Name        : constant Implementation_Constant_Name ;
    Num_Supported_Edge_Widths_Name  : constant Implementation_Constant_Name ;
    Num_Supported_Line_Widths_Name  : constant Implementation_Constant_Name ;
    Num_Supported_Marker_Sizes_Name : constant Implementation_Constant_Name ;
    Best_Color_Approximation_Name   : constant Implementation_Constant_Name ;
    Transparency_Supported_Name     : constant Implementation_Constant_Name ;
    Double_Buffering_Supported_Name : constant Implementation_Constant_Name ;
    Chromaticity_Red_U_Name         : constant Implementation_Constant_Name ;
    Chromaticity_Red_V_Name         : constant Implementation_Constant_Name ;
    Luminance_Red_Name              : constant Implementation_Constant_Name ;
    Chromaticity_Green_U_Name       : constant Implementation_Constant_Name ;
    Chromaticity_Green_V_Name       : constant Implementation_Constant_Name ;
    Luminance_Green_Name            : constant Implementation_Constant_Name ;
    Chromaticity_Blue_U_Name        : constant Implementation_Constant_Name ;
    Chromaticity_Blue_V_Name        : constant Implementation_Constant_Name ;
    Luminance_Blue_Name             : constant Implementation_Constant_Name ;
    Chromaticity_White_U_Name       : constant Implementation_Constant_Name ;
    Chromaticity_White_V_Name       : constant Implementation_Constant_Name ;
    Luminance_White_Name            : constant Implementation_Constant_Name ;
    Max_Hits_Event_Supported_Name   : constant Implementation_Constant_Name ;
    
    -- Constants for Color_Best_Approximation 
    type Color_Best_Choice is (Any_Values, Powers_Of_2) ;

    type Implementation_Constant_Values is
    record
        Dithering_Supported        : Boolean ;
        Max_Edge_Width             : Pixels ;
        Max_Line_Width             : Pixels ;
        Max_Marker_Size            : Pixels ;
        Max_Model_Clip_Planes      : X_Natural ;
        Max_Name_Set_Names         : X_Natural ;
        Max_Non_Ambient_Lights     : X_Natural ;
        Max_Nurb_Order             : Nurb_Order ;
        Max_Trim_Curve_Order       : Trim_Order ;
        Min_Edge_Width             : Pixels ;
        Min_Line_Width             : Pixels ;
        Min_Marker_Size            : Pixels ;
        Nominal_Edge_Width         : Pixels ;
        Nominal_Line_Width         : Pixels ;
        Nominal_Marker_Size        : Pixels ;
        Num_Supported_Edge_Widths  : X_Natural ;
        Num_Supported_Line_Widths  : X_Natural ;
        Num_Supported_Marker_Sizes : X_Natural ;
        Best_Color_Approximation   : Color_Best_Choice ;
        Transparency_Supported     : Boolean ;
        Double_Buffering_Supported : Boolean ;
        Chromaticity_Red_U         : Color_Float_Value ;
        Chromaticity_Red_V         : Color_Float_Value ;
        Luminance_Red              : Color_Float_Value ;
        Chromaticity_Green_U       : Color_Float_Value ;
        Chromaticity_Green_V       : Color_Float_Value ;
        Luminance_Green            : Color_Float_Value ;
        Chromaticity_Blue_U        : Color_Float_Value ;
        Chromaticity_Blue_V        : Color_Float_Value ;
        Luminance_Blue             : Color_Float_Value ;
        Chromaticity_White_U       : Color_Float_Value ;
        Chromaticity_White_V       : Color_Float_Value ;
        Luminance_White            : Color_Float_Value ;
        Max_Hits_Event_Supported   : Boolean ;
    end record ;
    
    -- Fonts
    
    -- per character font metric information.

    type Character_Record is
    record
        Left_Bearing  : Pixels := 0; -- origin to left edge of raster
        Right_Bearing : Pixels := 0; -- origin to right edge of raster
        Width         : Pixels := 0; -- advance to next char's origin
        Ascent        : Pixels := 0; -- baseline to top edge of raster
        Descent       : Pixels := 0; -- baseline to bottom edge of raster
        Attributes    : Pixels := 0; -- per char flags (not predefined)
    end record;
    for Character_Record use
    record
        Left_Bearing   at 0 * X_32_Bits range  0 .. 15 ;
        Right_Bearing  at 0 * X_32_Bits range 16 .. 31 ;
        Width          at 1 * X_32_Bits range  0 .. 15 ;
        Ascent         at 1 * X_32_Bits range 16 .. 31 ;
        Descent        at 2 * X_32_Bits range  0 .. 15 ;
        Attributes     at 2 * X_32_Bits range 16 .. 31 ;
    end record;

    subtype Pex_Character is Long_Character ;
    
    type Font_Record is
    record
        Font_Id       : Font;  -- Font id for this font
        First_Glyph   : Pex_Character;
        Last_Glyph    : Pex_Character;
        Default_Glyph : Pex_Character; -- char to print for undefined chars
        All_Exist    : Boolean; -- flag if all chars have non-zero size
        Stroke_Font  : Boolean; -- flag if it is a stroke font
        Properties   : Font_Property_List;-- list of additional properties
    end record;

    type Font_Record_Array is array (X_Natural range <>) of Font_Record ;
    type Font_Record_List ( Length : X_Short_Natural := 0 ) is
    record
        Font_Records : Font_Record_Array ( 1 .. Length ) ;
    end record ;

    type Pex_Error is
        ( Bad_Color_Type,     Bad_Renderer_State,   Bad_Floating_Point_Format,
          Bad_Label,          Bad_Lookup_Table,     Bad_Name_Set,
          Bad_Path,           Bad_Pex_Font,         Bad_Phigs_Workstation,
          Bad_Pick_Measure,   Bad_Pipeline_Context, Bad_Renderer,
          Bad_Search_Context, Bad_Structure,        Bad_Output_Command ) ;
    
    type Pex_Error_Record ( Kind : Pex_Error := Bad_Color_Type ) is
    record
        Minor : Request_Type ;
        case Kind is
        when Bad_Color_Type            => Color        : Color_Type ;
        when Bad_Renderer_State |
             Bad_Renderer              => Renderer_Id  : Renderer ;
        when Bad_Floating_Point_Format => Format       : Floating_Point_Format;
        when Bad_Label                 => Label        : Label_Id ;
        when Bad_Lookup_Table          => Table        : Resource ;
        when Bad_Name_Set              => Set          : Name_Set ;
        when Bad_Path |
             Bad_Structure             => Structure_Id : Structure ;
        when Bad_Pex_Font              => Font_Id      : Font ;
        when Bad_Phigs_Workstation     => Station      : Phigs_Workstation ;
        when Bad_Pick_Measure          => Measure      : Pick_Measure ;
        when Bad_Pipeline_Context      => Pipe_Context : Pipeline_Context ;
        when Bad_Search_Context        => S_Context    : Search_Context ;
        when Bad_Output_Command        =>
            Resource_Id  : Resource ;
            Command      : Element_Type ;
            Processed    : X_Natural ;
        end case ;
    end record ;
    
    type Pex_Event_Type is ( Pex_Error_Event, Max_Hits_Event ) ;
    
    type Max_Hits_Event_Record is
    record
        Renderer_Id : Renderer ;
    end record ;
    
    type Pex_Event (Kind : Pex_Event_Type := Max_Hits_Event) is
    record
        Serial      : X_Long_Integer;
        -- of last request processed by server
        Send_Event  : Boolean;
        -- true if this came from a Send_Event request
        Display_Id  : Display; -- Display the event was read from
        case Kind is
        when Pex_Error_Event => Error   : Pex_Error_Record ;
        when Max_Hits_Event  => Hitting : Max_Hits_Event_Record ;
        end case ;
    end record ;
    
    type Vendor_Id is range 0 .. 32767 ;
    
    type Escape_Id (Specific : Boolean := False) is
    record
        Request : Escape_Request ;
        case Specific is
        when False => null ;
        when True  => Vendor : Vendor_Id ;
        end case ;
    end record ;
    
private
    type Enumerated_Value is range -32_768 .. 32_767 ;
    
    type Implementation_Constant_Name is range -32_768 .. 34 ;
    Dithering_Supported_Name        : constant
        Implementation_Constant_Name :=  1 ;
    Max_Edge_Width_Name             : constant
        Implementation_Constant_Name :=  2 ;
    Max_Line_Width_Name             : constant
        Implementation_Constant_Name :=  3 ;
    Max_Marker_Size_Name            : constant
        Implementation_Constant_Name :=  4 ;
    Max_Model_Clip_Planes_Name      : constant
        Implementation_Constant_Name :=  5 ;
    Max_Name_Set_Names_Name         : constant
        Implementation_Constant_Name :=  6 ;
    Max_Non_Ambient_Lights_Name     : constant
        Implementation_Constant_Name :=  7 ;
    Max_Nurb_Order_Name             : constant
        Implementation_Constant_Name :=  8 ;
    Max_Trim_Curve_Order_Name       : constant
        Implementation_Constant_Name :=  9 ;
    Min_Edge_Width_Name             : constant
        Implementation_Constant_Name := 10 ;
    Min_Line_Width_Name             : constant
        Implementation_Constant_Name := 11 ;
    Min_Marker_Size_Name            : constant
        Implementation_Constant_Name := 12 ;
    Nominal_Edge_Width_Name         : constant
        Implementation_Constant_Name := 13 ;
    Nominal_Line_Width_Name         : constant
        Implementation_Constant_Name := 14 ;
    Nominal_Marker_Size_Name        : constant
        Implementation_Constant_Name := 15 ;
    Num_Supported_Edge_Widths_Name  : constant
        Implementation_Constant_Name := 16 ;
    Num_Supported_Line_Widths_Name  : constant
        Implementation_Constant_Name := 17 ;
    Num_Supported_Marker_Sizes_Name : constant
        Implementation_Constant_Name := 18 ;
    Best_Color_Approximation_Name   : constant
        Implementation_Constant_Name := 19 ;
    Transparency_Supported_Name     : constant
        Implementation_Constant_Name := 20 ;
    Double_Buffering_Supported_Name : constant
        Implementation_Constant_Name := 21 ;
    Chromaticity_Red_U_Name         : constant
        Implementation_Constant_Name := 22 ;
    Chromaticity_Red_V_Name         : constant
        Implementation_Constant_Name := 23 ;
    Luminance_Red_Name              : constant
        Implementation_Constant_Name := 24 ;
    Chromaticity_Green_U_Name       : constant
        Implementation_Constant_Name := 25 ;
    Chromaticity_Green_V_Name       : constant
        Implementation_Constant_Name := 26 ;
    Luminance_Green_Name            : constant
        Implementation_Constant_Name := 27 ;
    Chromaticity_Blue_U_Name        : constant
        Implementation_Constant_Name := 28 ;
    Chromaticity_Blue_V_Name        : constant
        Implementation_Constant_Name := 29 ;
    Luminance_Blue_Name             : constant
        Implementation_Constant_Name := 30 ;
    Chromaticity_White_U_Name       : constant
        Implementation_Constant_Name := 31 ;
    Chromaticity_White_V_Name       : constant
        Implementation_Constant_Name := 32 ;
    Luminance_White_Name            : constant
        Implementation_Constant_Name := 33 ;
    Max_Hits_Event_Supported_Name   : constant
        Implementation_Constant_Name := 34 ;
    
    type Enumerated_Types is range -32_768 .. 30 ;
    for Enumerated_Types'size use 16 ;
    pragma Pack (Enumerated_Types_Array) ;
    
    Enum_Marker_Type                   : constant Enumerated_Types :=  1 ;
    Enum_A_Text_Style                  : constant Enumerated_Types :=  2 ;
    Enum_Interior_Style                : constant Enumerated_Types :=  3 ;
    Enum_Hatch_Style                   : constant Enumerated_Types :=  4 ;
    Enum_Line_Type                     : constant Enumerated_Types :=  5 ;
    Enum_Surface_Edge_Type             : constant Enumerated_Types :=  6 ;
    Enum_Pick_Device_Type              : constant Enumerated_Types :=  7 ;
    Enum_Polyline_Interpolation_Method : constant Enumerated_Types :=  8 ;
    Enum_Curve_Approx_Method           : constant Enumerated_Types :=  9 ;
    Enum_Reflection_Model              : constant Enumerated_Types := 10 ;
    Enum_Surface_Interpolation_Method  : constant Enumerated_Types := 11 ;
    Enum_Surface_Approx_Method         : constant Enumerated_Types := 12 ;
    Enum_Model_Clip_Operator           : constant Enumerated_Types := 13 ;
    Enum_Light_Type                    : constant Enumerated_Types := 14 ;
    Enum_Color_Type                    : constant Enumerated_Types := 15 ;
    Enum_Float_Format                  : constant Enumerated_Types := 16 ;
    Enum_Hlhsr_Mode                    : constant Enumerated_Types := 17 ;
    Enum_Prompt_Echo_Type              : constant Enumerated_Types := 18 ;
    Enum_Display_Update_Mode           : constant Enumerated_Types := 19 ;
    Enum_Color_Approx_Type             : constant Enumerated_Types := 20 ;
    Enum_Color_Approx_Model            : constant Enumerated_Types := 21 ;
    Enum_GDP                           : constant Enumerated_Types := 22 ;
    Enum_GDP3                          : constant Enumerated_Types := 23 ;
    Enum_GSE                           : constant Enumerated_Types := 24 ;
    Enum_Trim_Curve_Approx_Method      : constant Enumerated_Types := 25 ;
    Enum_Rendering_Color_Model         : constant Enumerated_Types := 26 ;
    Enum_Para_Surf_Characteristics     : constant Enumerated_Types := 27 ;
    Enum_Escape                        : constant Enumerated_Types := 28 ;
    Enum_Pick_One_Method               : constant Enumerated_Types := 29 ;
    Enum_Pick_All_Method               : constant Enumerated_Types := 30 ;
    
    type Marker_Type is range -32_768 .. 5 ;
    Marker_Dot              : constant Marker_Type := 1 ;
    Marker_Cross            : constant Marker_Type := 2 ;
    Marker_Asterisk         : constant Marker_Type := 3 ;
    Marker_Circle           : constant Marker_Type := 4 ;
    Marker_X                : constant Marker_Type := 5 ;
    
    type Annotation_Style is range -32_768 .. 2 ;
    Annotation_Not_Connected    : constant Annotation_Style := 1 ;
    Annotation_Connected        : constant Annotation_Style := 2 ;
    
    type Hatch_Style is range -32_768 .. 0 ;
    
    type Line_Type is range -32_768 .. 4 ;
    Line_Type_Solid         : constant Line_Type := 1 ;
    Line_Type_Dashed        : constant Line_Type := 2 ;
    Line_Type_Dotted        : constant Line_Type := 3 ;
    Line_Type_Dash_Dot      : constant Line_Type := 4 ;
    
    type Surface_Edge_Type is range -32_768 .. 4 ;
    Surface_Edge_Solid      : constant Surface_Edge_Type := 1 ;
    Surface_Edge_Dashed     : constant Surface_Edge_Type := 2 ;
    Surface_Edge_Dotted     : constant Surface_Edge_Type := 3 ;
    Surface_Edge_Dash_Dot   : constant Surface_Edge_Type := 4 ;
    
    type Pick_Device_Type is range -32_768 .. 2 ;
    Pick_Device_DC_Hit_Box         : constant Pick_Device_Type := 1 ;
    Pick_Device_NPC_Hit_Volume     : constant Pick_Device_Type := 2 ;
    
    type Polyline_Interpolation_Method is range -32_768 .. 2 ;
    Polyline_Interpolation_None    : constant Polyline_Interpolation_Method
        := 1 ;
    Polyline_Interpolation_Color   : constant Polyline_Interpolation_Method
        := 2 ;
    
    type Approximation_Method is range -32_768 .. 11 ;
    Approx_Imp_Dep                    : constant Approximation_Method := 1 ;
    Approx_Constant_Between_Knots     : constant Approximation_Method := 2 ;
    Approx_Wcs_Chordal_Size           : constant Approximation_Method := 3 ;
    Approx_Npc_Chordal_Size           : constant Approximation_Method := 4 ;
    Approx_Dc_Chordal_Size            : constant Approximation_Method := 5 ;
    Curve_Approx_Wcs_Chordal_Dev      : constant Approximation_Method := 6 ;
    Curve_Approx_Npc_Chordal_Dev      : constant Approximation_Method := 7 ;
    Curve_Approx_Dc_Chordal_Dev       : constant Approximation_Method := 8 ;
    Surface_Approx_Wcs_Planar_Dev     : constant Approximation_Method := 6 ;
    Surface_Approx_Npc_Planar_Dev     : constant Approximation_Method := 7 ;
    Surface_Approx_Dc_Planar_Dev      : constant Approximation_Method := 8 ;
    Approx_Wcs_Relative               : constant Approximation_Method := 9 ;
    Approx_Npc_Relative               : constant Approximation_Method := 10 ;
    Approx_Dc_Relative                : constant Approximation_Method := 11 ;
    
    type Trim_Curve_Approx_Method is range -32_768 .. 2 ;
    Trim_Approx_Imp_Dep           : constant Trim_Curve_Approx_Method := 1 ;
    Trim_Approx_Constant_Between_Knots : constant Trim_Curve_Approx_Method := 2 ;
    
    type Reflection_Model is range -32_768 .. 4 ;
    Reflection_No_Shading   : constant Reflection_Model := 1 ;
    Reflection_Ambient      : constant Reflection_Model := 2 ;
    Reflection_Diffuse      : constant Reflection_Model := 3 ;
    Reflection_Specular     : constant Reflection_Model := 4 ;
    
    type Surface_Interpolation_Method is range -32_768 .. 4 ;
    Surface_Interpolation_None             : constant
        Surface_Interpolation_Method := 1 ;
    Surface_Interpolation_Color            : constant
        Surface_Interpolation_Method := 2 ;
    Surface_Interpolation_Dot_Product      : constant
        Surface_Interpolation_Method := 3 ;
    Surface_Interpolation_Normal           : constant
        Surface_Interpolation_Method := 4 ;
    
    type Model_Clip_Operator is range -32_768 .. 2 ;
    Model_Clip_Replace              : constant Model_Clip_Operator := 1 ;
    Model_Clip_Intersection         : constant Model_Clip_Operator := 2 ;
    
    type Light_Type is range -32_768 .. 4 ;
    Light_Ambient                   : constant Light_Type := 1 ;
    Light_Wcs_Vector                : constant Light_Type := 2 ;
    Light_Wcs_Point                 : constant Light_Type := 3 ;
    Light_Wcs_Spot                  : constant Light_Type := 4 ;
    
    type Color_Type is range -32_768 .. 6 ;
    Indexed_Color      : constant Color_Type := 0 ;
    Rgb_Float_Color    : constant Color_Type := 1 ;
    Cie_Float_Color    : constant Color_Type := 2 ;
    Hsv_Float_Color    : constant Color_Type := 3 ;
    Hls_Float_Color    : constant Color_Type := 4 ;
    Rgb_8_Color        : constant Color_Type := 5 ;
    Rgb_16_Color       : constant Color_Type := 6 ;
    Max_Color          : constant Color_Type := 6 ;
    
    type Floating_Point_Format is range -32_768 .. 4 ;
    Ieee_754_32             : constant Floating_Point_Format := 1 ;
    Dec_F_Floating          : constant Floating_Point_Format := 2 ;
    Ieee_754_64             : constant Floating_Point_Format := 3 ;
    Dec_D_Floating          : constant Floating_Point_Format := 4 ;
    
    type Hlhsr_Mode is range -32_768 .. 6 ;
    Hlhsr_Off               : constant Hlhsr_Mode := 1 ;
    Hlhsr_Z_Buffer          : constant Hlhsr_Mode := 2 ;
    Hlhsr_Painters          : constant Hlhsr_Mode := 3 ;
    Hlhsr_Scanline          : constant Hlhsr_Mode := 4 ;
    Hlhsr_Hidden_Line_Only  : constant Hlhsr_Mode := 5 ;
    Hlhsr_Z_Buffer_Id       : constant Hlhsr_Mode := 6 ;
    
    type Prompt_Echo_Type is range -32_768 .. 3 ;
    Echo_Primitive  : constant Prompt_Echo_Type := 1 ;
    Echo_Structure  : constant Prompt_Echo_Type := 2 ;
    Echo_Network    : constant Prompt_Echo_Type := 3 ;
    
    type Display_Update_Method is range -32_768 .. 5 ;
    Visualize_Each          : constant Display_Update_Method := 1 ;
    Visualize_Easy          : constant Display_Update_Method := 2 ;
    Visualize_None          : constant Display_Update_Method := 3 ;
    Simulate_Some           : constant Display_Update_Method := 4 ;
    Visualize_Whenever      : constant Display_Update_Method := 5 ;
    
    type Color_Approximation_Type is range -32_768 .. 2 ;
    Color_Space          : constant Color_Approximation_Type := 1 ;
    Color_Range          : constant Color_Approximation_Type := 2 ;
    
    type Color_Approximation_Method is range -32_768 .. 5 ;
    Color_Approx_RGB      : constant Color_Approximation_Method := 1 ;
    Color_Approx_CIE      : constant Color_Approximation_Method := 2 ;
    Color_Approx_HSV      : constant Color_Approximation_Method := 3 ;
    Color_Approx_HLS      : constant Color_Approximation_Method := 4 ;
    Color_Approx_YIQ      : constant Color_Approximation_Method := 5 ;
    
    type Rendering_Color_Model is range -32_768 .. 4 ;
    Rdr_Color_Model_Imp_Dep : constant Rendering_Color_Model := 0 ;
    Rdr_Color_Model_RGB    : constant Rendering_Color_Model := 1 ;
    Rdr_Color_Model_CIE    : constant Rendering_Color_Model := 2 ;
    Rdr_Color_Model_HSV    : constant Rendering_Color_Model := 3 ;
    Rdr_Color_Model_HLS    : constant Rendering_Color_Model := 4 ;
    
    type Parametric_Surface_Characteristics is range -32_768 .. 5 ;
    PSC_None            : constant Parametric_Surface_Characteristics := 1 ;
    PSC_Imp_Dep         : constant Parametric_Surface_Characteristics := 2 ;
    PSC_Iso_Curves      : constant Parametric_Surface_Characteristics := 3 ;
    PSC_Mc_Level_Curves : constant Parametric_Surface_Characteristics := 4 ;
    PSC_Wc_Level_Curves : constant Parametric_Surface_Characteristics := 5 ;
    
    type Isoparametric_Curves is range -32_768 .. 1 ;
    Uniform_Placement            : constant Isoparametric_Curves := 0 ;
    Non_Uniform_Placement        : constant Isoparametric_Curves := 1 ;
    
    type Escape_Request is range -32_768 .. 1 ;
    Set_Echo_Color      : constant Escape_Request := 1 ;
    
    type Pick_One_Method is range -32_768 .. 4 ;
    Last_Picked      : constant Pick_One_Method := 1 ;
    Closest_Z        : constant Pick_One_Method := 2 ;
    Any_Visible      : constant Pick_One_Method := 3 ;
    Visible_Closest  : constant Pick_One_Method := 4;
    
    type Pick_All_Method is range -32_768 .. 2 ;
    Retain_All      : constant Pick_All_Method := 1 ;
    Retain_Visible  : constant Pick_All_Method := 2 ;
    
    type Interior_Style_Index is range -32_768 .. 32_767 ;
    
    type Lookup_Table ( Kind : Lookup_Table_Type) is 
    record
      Lut_XId  : Resource := Null_Resource ;-- X resource Id
    end record;
    
    Null_Marker_Bundle_Lut             : constant Marker_Bundle_Lut :=
        ( Kind => Marker_Bundle,         Lut_Xid => Null_Resource ) ;
    Null_Text_Bundle_Lut               : constant Text_Bundle_Lut :=
        ( Kind => Text_Bundle,           Lut_Xid => Null_Resource) ;
    Null_Line_Bundle_Lut               : constant Line_Bundle_Lut :=
        ( Kind => Line_Bundle,           Lut_Xid => Null_Resource) ;
    Null_Interior_Bundle_Lut           : constant Interior_Bundle_Lut :=
        ( Kind => Interior_Bundle,       Lut_Xid => Null_Resource) ;
    Null_Edge_Bundle_Lut               : constant Edge_Bundle_Lut :=
        ( Kind => Edge_Bundle,           Lut_Xid => Null_Resource) ;
    Null_Color_Lut                     : constant Color_Lut :=
        ( Kind => Color,                 Lut_Xid => Null_Resource) ;
    Null_View_Lut                      : constant View_Lut :=
        ( Kind => View,                  Lut_Xid => Null_Resource) ;
    Null_Light_Lut                     : constant Light_Lut :=
        ( Kind => Light,                 Lut_Xid => Null_Resource) ;
    Null_Depth_Cue_Lut                 : constant Depth_Cue_Lut :=
        ( Kind => Depth_Cue,             Lut_Xid => Null_Resource) ;
    Null_Color_Approximation_Lut       : constant Color_Approximation_Lut :=
        ( Kind => Color_Approximation,   Lut_Xid => Null_Resource) ;
    Null_Pattern_Lut                   : constant Pattern_Lut :=
        ( Kind => Pattern,               Lut_Xid => Null_Resource) ;
    Null_Text_Font_Lut                 : constant Text_Font_Lut :=
        ( Kind => Text_Font,             Lut_Xid => Null_Resource) ;

    --
    type Null_Record is
    record
        null ;
    end record ;
    
    type Generic_Pointer is access Null_Record ;
    
    Req_Size : constant := 4 ;
    
    type Storage_Index is range 1 .. 4 ;
    
    type Output_Command_Buffer is
    record
        Index : Storage_Index := 1 ;
        Size  : X_Natural := 0;
        Used  : X_Natural := Req_Size ;
        Count : X_Natural := 0 ; -- Number of stored output commands
        Data  : Generic_Pointer ;
    end record ;
end Pex_Types ;
