---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
--
-- This Ada language binding to the PEX extension to the X Window System
-- inherits the Sun Microsystem copyright. The following copyright must
-- be included in all software utilising this binding.
--
-- Copyright 1992 J.C. Mahieux and B. Maudry
-- Copyright 1989, 1990, 1991 by Sun Microsystems, Inc. and the X Consortium.
--
--   
--   
--   Package : Pex_Workstations
--   
--   Version :
--   
--   Subject : PEX Workstation functions
--   
--   Section : 10.1 -- Phigs Workstation Resource Management
--   
--   Create_Phigs_Workstation     
--   Free_Phigs_Workstation
--   
--   Section : 10.2 -- Phigs Workstation Resource Inquiry
--   
--   Get_Workstation_Information            
--   Get_Dynamics       
--   Get_View_Representation
--   
--   Section : 10.3 -- PHIGS Workstation Manipulation
--   
--   Redraw_All_Structures
--   Update_Workstation
--   Redraw_Clip_Region      
--   Execute_Deferred_Actions
--   Set_View_Priority       
--   Set_Display_Update_Mode   
--   Map_DC_to_WC            
--   Map_WC_to_DC            
--  
--   Section : 10.4 -- PHIGS Workstation Update Requests
--  
--   Set_View_Representation
--   Set_Workstation_Window       
--   Set_Workstation_Viewport     
--   Set_Hlhsr_Mode
--   Set_Workstation_Buffer_Mode
--  
--   Section : 10.5 -- Posting/Unposting Structures
--  
--   Post_Structure     
--   Unpost_Structure
--   Unpost_All_Structures    
--   Get_Workstation_Postings     
--   
--  
with X_Types;
with Pex_Types;  
--  
package Pex_Workstations is
--  
    use X_Types;
    use Pex_Types;  
--  
--   
--   Section : 10.1 -- Phigs Workstation Resource Management
--   

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Create_Phigs_Workstation
    --
    --  <Description>
    --
    --  This request creates a PHIGS workstation resource for the
    --  returned wks_id . The window or pixmap specified by
    --  drawable_id is associated with the newly-created PHIGS
    --  workstation resource. The named tables and name sets
    --  are also bound to the PHIGS workstation resource for use
    --  during rendering. A view table that supports current and
    --  requested view table entries is allocated for the PHIGS
    --  workstation automatically at creation time. The requests
    --  PEXSetViewRep and PEXGetViewRep can be used to modify and
    --  query the PHIGS workstation view table. The buffer_mode
    --  attribute is used to specify whether the workstation will
    --  operate in single-buffered or double-buffered mode. If
    --  double-buffered, an additional image buffer will be
    --  created for the drawable in an implementation-dependent
    --  fashion in order to support double-buffering. If
    --  drawable_id is a pixmap, then no double buffering will be
    --  performed.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Drawable_Id   : The associated drawable.
    --  Tables        : The laready created lookup tables.
    --  Highlight     : Name set filter used for highlighting.
    --  Invisible     : Name set filter used for visibility.
    --  Mode          : Single or double buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    function Create_Phigs_Workstation 
        ( Display_Id    : in Display;
          Drawable_Id   : in Drawable;
          Tables        : in Workstation_Lookup_Tables;
          Highlight     : in Name_Set_Filter;
          Invisible     : in Name_Set_Filter;
          Mode          : in Buffer_Mode) return Phigs_Workstation;

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Free_Phigs_Workstation
    --
    --  <Description>
    --
    --  This request deletes the PHIGS workstation resource and
    --  any image buffers that were created by the PEX extension
    --  in order to support double-buffering. The storage
    --  associated with the resource is then freed as well. 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation to delete.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Free_Phigs_Workstation
        ( Display_Id        : in Display;
          Workstation       : in Phigs_Workstation);
    
--   
--   Section : 10.2 -- Phigs Workstation Resource Inquiry
--   
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Workstation_Information
    --
    --  <Description>
    --
    --  This request returns attributes of the PHIGS workstation
    --  resource indicated by wks_id . Floating point values will
    --  be returned in the floating point format specified by
    --  fp_format . The parameter item_mask indicates which
    --  attributes are to be returned. For each bit that is set
    --  in item_mask , a corresponding value is returned in
    --  item_list .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstationto inquire about.
    --  Item_Mask   : Selection of required items.
    --  Values      : Returned values of selected items.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Get_Workstation_Information 
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Item_Mask   : in Workstation_Bitmask;
          Values      : in out Workstation_Values);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Dynamics
    --
    --  <Description>
    --
    --  This request returns information about the dynamics that
    --  are supported by PHIGS workstations associated with
    --  drawables of the type specified by drawable_id . The list
    --  of dynamics, in order, is as follows: Attribute Name
    --  Description view_rep Changes to view table marker_bundle
    --  Changes to marker bundle text_bundle Changes to text
    --  bundle line_bundle Changes to line bundle interior_bundle
    --  Changes to interior bundle edge_bundle Changes to edge
    --  bundle color_table Changes to color table pattern_table
    --  Changes to pattern table wks_transform Changes to
    --  workstation transformations highlight_filter Changes to
    --  highlight name set invisibility_filter Changes to
    --  invisibility name set HLHSR_mode Changes to HLHSR mode
    --  structure_modify Structure modifications
    --  (edits)post_structure Additions to posted structure list
    --  unpost_structure Deletions from posted structure list
    --  delete_structure Deletion of structure resources
    --  reference_modify Structure reference modifications
    --  buffer_modify Changes to buffering mode light_table
    --  Changes to light table | depth_cue_table Changes to depth
    --  cue table | color_approx_table Changes to color
    --  approximation table | The value returned for each of the
    --  items in the list above can be one of IMM, IRG, or CBS ,
    --  where IMM means that the specified action can be
    --  performed and the correct image displayed immediately,
    --  IRG means that the specified action requires a
    --  regeneration of the image, and CBS means that the
    --  specified action can be simulated immediately if the
    --  PHIGS workstation's display_update mode is set to
    --  SimulateSome .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Drawable_Id : The drawable associated with a workstation.
    --  Reply       : The required information.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Get_Dynamics ( Display_Id  : in Display;
                             Drawable_Id : in Drawable;
                             Reply       : out Dynamics_Info);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_View_Representation
    --
    --  <Description>
    --
    --  This request returns the value of the view update state
    --  and the specified entries in the requested and current
    --  view tables in the PHIGS workstation specified by wks_id
    --  . The view_update will be Pending if a view change has
    --  been requested but not established. If the specified
    --  entry is not defined, an error will be generated and the
    --  contents of the reply parameters will be undefined.
    --  Floating point values will be returned in the floating
    --  point format specified by fp_format .
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation   : The workstation.
    --  Index         : The view lookup table index.
    --  Update        : The view update state.
    --  Requested_Rep : 
    --  Current_Rep   : 
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Get_View_Representation
        ( Display_Id    : in Display;
          Workstation   : in Phigs_Workstation;
          Index         : in Lookup_Table_Index;
          Update        : out View_Update;
          Requested_Rep : out View_Representation;
          Current_Rep   : out View_Representation);

--   
--   Section : 10.3 -- PHIGS Workstation Manipulation
--   
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Redraw_All_Structures
    --
    --  <Description>
    --
    --  This request redraws all the posted structures contained
    --  in the PHIGS workstation resource specified by wks_id .
    --  First, if the PHIGS workstation's display_surface
    --  attribute is NotEmpty , its drawable is cleared to the
    --  color stored in entry zero of its color lookup table.
    --  Then, if any of its view_update , wks_update ,
    --  HLHSR_update , or buffer_update attributes is set to
    --  Pending , the requested values are made the current
    --  values and the attribute is set to NotPending . After
    --  this, all the posted structures are traversed and
    --  rendered (in priority order)Finally, the PHIGS
    --  workstation's visual_state attribute is set to Correct.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation where redraw should occur.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Redraw_All_Structures
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation);


    --=====================================================================
    --  <Subprogram Name>
    --
    --  Update_Workstation
    --
    --  <Description>
    --
    --  This request will perform actions identical to
    --  PEXRedrawAllStructures on the PHIGS workstation specified
    --  by wks_id if its visual_state attribute is currently set
    --  to Deferred or Simulated . 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Update_Workstation
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation);


    --=====================================================================
    --  <Subprogram Name>
    --
    --  Redraw_Clip_Region
    --
    --  <Description>
    --
    --  This request will perform actions similar to the
    --  PEXRedrawAllStructures request, except that no PHIGS
    --  workstation state attributes are modified or updated by
    --  this request. Rendering will occur only in the region
    --  defined by clip_list . The color stored in entry zero of
    --  the wks_id's color lookup table is used to clear the
    --  region under the rectangles defined by clip_list . The
    --  rectangles in clip_list should be non- overlapping, or
    --  the graphics results will be undefined. (If a z-buffering
    --  algorithm is used, only those pixels under the rectangles
    --  in the clip list will have their z values
    --  initialized.)All of the posted structures for wks_id are
    --  then redrawn, but only pixels under the rectangles in
    --  clip_list are affected. Pending changes are not made
    --  current by this request, nor is the visual_state
    --  attribute modified.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Rectangles  : List of clipping rectangles.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Redraw_Clip_Region
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Rectangles  : in DC.Rectangular_Region_2_Array);
    

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Execute_Deferred_Actions
    --
    --  <Description>
    --
    --  This request causes all the deferred actions on the PHIGS
    --  workstation specified by wks_id to be executed. 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Execute_Deferred_Actions
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation);


    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_View_Priority
    --
    --  <Description>
    --
    --  This request sets the relative priorities of entries in
    --  wks_id 's current view table. The priority of view table
    --  entry index1 with respect to view table entry index2 is
    --  set to the next Higher or Lower priority. These
    --  priorities are used to determine the order in which view
    --  table entries are tested when selecting the inverse
    --  viewing transformation to use for transformation from
    --  device coordinates to world coordinates.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation  : The workstation.
    --  First_Index  : View index.
    --  Second_Index : View index.
    --  Priority     : Relative priority.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_View_Priority
        ( Display_Id   : in Display;
          Workstation  : in Phigs_Workstation;
          First_Index  : in Lookup_Table_Index;
          Second_Index : in Lookup_Table_Index;
          Priority     : in Relative_Priority);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Display_Update_Mode
    --
    --  <Description>
    --
    --  This request sets the display_update attribute of the
    --  PHIGS workstation resource specified by wks_id . This
    --  attribute defines how changes to the display surface will
    --  be visualized. The list of permissible display update
    --  modes is defined in the "Extension Information" section
    --  that describes enumerated types. If double buffering is
    --  enabled (see PEXSetWKSBufferMode )the display update mode
    --  affects which buffer is rendered into. If the display
    --  update mode is VisualizeEach or VisualizeWhenever ,
    --  output primitives are rendered into the back
    --  (undisplayed)buffer while the structure network is being
    --  traversed. When the traversal is complete, the front
    --  and back buffers are swapped, so the rendered image is
    --  displayed. If the display update mode is VisualizeEasy
    --  or SimulateSome , output primitives are always rendered
    --  into the front (displayed)buffer. If the display update
    --  mode is VisualizeNone , output primitives are not
    --  rendered to either buffer.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Update_Mode : Display update mode.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Display_Update_Mode
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Update_Mode : in Display_Update_Method);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Map_DC_to_WC
    --
    --  <Description>
    --
    --  This request maps the device coordinate points in
    --  dc_points to the world coordinate points in wc_points
    --  using the PHIGS workstation resource specified by wks_id
    --  . (The client must convert pointer position values in
    --  drawable coordinates into device coordinates.)Each view
    --  in the PHIGS workstation's current view table is checked
    --  to see if it contains all the specified device coordinate
    --  points. The index of the view with the highest view
    --  transformation input priority that contains all of the
    --  points is returned in view_index . If no view contains
    --  all of the points, the index of the view containing the
    --  most points is returned. The points are transformed to
    --  world coordinates by passing them through the inverse of
    --  the view transform associated with the view index and are
    --  returned in wc_points . Floating point values are passed
    --  and will be returned in the floating point format
    --  specified by fp_format . Points that are clipped (outside
    --  the viewport) will not be transformed and returned in
    --  the wc_points list, so the number of points returned may
    --  be less than the number sent. 
    --
    --  <Parameters>
    --
    --  Display_Id      : The X Server connection.
    --  Workstation     : The workstation.
    --  Coordinates_In  : Coordinates to map (DC)
    --  View            : View index.
    --  Coordinates_Out : WC returned.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Map_DC_to_WC
        ( Display_Id      : in Display;
          Workstation     : in Phigs_Workstation;
          Coordinates_In  : in DC.Point_List_3;
          View            : out View_Index;
          Coordinates_Out : out WC.Point_List_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Map_WC_to_DC
    --
    --  <Description>
    --
    --  This request maps the world coordinate points in
    --  wc_points to the device coordinate points in dc_points
    --  using the PHIGS workstation resource specified by wks_id
    --  and the view specified by view_index . The points are
    --  transformed to device coordinates by passing them through
    --  the view transform associated with the view_index .
    --  Floating point values sent and received will be in the
    --  floating point format specified by fp_format . Points
    --  that are clipped will not be returned in the dc_points
    --  list, so the number of points returned may be less than
    --  the number sent.
    --
    --  <Parameters>
    --
    --  Display_Id      : The X Server connection.
    --  Workstation     : The workstation.
    --  View            : View index.
    --  Coordinates_In  : Coordinates to map (WC).
    --  Coordinates_Out : Returned DC.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Map_WC_to_DC
        ( Display_Id      : in Display;
          Workstation     : in Phigs_Workstation;
          View            : in View_Index;
          Coordinates_In  : in WC.Point_List_3;
          Coordinates_Out : out DC.Point_List_3);

--  
--   Section : 10.4 -- PHIGS Workstation Update Requests
--  
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_View_Representation
    --
    --  <Description>
    --
    --  This request sets the requested values of the specified
    --  view table entry of the view table in the PHIGS
    --  workstation specified by wks_id to the view
    --  representation indicated by view_rep . The view_update
    --  attribute in the PHIGS workstation is set to Pending if
    --  the change cannot be visualized immediately; otherwise
    --  it is set to NotPending . If the view_update attribute is
    --  NotPending , the current view table entry is set to the
    --  requested values; otherwise the current values are not
    --  changed until the PHIGS workstation is updated.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  View_Rep    : The view representation.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_View_Representation
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          View_Rep    : in View_Representation);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Workstation_Window
    --
    --  <Description>
    --
    --  This request sets the req_NPC_subvolume of the PHIGS
    --  workstation specified by wks_id to the values in
    --  NPC_subvolume . The wks_update attribute in the PHIGS
    --  workstation is set to Pending if the change cannot be
    --  visualized immediately; otherwise it is set to NotPending
    --  . If the wks_update attribute is NotPending , the
    --  cur_NPC_subvolume is set to the requested values;
    --  otherwise the current values are not changed until the
    --  PHIGS workstation is updated.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation   : The workstation.
    --  Npc_Subvolume : The window space.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Workstation_Window
        ( Display_Id    : in Display;
          Workstation   : in Phigs_Workstation;
          Npc_Subvolume : in NPC.Rectangular_Region_3);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Workstation_Viewport
    --
    --  <Description>
    --
    --  This request sets the req_wks_viewpt of the PHIGS
    --  workstation specified by wks_id to the values in
    --  viewport . The wks_update attribute in the PHIGS
    --  workstation is set to Pending if the change cannot be
    --  visualized immediately; otherwise it is set to NotPending
    --  . If the wks_update attribute is NotPending , the
    --  cur_wks_viewpt is set to the requested values; otherwise
    --  the current values are not changed until the PHIGS
    --  workstation is updated. 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation   : The workstation.
    --  Viewport      : The device coordinates area.
    --  Use_Drawable  :
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Workstation_Viewport
        ( Display_Id    : in Display;
          Workstation   : in Phigs_Workstation;
          Viewport      : in DC.Rectangular_Region_3 ;
          Use_Drawable  : in Boolean := True);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Hlhsr_Mode
    --
    --  <Description>
    --
    --  This request sets the req_HLHSR_mode of the PHIGS
    --  workstation specified by wks_id to the values in mode .
    --  If the PHIGS workstation's display_surface attribute is
    --  Empty , or if the dynamic modification for its HLHSR_mode
    --  is IMM , its cur_HLHSR_mode attribute is modified with
    --  the value contained in mode and its HLHSR_update is set
    --  to NotPending ; otherwise, its HLHSR_update is set to
    --  Pending and the current value is not changed until the
    --  PHIGS workstation is updated.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Mode        : HLHSR mode.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Hlhsr_Mode
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Mode        : in Hlhsr_Mode);
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Set_Workstation_Buffer_Mode
    --
    --  <Description>
    --
    --  This request sets the req_buffer_mode of the PHIGS
    --  workstation specified by wks_id to the values in
    --  buffer_mode . If the PHIGS workstation's display_surface
    --  attribute is Empty , or if the dynamic modification for
    --  its buffer_mode is IMM , its cur_buffer_mode attribute is
    --  modified with the value contained in buffer_mode and
    --  its buffer_update is set to NotPending ; otherwise, its
    --  buffer_update is set to Pending and the current value
    --  is not changed until the PHIGS workstation is updated.
    --  Buffer_mode may be one of the constants Single (rendering
    --  is to be single-buffered)or Double (rendering is to be
    --  double-buffered)An Alloc error will be returned if the
    --  requested mode is Double and the server cannot allocate
    --  any more image buffers.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Mode        : single or double buffer.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Set_Workstation_Buffer_Mode
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Mode        : in Buffer_Mode);

--  
--   Section : 10.5 -- Posting/Unposting Structures
--  
    
    --=====================================================================
    --  <Subprogram Name>
    --
    --  Post_Structure
    --
    --  <Description>
    --
    --  This request adds the structure specified by s_id to the
    --  list of posted structures in the PHIGS workstation wks_id
    --  . A priority is also provided to indicate the priority of
    --  the newly-posted structure with respect to the structures
    --  already in the posted structure list. If multiple
    --  structures are posted for display to the same display
    --  space location, the implementation will ensure the
    --  display of the higher priority structure. If two
    --  structures have the same priority, the last posted
    --  structure has the higher priority. If s_id is not a valid
    --  structure resource ID, the request is ignored, and an
    --  error is generated.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Struct_Id   : The structure to post.
    --  Priority    : The structure priority.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Post_Structure
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Struct_Id   : in Structure;
          Priority    : in Display_Priority);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Unpost_All_Structures
    --
    --  <Description>
    --
    --  This request removes all structures from wks_id 's posted
    --  structure list.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Unpost_All_Structures
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation);


    --=====================================================================
    --  <Subprogram Name>
    --
    --  Unpost_Structure
    --
    --  <Description>
    --
    --  This request removes the structure specified by s_id from
    --  wks_id 's posted structure list. If s_id is not found in
    --  the posted structure list, the request is ignored, and an
    --  error is generated.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Workstation : The workstation.
    --  Struct_Id   : The structure to unpost.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Unpost_Structure
        ( Display_Id  : in Display;
          Workstation : in Phigs_Workstation;
          Struct_Id   : in Structure);

    --=====================================================================
    --  <Subprogram Name>
    --
    --  Get_Workstation_Postings
    --
    --  <Description>
    --
    --  This request returns a list of all of the PHIGS
    --  workstation resources to which s_id has been posted. 
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Struct_Id  : The structure to find.
    --  Wks_Ids    : The workstations where Struct_Id is posted.
    --
    --  <Errors Raised>
    --
    --=====================================================================

    procedure Get_Workstation_Postings
        ( Display_Id : in Display;
          Struct_Id  : in Structure;
          Wks_Ids    : out Phigs_Workstation_List);
end Pex_Workstations; 
