---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob 
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Colors is
    use X_Types ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Allocate_Color
    --
    --  <Description>
    --
    --  This procedure returns the pixel values indicating the closest
    --  color supported by the hardware given a color definition. It also
    --  returns the red, green and blue values actually used.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Definition : Returns the values actually used in the colormap.
    --  Success    : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if the Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Allocate_Color ( Display_Id : in Display;
                               Map_Id     : in Color_Map;
                               Definition : in out Color_Record;
                               Success    : out Boolean);



    --====================================================================
    --  <Subprogram Name>
    --
    --  Allocate_Color_Cells
    --
    --  <Description>
    --
    --  This procedure allocates color cells. It returns the pixels and
    --  plane masks requested. All are allocated writeable by the request.
    --  The RGB values of the allocated entries are undefined.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Map_Id       : Specifies the Color_Map ID.
    --  Contiguous   : Specifies if the planes must be contiguous.
    --  Planes       : Returns an array of allocated planes.
    --  Pixel_Values : Returns an array of allocated Pixel_Values.
    --  Success      : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if Map_Id is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Allocate_Color_Cells
        ( Display_Id   : in Display;
          Map_Id       : in Color_Map;
          Contiguous   : in Boolean;
          Planes       : out Plane_Mask_Array;
          Pixel_Values : out Pixel_Array;
          Success      : out Boolean);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Allocate_Color_Planes
    --
    --  <Description>
    --
    --  This procedure allocates color planes.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Map_Id       : Specifies the Color_Map ID.
    --  Contiguous   : Specifies if the planes must be contiguous.
    --  Red_Shades   : Specifies the number of red colors.
    --  Green_Shades : Specifies the number of green colors.
    --  Blue_Shades  : Specifies the number of blue colors.
    --  Pixel_Values : Returns an array of allocated Pixel_Values.
    --  Red_Mask     : Returns a bit mask for the red plane.
    --  Green_Mask   : Returns a bit mask for the green plane.
    --  Blue_Mask    : Returns a bit mask for the blue plane.
    --  Success      : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised is Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Allocate_Color_Planes ( Display_Id   : in Display;
                                      Map_Id       : in Color_Map;
                                      Contiguous   : in Boolean;
                                      Red_Shades   : in Natural;
                                      Green_Shades : in Natural;
                                      Blue_Shades  : in Natural;
                                      Pixel_Values : out Pixel_Array;
                                      Red_Mask     : out Plane_Mask;
                                      Green_Mask   : out Plane_Mask;
                                      Blue_Mask    : out Plane_Mask;
                                      Success      : out Boolean);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Allocate_Named_Color
    --
    --  <Description>
    --
    --  This procedure determines the correct color(shade) for the given
    --  screen. It returns both the exact definition and the screen
    --  definition.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Color_Name : Specifies the name of the color whose definition
    --               you want.
    --  Screen_Def : Returns the values actually used in the colormap.
    --  Exact_Def  : Returns the true pixel values of the closet color
    --               supported by the hardware.
    --  Success    : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if the Map_Id is invalid.
    --  Bad_Name is raised if the Color_Name is invalid.
    --
    --  <Called By>
    --
    --  External.
    --

    --====================================================================

    procedure Allocate_Named_Color ( Display_Id : in Display;
                                     Map_Id     : in Color_Map;
                                     Color_Name : in String;
                                     Screen_Def : out Color_Record;
                                     Exact_Def  : out Color_Record;
                                     Success    : out Boolean) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  "and"
    --
    --  <Description>
    --
    --  This operator returns the pixel value resulting in a bit to bit
    --  logical "and" on the given pixel values.
    --
    --  <Parameters>
    --
    --  Left   : Specifies the pixel values
    --  Right  : to operate on.
    --
    --  <Exceptions Raised>   None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    function "and" ( Left, Right : in Pixel_Value ) return Pixel_Value ;
    
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Copy_Colormap_And_Free
    --
    --  <Description>
    --
    --  This procedure creates a new colormap of the same visual type and
    --  for the same screen as the Source, returning this new ID as
    --  Target. It then moves all of the client's existing allocation from
    --  Source to Target with their color values intact and their read-
    --  only/writable characteristics intact and frees those entries.
    --  It then frees the Source colormap.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Source     : The Color_Map ID to be copied from.
    --  Target     : The returned Color_Map ID.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Alloc
    --  Bad_Color is raised if the Source is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Copy_Colormap_And_Free ( Display_Id : in Display;
                                      Source     : in Color_Map )
        return  Color_Map ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Create_Colormap
    --
    --  <Description>
    --
    --  This function creates a colormap of the specified Visual type for
    --  the screen on which the window resides and associates the color
    --  map id with it.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Window_Id  : Specifies the window on which you create the colormap
    --  Visual_Id  : Specifies the visual type supported on the screen.
    --  Allocate   : Specifies either Allocate_None or Allocate_All.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the Visual is not supported by the screen.
    --  Bad_Window is raised if the Window_Id is not valid.
    --  Bad_Alloc is raised if Allocate = Alloc_All and the class member
    --            element of the visual is Static_Gray, Static_Color or
    --            True_Color.
    --  Bad_Value is raised if a bad color mask element of the visual is
    --            passed.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Create_Colormap
        ( Display_Id : in Display;
          Window_Id  : in Window;
          Visual_Id  : in Visual;
          Allocate   : in Colormap_Allocator) return Color_Map;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Free_Colormap
    --
    --  <Description>
    --
    --  This procedure deletes the association between a colormap and its
    --  ID, and uninstalls it if it was installed. It has no effect on the
    --  default color map for a screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : The Color_Map ID to be freed.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Free_Colormap ( Display_Id : in Display ;
                              Map_Id     : in out Color_Map ) ;

    --========================================================================
    --  <Subprogram Name>
    --
    --  Free_Colors
    --
    --  <Description>
    --
    --  This procedure frees the cells represented by pixels values in
    --  the Pixels_To_Free array. The Planes should not have any pixels 
    --  in common with any of the pixels.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Map_Id         : Specifies the Color_Map ID.
    --  Pixels_To_Free : Specifies the pixels to be freed.
    --  Planes         : Specifies the planes to be freed.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Access is raised if a specified pixel is unallocated or read-only.
    --  Bad_Color is raised is Map_Id is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into 
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    procedure Free_Colors ( Display_Id     : in Display;
                            Map_Id         : in Color_Map;
                            Pixels_To_Free : in Pixel_Array;
                            Planes         : in Plane_Mask) ;
    
    --========================================================================
    --  <Subprogram Name>
    --
    --  Get_Standard_Colormaps
    --
    --  <Description>
    --
    --  This procedure gets the standard colormaps associated with one of the
    --  described atoms.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the X Server connection.
    --  Window_Id  : Specifies the Window.
    --  Property   : Specifies which property to set.
    --  Cmaps      : Returns the colormap list for the property.
    --  Success    : Indicates if the operation was successful.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    procedure Get_Standard_Colormaps
        ( Display_Id  : in Display;
          Window_Id   : in Window;
          Property    : in Atom;
          Cmaps       : out Standard_Colormap_List;
          Success     : out Boolean);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Install_Colormap
    --
    --  <Description>
    --
    --  This procedure installs the specified color map. All windows
    --  associated with this color map immediately display with true
    --  colors. This association was made when the windows were created.
    --
    --  This procedure is primarily used by window managers.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Install_Colormap ( Display_Id : in Display;
                                 Map_Id     : in Color_Map) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  List_Installed_Colormaps
    --
    --  <Description>
    --
    --  This function obtains a list of the currently installed color maps
    --  for the screen of the specified window.
    --
    --  This procedure is primarily used by window managers.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Window_Id  : Specifies the window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function List_Installed_Colormaps ( Display_Id : in Display ;
                                        Window_Id  : in Window )
        return Color_Map_List ;

    --========================================================================
    --  <Subprogram Name>
    --
    --  Lookup_Color
    --
    --  <Description>
    --
    --  This procedure looks up the string name of a color for the given
    --  screen. It returns both the exact definition and the screen
    --  definition.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Color_Name : Specifies the name of the color whose definition is
    --               requested.
    --  Screen_Def : Returns the values actually used in the colormap.
    --  Exact_Def  : Returns the true pixel values of the closest color
    --               supported by the hardware.
    --  Success    : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if the Map_Id is invalid.
    --  Bad_Name is raised if the Color_Name is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    procedure Lookup_Color ( Display_Id : in Display;
                             Map_Id     : in Color_Map;
                             Color_Name : in String;
                             Screen_Def : out Color_Record;
                             Exact_Def  : out Color_Record;
                             Success    : out Boolean) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  "or"
    --
    --  <Description>
    --
    --  This operator returns the pixel value resulting in a bit to bit
    --  logical "or" on the given pixel values.
    --
    --  <Parameters>
    --
    --  Left   : Specifies the pixel values
    --  Right  : to operate on.
    --
    --  <Exceptions Raised>   None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    function "or" ( Left, Right : in Pixel_Value ) return Pixel_Value ;
    
    
    --======================================================================
    --  <Subprogram Name>
    --
    --  Parse_Color
    --
    --  <Description>
    --
    --  This procedure provides a simple way to create a user interface to
    --  color. It takes a string specification of a color, typically from
    --  a command line or Get_Default option, and returns the corresponding
    --  red, green and blue values that are suitable for a subsequent call
    --  to Allocate_Color or Store_Color. The color can be specified as a
    --  color name, or as an initial sharp character followed by a numeric
    --  specification in one of the following formats:
    --
    --    #RGB                      ( 4 bits each)
    --    #RRGGBB                   ( 8 bits each)
    --    #RRRGGGBBB                (12 bits each)
    --    #RRRRGGGGBBBB             (16 bits each)
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Color_Name : Specifies the name or specification.
    --  Screen_Def : Returns the actual values used in the color map.
    --  Success    : Returns the success of the operation.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Parse_Color ( Display_Id : in Display;
                            Map_Id     : in Color_Map;
                            Color_Name : in String;
                            Screen_Def : out Color_Record;
                            Success    : out Boolean);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Color
    --
    --  <Description>
    --
    --  This procedure obtains the color values for a single specified pixel
    --  value.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  The_Color  : Returns the RGB values for the pixel specified.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised is Map_Id is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Color ( Display_Id : in Display;
                            Map_Id     : in Color_Map;
                            The_Color  : in out Color_Record);


    --========================================================================
    --  <Subprogram Name>
    --
    --  Query_Colors
    --
    --  <Description>
    --
    --  This procedure obtains the color values for an array of pixels stored
    --  in color structures.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  The_Colors : Returns the RGB values for the pixels specified.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised is Map_Id is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into 
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --

    --========================================================================

    procedure Query_Colors ( Display_Id : in Display;
                             Map_Id     : in Color_Map;
                             The_Colors : in out Color_Array) ;

    --========================================================================
    --  <Subprogram Name>
    --
    --  Set_Standard_Colormap
    --
    --  <Description>
    --
    --  This procedure sets the standard colormap associated with on of the
    --  described atoms.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the X Server connection.
    --  Window_Id  : Specifies the Window.
    --  Cmaps      : Specifies the colormap list for the property.
    --  Property   : Specifies which property to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    procedure Set_Standard_Colormaps ( Display_Id : in Display;
                                       Window_Id  : in Window;
                                       Cmaps      : in Standard_Colormap_List;
                                       Property   : in Atom ) ;
    
    --======================================================================
    --  <Subprogram Name>
    --
    --  Store_Color
    --
    --  <Description>
    --
    --  This procedure changes the color map entry of the pixel value
    --  specified by the pixel member of the Screen_Def parameter and
    --  returns the actual value set.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Screen_Def : Returns the values actually used in the color map.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if the Map_Id is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Store_Color ( Display_Id : in Display;
                            Map_Id     : in Color_Map;
                            Screen_Def : in Color_Record);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Store_Colors
    --
    --  <Description>
    --
    --  This procedure changes the color map entries of the pixel values
    --  specified in the pixel members of the Color_Array members. You
    --  specify which color components to be changed by setting the
    --  Color_Flag in each member. If the color map is an installed color
    --  map, the effect is immediate.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Colors     : Specifies the list of color definition structures.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Value is raised if a specified pixel is not a valid index into
    --            Map_Id.
    --  Bad_Access is raised if a specified pixel is unallocated or
    --            read-only.
    --  Bad_Color is raised if the Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Store_Colors ( Display_Id : in Display;
                             Map_Id     : in Color_Map;
                             Colors     : in Color_Array);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Store_Named_Color
    --
    --  <Description>
    --
    --  This procedure allocates a color cell by looking up the named color.
    --  The Map_Entry parameter determines the entry in the color map.
    --  The Flags determine which of the red, green and blue indices are set.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --  Color_Name : Specifies the color name.
    --  Map_Entry  : Specifies the entry in the color map.
    --  Flags      : Specifies which red, green and blue indices to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Access is raised if a specified pixel is unallocated or read-only.
    --  Bad_Color is raised is Map_Id is invalid.
    --  Bad_Name is raised if Color_Name is invalid.
    --  Bad_Value is raised if a specified pixel is not a valid index into
    --            Map_Id.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Store_Named_Color ( Display_Id : in Display;
                                  Map_Id     : in Color_Map;
                                  Color_Name : in String;
                                  Map_Entry  : in Pixel_Value;
                                  Flags      : in Color_Flag);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Uninstall_Colormap
    --
    --  <Description>
    --
    --  This procedure removes the specified color map. All windows
    --  associated with this color map immediately display with true
    --  colors. This association was made when the windows were created.
    --
    --  This procedure is primarily used by window managers.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map_Id     : Specifies the Color_Map ID.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Color is raised if Map_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Uninstall_Colormap ( Display_Id : in Display;
                                   Map_Id     : in Color_Map) ;


    --====================================================================
    --  <Subprogram Name>
    --
    --  "xor"
    --
    --  <Description>
    --
    --  This operator returns the pixel value resulting in a bit to bit
    --  logical "xor" on the given pixel values.
    --
    --  <Parameters>
    --
    --  Left   : Specifies the pixel values
    --  Right  : to operate on.
    --
    --  <Exceptions Raised>   None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    function "xor" ( Left, Right : in Pixel_Value ) return Pixel_Value ;
    
    
end X_Colors;
