---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Display_Manager is
    use X_Types ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Activate_Screen_Saver
    --
    --  <Description>
    --
    --  This procedure is used to activate the screen saver.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Activate_Screen_Saver (Display_Id : in Display) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Add_Host
    --
    --  <Description>
    --
    --  This procedure is used to add the specified host to the access
    --  control list for that display. The display hardware must be on the
    --  same host as the program issuing the command.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    procedure Add_Host ( Display_Id : in Display;
                         Host       : in Host_Name_Record);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Add_Hosts
    --
    --  <Description>
    --
    --  This procedure is used to add each specified host to the access
    --  control list for that display. The display hardware must be on the
    --  same host as the program issuing the command.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Add_Hosts ( Display_Id : in Display;
                          Hosts      : in Host_Array) ;

    --========================================================================
    --  <Subprogram Name>
    --
    --  Bitmap_Bit_Order
    --
    --  <Description>
    --
    --  This function is used to determine whether the leftmost bit in the
    --  bitmap is either the least or most significant bit in the unit.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Bitmap_Bit_Order (Display_Id : in Display) return Order_Type;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Bitmap_Pad
    --
    --  <Description>
    --
    --  This function is used to determine the number of bits each scanline
    --  must be padded to.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Bitmap_Pad (Display_Id : in Display) return Pixels;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Bitmap_Unit
    --
    --  <Description>
    --
    --  This function is used to get the size of the bitmap's unit in
    --  bits.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Bitmap_Unit (Display_Id : in Display) return Pixels;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Black_Pixel
    --
    --  <Description>
    --
    --  This function is used to return the black pixel value of the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Black_Pixel ( Display_Id : in Display;
                           Screen_Id  : in Screen_Number)
        return Pixel_Value;



    --====================================================================
    --  <Subprogram Name>
    --
    --  Black_Pixel_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the black pixel value of the specified
    --  screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Black_Pixel_Of_Screen (Screen_Id : in Screen) return Pixel_Value;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Cells_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the number of colormap cells in the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Cells_Of_Screen (Screen_Id : in Screen) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Close_Display
    --
    --  <Description>
    --
    --  This procedure closes the connection with the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Close_Display ( Display_Id : in out Display);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Colormap
    --
    --  <Description>
    --
    --  This function is used to determine the default colormap Id for
    --  allocation on the specified screen. Most routine allocations. of
    --  color should be made out of this map.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Colormap
        ( Display_Id : in Display;
          Screen_Id  : in Screen_Number) return Color_Map;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Colormap_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the default colormap from the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Colormap_Of_Screen
        (Screen_Id : in Screen) return Color_Map;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Depth
    --
    --  <Description>
    --
    --  This function is used to return the depth (number of planes) of
    --  the default root window for the specified screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Depth ( Display_Id : in Display;
                             Screen_Id  : in Screen_Number)
        return Depth_Type;


    --======================================================================
    --  <Subprogram Name>
    --
    --  Default_Depth_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the default depth of the specified
    --  screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Depth_Of_Screen (Screen_Id : in Screen)
        return Depth_Type;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Graphic_Context
    --
    --  <Description>
    --
    --  This function is used to return the default graphic context for
    --  the root window of the specified screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Graphic_Context
         ( Display_Id : in Display;
           Screen_Id  : in Screen_Number)
        return Graphic_Context;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_GC_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the default Graphic Context of from
    --  the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --=====================================================================

    function Default_Gc_Of_Screen
        (Screen_Id : in Screen) return Graphic_Context;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Root_Window
    --
    --  <Description>
    --
    --  This function is used to determine the default root window for the
    --  default screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Root_Window (Display_Id : in Display) return Window;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Screen
    --
    --  <Description>
    --
    --  This function is used to return the default screen referenced in
    --  the Open_Display routine.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Screen (Display_Id : in Display)
        return Screen_Number;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Screen_Of_Display
    --
    --  <Description>
    --
    --  This function is used to return the default screen of the
    --  specified display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Screen_Of_Display (Display_Id : in Display)
        return Screen;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Visual
    --
    --  <Description>
    --
    --  This function is used to return the default visual type for the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Visual
        ( Display_Id : in Display;
          Screen_Id  : in Screen_Number) return Visual;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Default_Visual_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the default visual from the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Default_Visual_Of_Screen (Screen_Id : in Screen) return Visual;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Disable_Access_Control
    --
    --  <Description>
    --
    --  This procedure is used to disable the use of the access control list
    --  at connection setups.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Disable_Access_Control (Display_Id : in Display) ;
    
    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Cells
    --
    --  <Description>
    --
    --  This function is used to return the number of entries in the default
    --  color map.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Cells ( Display_Id : in Display;
                             Screen_Id  : in Screen_Number) return Natural;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Height
    --
    --  <Description>
    --
    --  This function is used to return an integer that describes the height
    --  of the screen in pixels.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Height ( Display_Id : in Display;
                              Screen_Id  : in Screen_Number) return Pixels;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Height
    --
    --  <Description>
    --
    --  This function is used to return the height of the specified screen
    --  in millimeters.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Height ( Display_Id : in Display;
                              Screen_Id  : in Screen_Number)
        return Millimeters;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Name
    --
    --  <Description>
    --
    --  This function is used to return the name of the display that you are
    --  currently using. If a null string ("") is specified, Display_Name
    --  looks in the environment for the display and returns the display name
    --  that the user was requesting. This makes easier to report to the
    --  user precisely which display the program attempted to open when the
    --  initial connection attempt failed.
    --
    --  <Parameters>
    --
    --  Name : Specifies the name.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Name (Name : in String) return String;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the display of the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Of_Screen (Screen_Id : in Screen) return Display;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Planes
    --
    --  <Description>
    --
    --  This function is used to get the depth of the root window
    --  of the specified screen.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Planes ( Display_Id : in Display;
                              Screen_Id  : in Screen_Number)
        return Depth_Type;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_String
    --
    --  <Description>
    --
    --  This function is used to return the string that was passed to the
    --  Open_Display function. On UNIX-based systems, if the passed string was
    --  null then the function returns the DISPLAY environment variable when
    --  the current display was opened.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_String (Display_Id : in Display) return String;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Width
    --
    --  <Description>
    --
    --  This function is used to obtain the width of the screen in pixels.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Width ( Display_Id : in Display;
                             Screen_Id  : in Screen_Number) return Pixels;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Display_Width
    --
    --  <Description>
    --
    --  This function is used to obtain the width of the screen in millimeters
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Display_Width ( Display_Id : in Display;
                             Screen_Id  : in Screen_Number)
        return Millimeters;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Does_Backing_Store
    --
    --  <Description>
    --
    --  This function is used to return a value indicating whether
    --  the screen supports backing stores. The value returned can be one of
    --  When_Mapped, Not_Useful or Always.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Does_Backing_Store (Screen_Id : in Screen)
        return Backing_Store_Type;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Does_Save_Unders
    --
    --  <Description>
    --
    --  This function is used to return a Boolean value indicating whether
    --  the screen supports save unders.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Does_Save_Unders (Screen_Id : in Screen) return Boolean;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Enable_Access_Control
    --
    --  <Description>
    --
    --  This procedure is used to enable the use of the access control list
    --  at connection setups.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Enable_Access_Control (Display_Id : in Display) ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Event_Mask_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the initial root event mask for the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Event_Mask_Of_Screen
        (Screen_Id : in Screen) return Event_Mask_Type;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Force_Screen_Saver
    --
    --  <Description>
    --
    --  This procedure is used to force the screen saver. If the specified
    --  mode is active and the screen saver currently is deactivated, the
    --  screen saver is activated, even if the screen saver had been disabled
    --  with a timeout of (0).
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Force_Screen_Saver ( Display_Id : in Display;
                                   Mode       : in Screen_Saver_Mode) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Screen_Saver
    --
    --  <Description>
    --
    --  This procedure is used to get the current screen saver values.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    procedure Get_Screen_Saver ( Display_Id : in Display;
                                 Time_Out   : out Time;
                                 Interval   : out Time;
                                 Blanking   : out Blanking_Type;
                                 Exposures  : out Exposure_Type);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Visual_Info
    --
    --  <Description>
    --
    --  This function obtains a list of visual information records matching
    --  the Info_Template.
    --
    --  <Parameters>
    --
    --  Display_Id    : Specifies the connection to the X Server.
    --  Info_Mask     : Specifies which components of the Info_Template are
    --                  to be matched.
    --  Info_Template : Specifies the template to match.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Visual_Info
        ( Display_Id    : in Display;
          Info_Mask     : in Visual_Mask_Type;
          Info_Template : in Visual_Info_Record) return Visual_Info_List;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Height_MM_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the height of the specified screen in
    --  millimeters.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Height_Mm_Of_Screen (Screen_Id : in Screen) return Millimeters;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Height_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the height of the specified screen
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Height_Of_Screen (Screen_Id : in Screen) return Pixels;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Image_Byte_Order
    --
    --  <Description>
    --
    --  This function is used to specify the required byte order order for
    --  images for each scanline unit in XY_Format (bitmap) or for each pixel
    --  value in Z_Format. The function can return one of the constants
    --  LSB_First or MSB_First.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Image_Byte_Order (Display_Id : in Display) return Order_Type;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Last_Know_Request_Processed
    --
    --  <Description>
    --
    --  This function returns the full serial number of the last request
    --  known by this X library to have been processed by the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Last_Known_Request_Processed
        (Display_Id : in Display) return X_Long_Integer;


    --====================================================================
    --  <Subprogram Name>
    --
    --  List_Hosts
    --
    --  <Description>
    --
    --  This procedure is used to return the current access control list
    --  as well as whether the use of the list at connection setup was
    --  enabled or disabled.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure List_Hosts ( Display_Id : in Display;
                           State      : out Boolean ;
                           Hosts      : out Host_List) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Match_Visual_Info
    --
    --  <Description>
    --
    --  This procedure returns a visual information record matching the
    --  specified depth and class of the screen.
    --
    --  <Parameters>
    --
    --  Display_Id      : Specifies the connection to the X Server.
    --  Screen_Id       : Specifies the screen.
    --  Depth_Of_Screen : Specifies the Depth to match.
    --  Class_Of_Screen : Specifies the Class to match.
    --  Visual_Info     : Returns the matching Visual information record.
    --  Success         : True is a visual is found.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Match_Visual_Info ( Display_Id      : in Display;
                                  Screen_Id       : in Screen_Number;
                                  Depth_Of_Screen : in Depth_Type;
                                  Class_Of_Screen : in Visual_Class_Type;
                                  Visual_Info     : out Visual_Info_Record;
                                  Success         : out Boolean);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Max_Cmaps_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the maximum number of color maps
    --  supported by the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Max_Cmaps_Of_Screen (Screen_Id : in Screen) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Max_Keycode
    --
    --  <Description>
    --
    --  This function is used to return the greatest keycode supported by the
    --  specified display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Max_Keycode (Display_Id : in Display) return Keycode;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Min_Cmaps_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the minimum number of color maps
    --  supported by the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Min_Cmaps_Of_Screen (Screen_Id : in Screen) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Min_Keycode
    --
    --  <Description>
    --
    --  This function is used to return the lowest keycode supported by the
    --  specified display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Min_Keycode (Display_Id : in Display) return Keycode;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Next_Request
    --
    --  <Description>
    --
    --  This function returns the full serial number to be used for the
    --  next request.
    --  If output is done by several tasks in parallel, this number can
    --  be actually associated to a request done by another task.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Next_Request (Display_Id : in Display) return X_Long_Integer;


    --====================================================================
    --  <Subprogram Name>
    --
    --  No_Operation
    --
    --  <Description>
    --
    --  This procedure executes a NoOperation protocol request to exercise
    --  the X Server (useful to avoid conections timeouts). The output buffer
    --  is not flushed.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure No_Operation (Display_Id : in Display) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Open_Display
    --
    --  <Description>
    --
    --  This function opens a connection to the X Server controlling the
    --  specified display and returns a Display ID for use in all
    --  subsequent calls.
    --
    --  <Parameters>
    --
    --  Name        : Specifies the hardware display name.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Open_Display ( Display_Id : in out Display ;
                             Name : in String ) ;



    --====================================================================
    --  <Subprogram Name>
    --
    --  Planes_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the number of planes in the
    --  specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Planes_Of_Screen (Screen_Id : in Screen) return Depth_Type;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Reserve_Display
    --
    --  <Description>
    --
    -- This procedure reserves access to the display against the other tasks
    -- requiring its use, and this until the call to Unreserve_Display.
    -- 
    -- The value provided as Display_Id is changed by the call and must be
    -- used for each reserved access to this display (i.e. each call to a
    -- XInAda subprogram). Any call with the original value will be blocked
    -- as it is the case for the other tasks. It is recommended to provide as
    -- parameter a local copy (in the task) of the original display variable
    -- and to use that copy as the display everywhere in the task.
    -- That changed value MUST NOT be used by any other task.
    -- The original value is restored by Unreserve_Display.
    -- To avoid any dead-lock, care should be taken to handle any exception
    -- raised during the critical section (until call to Unreserve_Display).
    --
    -- As the display value stored in the events is never the reserved one,
    -- no event processing must be done by a task which has called
    -- Reserve_Display and is still in this reserved mode.
    --
    -- As the call to this procedure suppresses the need for the XInAda
    -- subprograms called in that critical section to use the locking
    -- mecanisms, their execution is speeded up.
    -- For that reason, if the application uses a single task for accessing
    -- the display or use no tasking, it can call that procedure after the 
    -- call to Open_Display to speed up its X processing.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>  None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    procedure Reserve_Display ( Display_Id : in out Display ) ;
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Reserve_Display
    --
    --  <Description>
    --
    -- This second version of Reserved_Display provides the same services
    -- as the previous one but will only wait the provided delay for
    -- obtaining the reservation. If the delay expires before the display
    -- is freed, the procedure returns, the parameter Reserved is set
    -- to False and the diplay is NOT reserved. Otherwise, the parameter
    -- Reserved is set to True and the reservation is done with the same
    -- results as the first Reserve_Display.
    -- 
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  Wait_Up_To     : Max delay to wait for the reservation
    --  Reserved       : indicates if the reservation is successful
    --
    --  <Exceptions Raised>  None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    procedure Reserve_Display ( Display_Id : in out Display ;
                                Wait_Up_To : in Duration ;
                                Reserved   : out Boolean ) ;
    
    --===================================================================
    --  <Subprogram Name>
    --
    --  Protocol_Revision
    --
    --  <Description>
    --
    --  This function is used to return the minor protocol revision of the
    --  X Server.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --===================================================================

    function Protocol_Revision (Display_Id : in Display) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Protocol_Version
    --
    --  <Description>
    --
    --  This function is used to return the major version number (11) of the
    --  X Protocol associated with the connected Display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Protocol_Version (Display_Id : in Display) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Query_Pointer
    --
    --  <Description>
    --
    --  This procedure obtains the root window the pointer is currently on
    --  and the pointer coordinates relative to the root's origin.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Window_Id   : Specifies the window.
    --  Root        : Returns the root window Id.
    --  Child       : Returns the child, if any, the pointer was in.
    --  Root_Xy     : Returns pointer coordinates relative to the root.
    --  Window_Xy   : Returns coordinates relative to the specified window
    --  Mask        : Returns the current state of the modifier keys and
    --                pointer buttons.
    --  Same_Screen : True if the pointer is on the same screen as the
    --                specified window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Query_Pointer
        ( Display_Id       : in Display;
          Window_Id        : in Window;
          Root             : out Window;
          Child            : out Window;
          Root_Xy          : out Point;
          Window_Xy        : out Point;
          Keys_And_Buttons : out Key_And_Button_Mask;
          Same_Screen      : out Boolean);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Remove_Host
    --
    --  <Description>
    --
    --  This procedure is used to remove the specified host from the access
    --  control list for that display.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Remove_Host ( Display_Id : in Display;
                            Host       : in Host_Name_Record) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Remove_Hosts
    --
    --  <Description>
    --
    --  This procedure is used to remove each specified host from the access
    --  control list for that display.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Remove_Hosts ( Display_Id : in Display;
                             Hosts      : in Host_Array) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Reset_Screen_Saver
    --
    --  <Description>
    --
    --  This procedure is used to reset the screen saver.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Reset_Screen_Saver (Display_Id : in Display) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Root_Window
    --
    --  <Description>
    --
    --  This function is used to return the root window. This function is
    --  useful with function that take a parent window as an argument.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Root_Window ( Display_Id : in Display;
                           Screen_Id  : in Screen_Number) return Window;


    --===================================================================
    --  <Subprogram Name>
    --
    --  Root_Window_Of_Screen
    --
    --  <Description>
    --
    --  This function returns the root window of the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Root_Window_Of_Screen (Screen_Id : in Screen) return Window;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Screen_Count
    --
    --  <Description>
    --
    --  This function is used to return the number of available screens.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Screen_Count (Display_Id : in Display) return Screen_Number;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Screen_Of_Display
    --
    --  <Description>
    --
    --  This function is used to return a pointer to the screen record of
    --  the specified display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Num : Specifies which screen to return.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Screen_Of_Display ( Display_Id : in Display;
                                 Screen_Num : in Screen_Number)
        return Screen;


    --========================================================================
    --  <Subprogram Name>
    --
    --  Server_Vendor
    --
    --  <Description>
    --
    --  This function is used to return a string that provides some
    --  identification of the owner of the X Server implementation.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Server_Vendor (Display_Id : in Display) return String;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Access_Control
    --
    --  <Description>
    --
    --  This procedure is used to either enable or disable the use of the
    --  access control list at connection setups.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Access_Control ( Display_Id : in Display;
                                   Mode       : in Access_Control_Mode) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Output_Buffer_Size
    --
    --  <Description>
    --
    --  This procedure is used to change the output buffer size for
    --  the given display. The value is understood as a number of
    --  32 bits memory units and the actual value will be the smallest power
    --  of two bigger than the given value, which cannot exceeds the
    --  maximum request size accepted by the X server.
    --
    --  If not empty, the output buffer is flushed prior to the size change. 
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    procedure Set_Output_Buffer_Size ( Display_Id : in Display;
                                       Size       : in X_Natural);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Screen_Saver
    --
    --  <Description>
    --
    --  This procedure is used to set the screen saver. Timeout and
    --  intervals are specified in seconds. A timeout of 0 seconds
    --  disables the screen saver, while a timeout of -1 restores the
    --  default.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    procedure Set_Screen_Saver ( Display_Id : in Display;
                                 Time_Out   : in Time;
                                 Interval   : in Time;
                                 Blanking   : in Blanking_Type;
                                 Exposures  : in Exposure_Type);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Synchronize
    --
    --  <Description>
    --
    --  This procedure enables or disables synchronization for debugging
    --  clients. Synchronization ensures that errors will be seen when
    --  they are generated and not some time after.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Synchronize ( Display_Id : in Display;
                            Mode       : in Synchronize_Mode := On);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Translate_Coordinates
    --
    --  <Description>
    --
    -- This procedure performs a coordinate transformation from the
    -- coordinates of one window to another.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Source      : Specifies the source window.
    --  Destination : Specifies the destination window.
    --  From        : Specifies the coordinates within the source window.
    --  To          : Returns the coordinate within the destination window
    --  Child       : Returns the child if the coordinates are contained
    --                in the mapped child of the destination window.
    --  Same_Screen : True if source and destination are on the same
    --                screen.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Window is raised if the Window_Id is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Translate_Coordinates ( Display_Id  : in Display;
                                      Source      : in Window;
                                      Destination : in Window;
                                      From        : in Point;
                                      To          : out Point;
                                      Child       : out Window;
                                      Same_Screen : out Boolean);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Unreserve_Display
    --
    --  <Description>
    --
    -- This procedure remove the lock (if any) created by Reserve_Display and
    -- reserving access to the display against the other tasks requiring
    -- its use.
    -- The original value (before the call to Reserve_Display) of the display
    -- variable is restored.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>  None
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================
    
    procedure Unreserve_Display ( Display_Id : in out Display ) ;
    
    
    --========================================================================
    --  <Subprogram Name>
    --
    --  Vendor_Release
    --
    --  <Description>
    --
    --  This function is used to return a number related to a vendor's
    --  release of the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --========================================================================

    function Vendor_Release (Display_Id : in Display) return Natural;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Width_MM_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the width of the specified screen in
    --  millimeters.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Width_Mm_Of_Screen (Screen_Id : in Screen) return Millimeters;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Width_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the width of the specified screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Width_Of_Screen (Screen_Id : in Screen) return Pixels;

    --====================================================================
    --  <Subprogram Name>
    --
    --  White_Pixel
    --
    --  <Description>
    --
    --  This function is used to return the white pixel value for the
    --  specified display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Screen_Id  : Specifies the screen number.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function White_Pixel ( Display_Id : in Display;
                           Screen_Id  : in Screen_Number)
        return Pixel_Value;


    --====================================================================
    --  <Subprogram Name>
    --
    --  White_Pixel_Of_Screen
    --
    --  <Description>
    --
    --  This function is used to return the white pixel value of the specified
    --  screen.
    --
    --  <Parameters>
    --
    --  Screen_Id  : Specifies the screen Id.
    --
    --  <Exceptions Raised>
    --
    --  none.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function White_Pixel_Of_Screen (Screen_Id : in Screen) return Pixel_Value;


    pragma INLINE ( Bitmap_Bit_Order,
                    Bitmap_Pad,
                    Bitmap_Unit,
                    Black_Pixel,
                    Black_Pixel_Of_Screen,
                    Cells_Of_Screen,
                    Default_Colormap,
                    Default_Colormap_Of_Screen,
                    Default_Depth,
                    Default_Depth_Of_Screen,
                    Default_Graphic_Context,
                    Default_Gc_Of_Screen,
                    Default_Root_Window,
                    Default_Screen,
                    Default_Screen_Of_Display,
                    Default_Visual,
                    Default_Visual_Of_Screen,
                    Display_Cells,
                    Display_Height,
                    Display_Name,
                    Display_Of_Screen,
                    Display_Planes,
                    Display_String,
                    Display_Width,
                    Does_Backing_Store,
                    Does_Save_Unders,
                    Event_Mask_Of_Screen,
                    Height_Of_Screen,
                    Height_Mm_Of_Screen,
                    Image_Byte_Order,
                    Max_Cmaps_Of_Screen,
                    Max_Keycode,
                    Min_Cmaps_Of_Screen,
                    Min_Keycode,
                    Planes_Of_Screen,
                    Protocol_Version,
                    Protocol_Revision,
                    Root_Window,
                    Root_Window_Of_Screen,
                    Screen_Count,
                    Screen_Of_Display,
                    Server_Vendor,
                    White_Pixel,
                    White_Pixel_Of_Screen,
                    Width_Of_Screen,
                    Width_Mm_Of_Screen);


end X_Display_Manager ;
