---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Extensions is
    use X_Types ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  List_Extensions
    --
    --  <Description>
    --
    --  This function returns a list of all the extensions supported
    --  by the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function List_Extensions (Display_Id : in Display) return String_List ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Extension
    --
    --  <Description>
    --
    --  This function returns a boolean that indicates if the named 
    --  extension is supported by the X server. If the boolean is true,
    --  the basic info to use the extension is returned.
    --
    --  <Parameters>
    --
    --  Display_Id      : The X Server connection.
    --  Extension_Name  : Name of the extension to query.
    --  Major_Opcode    : Major opcode of the extension.
    --  First_Event     : First event of the extension.
    --  First_Error     : First error of the extension.
    --  
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Extension ( Display_Id     : in Display ;
                                Extension_Name : in String ;
                                Major_Opcode   : out Major_Code_Type ;
                                First_Event    : out Event_Code_Type ;
                                First_Error    : out Error_Code_Type ;
                                Success        : out Boolean ) ;

end X_Extensions ;
