---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Fonts is
    use X_Types ;
    
    --======================================================================
    --  <Subprogram Name>
    --
    --  Free_Font
    --
    --  <Description>
    --
    --  This procedure deletes the association between the font id and the
    --  specified font. The font itself will be freed when no other resource
    --  references it. The data and font should not be referenced again.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Font_Info  : The associated font information.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Font is raised if the specified font is not found.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Free_Font ( Display_Id : in Display;
                          Font_Info  : in out Font_Record);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Free_Font_Info
    --
    --  <Description>
    --
    --  This procedure frees the Name_List and the Font_Info_List created
    --  in a previous call.
    --
    --  <Parameters>
    --
    --  Name_List      : The list to free.
    --  Font_Info_List : The list to be freed.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Free_Font_Info ( Names_List     : in out String_List;
                               Font_Info_List : in out Font_Record_List);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Free_Font_Names
    --
    --  <Description>
    --
    --  This procedure frees a font name list.
    --
    --  <Parameters>
    --
    --  Name_List : The list to free.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Free_Font_Names ( Name_List : in out String_List);



    --======================================================================
    --  <Subprogram Name>
    --
    --  Free_Font_Path
    --
    --  <Description>
    --
    --  This procedure frees the Directories List.
    --
    --  <Parameters>
    --
    --  Directories : The list of directories to be freed.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Free_Font_Path (Directories : in out String_List);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Get_Font_Path
    --
    --  <Description>
    --
    --  This function returns the directory search path for the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Get_Font_Path (Display_Id : in Display) return String_List ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Get_Font_Property
    --
    --  <Description>
    --
    --  This procedure returns the value of the specified font property, if
    --  Defined. The set of predefined font properties are listed in the
    --  package X_Types.
    --
    --  The knowledge of the given property is needed to interpret the
    --  "raw" value returned.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font description.
    --  Property   : The specified property.
    --  Value      : The value of that property.
    --  Defined    : True if the property is defined.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Get_Font_Property ( Font_Info : in Font_Record;
                                  Property  : in Atom;
                                  Value     : out X_Long_Integer;
                                  Defined   : out Boolean);

    --======================================================================
    --  <Subprogram Name>
    --
    --  List_Fonts
    --
    --  <Description>
    --
    --  This function returns a list of font names that match the string
    --  passed as the pattern argument. The length of the list passed
    --  determines the maximum number returned. The length of the returned
    --  list is the actual number found.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Pattern    : The name(s) to be searched.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Name is raised if no names are found matching the pattern.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure List_Fonts ( Display_Id : in Display;
                           Pattern    : in String;
                           Names      : in out String_List) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  List_Fonts_With_Info
    --
    --  <Description>
    --
    --  This procedure returns a list of font names that match the specified
    --  pattern and their associated font information. The list of names is
    --  limited to the length of the Names_List. The information returned for
    --  each font is identical to what Load_Query_Font would return, except
    --  that per-character metrics are not returned. Note that Names_List and
    --  Font_Info_List are dynamic. Use .Length to determine the length of the
    --  returned lists.
    --
    --  Use Free_Font_Names to free the Names_List. Use Free_Font_Info to free
    --  the Font_Info_List.
    --
    --  <Parameters>
    --
    --  Display_Id     : The X Server connection.
    --  Pattern        : Specifies the string associated with the font names
    --                   that you want to be returned.
    --  Font_Info_List : The list of font records returned, one for each name
    --                   found.
    --  Names_List     : The list of names found matching the pattern.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Name is raised if no matching names are found.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure List_Fonts_With_Info ( Display_Id     : in Display;
                                     Pattern        : in String;
                                     Font_Info_List : in out Font_Record_List;
                                     Names_List     : in out String_List ) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Load_Font
    --
    --  <Description>
    --
    --  This function loads the specified font and returns its associated
    --  Font_Id. When the font is no longer needed, the client should call
    --  Unload_Font.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Font_Name  : The name of the font.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Name is raised if the specified font could not be loaded.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Load_Font ( Display_Id : in Display;
                         Font_Name  : in String) return Font ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Load_Query_Font
    --
    --  <Description>
    --
    --  This function loads a font and returns its associated font information
    --  in a single call.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Font_Name  : The name of the font.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Name is raised if the font does not exist.
    --  Bad_Alloc is raised if space for the font could not be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Load_Query_Font ( Display_Id : in Display;
                               Font_Name  : in String) return Font_Record;


    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Font
    --
    --  <Description>
    --
    --  This procedure returns the Font_Record, which contains information
    --  associated with the font. Use Free_Font_Info to free this structure.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Font_Id    : The associated Id.
    --  Font_Info  : Associated font information.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Query_Font ( Display_Id : in Display;
                          Font_Id    : in Font) return Font_Record ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Text_Extents
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified String.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  This function queries the X Server and, therefore suffers the round
    --  trip overhead avoided by Text_Extents.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The String.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Text_Extents ( Display_Id : in Display;
                                   Font_Id    : in Font;
                                   Text       : in String;
                                   Direction  : out Font_Direction;
                                   Ascent     : out Pixels;
                                   Descent    : out Pixels;
                                   Overall    : out Character_Record) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Text_Extents_8
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified String_8.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  This function queries the X Server and, therefore suffers the round
    --  trip overhead avoided by Text_Extents.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The String_8.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Text_Extents_8 ( Display_Id : in Display;
                                     Font_Id    : in Font;
                                     Text       : in String_8;
                                     Direction  : out Font_Direction;
                                     Ascent     : out Pixels;
                                     Descent    : out Pixels;
                                     Overall    : out Character_Record) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Text_Extents_16
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified String_16.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  This function queries the X Server and, therefore suffers the round
    --  trip overhead avoided by Text_Extents.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The String_16.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Text_Extents_16 ( Display_Id : in Display;
                                      Font_Id    : in Font;
                                      Text       : in String_16;
                                      Direction  : out Font_Direction;
                                      Ascent     : out Pixels;
                                      Descent    : out Pixels;
                                      Overall    : out Character_Record) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Font_Path
    --
    --  <Description>
    --
    --  This procedure specifies the directory search path for font
    --  lookup. There is only one search path per X Server, not per
    --  client. The interpretation of search strings is operating system
    --  dependent, but they are intended to specify directories to be
    --  searched in the order listed.
    --
    --  <Parameters>
    --
    --  Display_Id  : The X Server connection.
    --  Directories : The list of directories.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Value
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Font_Path ( Display_Id  : in Display;
                              Directories : in String_List) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Extents
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified string.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The ASCII Character string.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Text_Extents ( Font_Info : in Font_Record;
                             Text      : in String;
                             Direction : out Font_Direction;
                             Ascent    : out Pixels;
                             Descent   : out Pixels;
                             Overall   : out Character_Record);
    
    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Extents
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified string.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The X_Character string.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Text_Extents ( Font_Info : in Font_Record;
                             Text      : in String_8;
                             Direction : out Font_Direction;
                             Ascent    : out Pixels;
                             Descent   : out Pixels;
                             Overall   : out Character_Record) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Extents_16
    --
    --  <Description>
    --
    --  This procedure returns the logical extents of the specified String_16.
    --  The logical extents of a string is the width and height of the
    --  bounding box occupied by the string in the specified font.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The String_16.
    --  Direction  : Returns Font_Left_To_Right or Font_Right_To_Left.
    --  Ascent     : The maximum ascent of all characters in the string.
    --  Descent    : The maximum descent of all characters in the string.
    --  Overall    : The overall size.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Text_Extents_16 ( Font_Info : in Font_Record;
                                Text      : in String_16;
                                Direction : out Font_Direction;
                                Ascent    : out Pixels;
                                Descent   : out Pixels;
                                Overall   : out Character_Record) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Width
    --
    --  <Description>
    --
    --  This function returns the width in pixels of an ASCII string.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The ASCII Character string.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Text_Width ( Font_Info : in Font_Record;
                          Text      : in String) return Pixels ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Width
    --
    --  <Description>
    --
    --  This function returns the width in pixels of an X_Character string.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The X_Character string.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Text_Width ( Font_Info : in Font_Record;
                          Text      : in String_8) return Pixels ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Text_Width_16
    --
    --  <Description>
    --
    --  This function returns the width in pixels of a Char_2b String_16.
    --
    --  <Parameters>
    --
    --  Font_Info  : The font information used for the computation.
    --  Text       : The String_16.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Text_Width_16 ( Font_Info : in Font_Record;
                             Text      : in String_16) return Pixels ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Unload_Font
    --
    --  <Description>
    --
    --  This procedure unloads a font loaded with Load_Font.
    --
    --  <Parameters>
    --
    --  Display_Id : The X Server connection.
    --  Font_Id    : The associated Id.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Font is raised if the font is undefined.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Unload_Font ( Display_Id : in Display;
                            Font_Id    : in Font) ;

end X_Fonts;
