---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Graphic_Output is
    use X_Types ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Change_GC
    --
    --  <Description>
    --
    --  This procedure changes the components specified by the Value_Mask
    --  for the specified graphic context. The values argument contains
    --  the values to be set and are the same, as are the restrictions as
    --  for Create_GC.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  GC         : Specifies the graphic context ID.
    --  Value_Mask : Specifies which components in the GC_Value_Record are
    --               to be set. This argument is the bitwise OR of one or
    --               more of the GC_Mask_Type constants.
    --  Values     : Specifies the GC_Values_Record.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --  Bad_Match is raised if the clip_mask is a pixmap with depth > 1 or
    --            of a different root than the graphic context.
    --  Bad_Value is raised if the dash_list is zero.
    --  Bad_Pixmap is raised if a pixmap is invalid.
    --  Bad_Font is raised if the font is invalid.
    --  Bad_Alloc is raised if space cannot be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Change_Gc ( Display_Id : in Display;
                          Gc         : in Graphic_Context;
                          Value_Mask : in Gc_Mask_Type;
                          Values     : in Gc_Value_Record);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Clear_Area
    --
    --  <Description>
    --
    --  This procedure clears a specified rectangular area in the
    --  specified window. If either width or height are zero, they are
    --  replaced by the current width or height of the specified window
    --  minus x or y.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Window_Id    : Specifies the window.
    --  Bounds       : Specifies the rectangle to clear.
    --  Exposures    : Specifies that exposure event are to be generated.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the window is input only.
    --  Bad_Window is raised if the window is invalid.
    --  Bad_Value is raised if any of the coordinates are invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Clear_Area ( Display_Id : in Display;
                           Window_Id  : in Window;
                           Bounds     : in Rectangle;
                           Exposures  : in Boolean) ;


    --======================================================================
    --  <Subprogram Name>
    --
    --  Clear_Window
    --
    --  <Description>
    --
    --  This procedure clears the entire area in the specified window.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Window_Id    : Specifies the window.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the window is input only.
    --  Bad_Window is raised if the window is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Clear_Window ( Display_Id : in Display;
                             Window_Id  : in Window);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Copy_Area
    --
    --  <Description>
    --
    --  This procedure copies an area of the source drawable to the target
    --  drawable (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Source       : Specifies the source window or pixmap.
    --  Destination  : Specifies the source window or pixmap.
    --  GC           : Specifies the graphic context.
    --  Source_Area  : Specifies origin and size of the source rectangle.
    --  Dest_Origin  : Specifies the destination origin.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if either source or destination are invalid
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

     procedure Copy_Area ( Display_Id  : in Display;
                           Source      : in Drawable;
                           Destination : in Drawable;
                           Gc          : in Graphic_Context;
                           Source_Area : in Rectangle;
                           Dest_Origin : in Point) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Copy_GC
    --
    --  <Description>
    --
    --  This procedure copies the components specified in Value_Mask from
    --  the source graphic context to the destination graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Value_Mask : Specifies which components in the GC_Value_Record are
    --               to be set. This argument is the bitwise OR of one or
    --               more of the GC_Mask_Type constants.
    -- Source      : Specifies the source graphic context.
    -- Destination : Specifies the destination graphic context.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the source and destination do not have
    --            the same root and depth.
    --  Bad_Value is raised if the dash_list is zero.
    --  Bad_GC is raised if either source or destination are invalid.
    --  Bad_Alloc is raised if space cannot be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Copy_Gc ( Display_Id  : in Display;
                        Value_Mask  : in Gc_Mask_Type;
                        Source      : in Graphic_Context;
                        Destination : in Graphic_Context);


     --===================================================================
     --  <Subprogram Name>
     --
     --  Copy_Plane
     --
     --  <Description>
     --
     --  This procedure copies an area of the source drawable to the
     --  target drawable (pixmap or window).
     --
     --  <Parameters>
     --
     --  Display_Id   : Specifies the connection to the X Server.
     --  Source       : Specifies the source window or pixmap.
     --  Destination  : Specifies the source window or pixmap.
     --  GC           : Specifies the graphic context.
     --  Source_Area  : Specifies the origin of the source rectangle.
     --  Dest_Origin  : Specifies the destination origin.
     --
     --  <Exceptions Raised>
     --
     --  Bad_Match is raised if the destination window is input only.
     --  Bad_Drawable is raised if source or destination are invalid.
     --  Bad_GC is raised if the graphic context is invalid.
     --
     --  <Called By>
     --
     --  External.
     --
     --===================================================================

     procedure Copy_Plane ( Display_Id  : in Display;
                            Source      : in Drawable;
                            Destination : in Drawable;
                            Gc          : in Graphic_Context;
                            Source_Area : in Rectangle;
                            Dest_Origin : in Point;
                            Plane       : in Plane_Mask);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Create_GC
    --
    --  <Description>
    --
    --  This function creates a graphic context and returns its ID.
    --  The graphic context can be used with any destination drawable
    --  having the same root and depth as the specified drawable.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable.
    --  Value_Mask  : Specifies which components in the GC_Value_Record
    --                are to be set. This argument is the bitwise OR of
    --                one or more of the GC_Mask_Type constants.
    --  Values      : Specifies the GC_Values_Record.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the clip_mask is a pixmap with depth > 1 or
    --            of a different root than the graphic context.
    --  Bad_Value is raised if the dash_list is zero.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_Pixmap is raised if a pixmap is invalid.
    --  Bad_Font is raised if the font is invalid.
    --  Bad_Alloc is raised if space cannot be allocated for the context.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Create_Gc
        ( Display_Id  : in Display;
          Drawable_Id : in Drawable;
          Value_Mask  : in Gc_Mask_Type;
          Values      : in Gc_Value_Record) return Graphic_Context;



    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Arc
    --
    --  <Description>
    --
    --  This procedure draws a single arc in the specified drawable
    --  (pixmap or window). The x and y components of the arc are drawn
    --  relative to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Bounds      : Specifies the origin, width, height & angles or arc.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Arc ( Display_Id  : in Display;
                         Drawable_Id : in Drawable;
                         Gc          : in Graphic_Context;
                         Bounds      : in Arc);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Arcs
    --
    --  <Description>
    --
    --  This procedure draws multiple arcs in the specified drawable
    --  (pixmap or window). The x and y components of the arcs are drawn
    --  relative to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Arcs         : Specifies an array of arcs to draw.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Arcs ( Display_Id  : in Display;
                          Drawable_Id : in Drawable;
                          Gc          : in Graphic_Context;
                          Arcs        : in Arc_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Image_String
    --
    --  <Description>
    --
    --  This procedure draws an ASCII text string at the baseline position
    --  in the drawable (pixmap or window). This procedure differs from
    --  Draw_String by drawing both the foreground and background bits of
    --  each character painted to avoid flicker.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Baseline     : Specifies the starting position for the string.
    --  Text         : Specifies the ASCII string to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Image_String ( Display_Id  : in Display;
                                  Drawable_Id : in Drawable;
                                  Gc          : in Graphic_Context;
                                  Baseline    : in Point;
                                  Text        : in String);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Image_String
    --
    --  <Description>
    --
    --  This procedure draws an X_Character string at the baseline
    --  position in the drawable (pixmap or window). This procedure
    --  differs from Draw_String by drawing both the foreground and
    --  background bits of each character painted to avoid flicker.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Baseline     : Specifies the starting position for the string.
    --  Text         : Specifies the X_Character string to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Image_String ( Display_Id  : in Display;
                                  Drawable_Id : in Drawable;
                                  Gc          : in Graphic_Context;
                                  Baseline    : in Point;
                                  Text        : in String_8);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Image_String_16
    --
    --  <Description>
    --
    --  This procedure draws a String_16 (Char_2b) at the baseline
    --  position in the drawable. This procedure differs from
    --  Draw_String_16 by drawing both the foreground and background bits
    --  of each character painted to avoid flicker.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Baseline    : Specifies the starting position for the string.
    --  Text        : Specifies the String_16 to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Image_String_16 ( Display_Id  : in Display;
                                     Drawable_Id : in Drawable;
                                     Gc          : in Graphic_Context;
                                     Baseline    : in Point;
                                     Text        : in String_16);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Line
    --
    --  <Description>
    --
    --  This procedure draws a single line in the drawable
    --  (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Point_1      : Specifies the beginning point of the line.
    --  Point_2      : Specifies the ending point of the line.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Line ( Display_Id  : in Display;
                          Drawable_Id : in Drawable;
                          Gc          : in Graphic_Context;
                          Point_1     : in Point;
                          Point_2     : in Point);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Lines
    --
    --  <Description>
    --
    --  This procedure draws multiple, connected lines in the drawable.
    --  If Coordinate_Mode is Origin, all points are treated as relative
    --  to the drawable origin. If Previous, all points are treated
    --  as relative to the previous point.
    --
    --  <Parameters>
    --
    --  Display_Id      : Specifies the connection to the X Server.
    --  Drawable_Id     : Specifies the drawable (pixmap or window).
    --  GC              : Specifies the graphic context.
    --  Points          : Specifies an array of points.
    --  Coordinate_Mode : Specifies either Origin or Previous.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Lines ( Display_Id      : in Display;
                           Drawable_Id     : in Drawable;
                           Gc              : in Graphic_Context;
                           Points          : in Point_Array;
                           Coordinate_Mode : in Coordinate_Mode_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Point
    --
    --  <Description>
    --
    --  This procedure draws a single point in the drawable
    --  (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Xy           : Specifies the point location.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Point ( Display_Id  : in Display;
                           Drawable_Id : in Drawable;
                           Gc          : in Graphic_Context;
                           Xy          : in Point);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Points
    --
    --  <Description>
    --
    --  This procedure draws the points specified in the point array. If
    --  the Coordinate_Mode is Origin, all points are relative to the
    --  drawable's origin. If the mode is Previous, all points after the
    --  first are relative to the previous point.
    --
    --  <Parameters>
    --
    --  Display_Id      : Specifies the connection to the X Server.
    --  Drawable_Id     : Specifies the drawable (pixmap or window).
    --  GC              : Specifies the graphic context.
    --  Points          : Specifies an array of point locations.
    --  Coordinate_Mode : Specifies Previous or Origin.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Points ( Display_Id      : in Display;
                            Drawable_Id     : in Drawable;
                            Gc              : in Graphic_Context;
                            Points          : in Point_Array;
                            Coordinate_Mode : in Coordinate_Mode_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Rectangle
    --
    --  <Description>
    --
    --  This procedure draws a single rectangle in the specified drawable.
    --  The x and y components of the rectangle are drawn relative
    --  to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Bounds       : Specifies the rectangle to draw.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Rectangle ( Display_Id  : in Display;
                               Drawable_Id : in Drawable;
                               Gc          : in Graphic_Context;
                               Bounds      : in Rectangle);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Rectangles
    --
    --  <Description>
    --
    --  This procedure draws multiple rectangles in the specified drawable
    --  The x and y components of the rectangles are drawn relative
    --  to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Rectangles   : Specifies an array of rectangles to draw.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Rectangles ( Display_Id  : in Display;
                                Drawable_Id : in Drawable;
                                Gc          : in Graphic_Context;
                                Rectangles  : in Rectangle_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Segments
    --
    --  <Description>
    --
    --  This procedure draws multiple, unconnected lines in the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Segments    : Specifies an array of pairs of points (start & end).
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Segments ( Display_Id  : in Display;
                              Drawable_Id : in Drawable;
                              Gc          : in Graphic_Context;
                              Segments    : in Segment_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_String
    --
    --  <Description>
    --
    --  This procedure draws an ASCII text string at the baseline position
    --  in the drawable (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Baseline    : Specifies the starting position for the string.
    --  Text        : Specifies the ASCII string to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_String ( Display_Id  : in Display;
                            Drawable_Id : in Drawable;
                            Gc          : in Graphic_Context;
                            Baseline    : in Point;
                            Text        : in String);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_String
    --
    --  <Description>
    --
    --  This procedure draws an X_Character string at the baseline
    --  position in the drawable (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Baseline    : Specifies the starting position for the string.
    --  Text        : Specifies the X_Character string to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_String ( Display_Id  : in Display;
                            Drawable_Id : in Drawable;
                            Gc          : in Graphic_Context;
                            Baseline    : in Point;
                            Text        : in String_8);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_String_16
    --
    --  <Description>
    --
    --  This procedure draws a String_16 (Char_2b) at the baseline
    --  position in the drawable (pixmap or window).
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Baseline    : Specifies the starting position for the string.
    --  Text        : Specifies the String_16 to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font in the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_String_16 ( Display_Id  : in Display;
                               Drawable_Id : in Drawable;
                               Gc          : in Graphic_Context;
                               Baseline    : in Point;
                               Text        : in String_16);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Text
    --
    --  <Description>
    --
    --  This convenience procedure draws an array of ASCII text strings at
    --  the baseline position in the drawable (pixmap or window). This
    --  array specifies, for each string, a font and the distance between
    --  the string and the previously drawn string.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Baseline     : Specifies the starting position for the 1st string.
    --  Text_Items   : Specifies the array of text strings to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the specified font is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Text ( Display_Id  : in Display;
                          Drawable_Id : in Drawable;
                          Gc          : in Graphic_Context;
                          Baseline    : in Point;
                          Text_Items  : in Text_Item_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Text
    --
    --  <Description>
    --
    --  This procedure draws an array of X_Character strings at the
    --  baseline position in the drawable (pixmap or window). This array,
    --  specifies for each string, a font and the distance between the
    --  string and the previously drawn string.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Baseline     : Specifies the starting position for the 1st string.
    --  Text_Items   : Specifies the array of text strings to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the specified font is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Text ( Display_Id  : in Display;
                          Drawable_Id : in Drawable;
                          Gc          : in Graphic_Context;
                          Baseline    : in Point;
                          Text_Items  : in Text_Item_8_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Draw_Text_16
    --
    --  <Description>
    --
    --  This procedure draws an array of Char_2b String_16 at the baseline
    --  position in the drawable (pixmap or window).  This array,
    --  specifies for each string, a font and the distance between the
    --  string and the previously drawn string.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Baseline     : Specifies the starting position for the 1st string.
    --  Text_Items   : Specifies the array of text strings to be drawn.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Font is raised if the specified font is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Draw_Text_16 ( Display_Id  : in Display;
                             Drawable_Id : in Drawable;
                             Gc          : in Graphic_Context;
                             Baseline    : in Point;
                             Text_Items  : in Text_Item_16_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Arc
    --
    --  <Description>
    --
    --  This procedure draws a single filled arc in the specified drawable
    --  The x and y components of the arc are drawn relative
    --  to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  Drawable_Id : Specifies the drawable (pixmap or window).
    --  GC          : Specifies the graphic context.
    --  Bounds      : Specifies the origin, width, height & angles or arc.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Fill_Arc ( Display_Id  : in Display;
                         Drawable_Id : in Drawable;
                         Gc          : in Graphic_Context;
                         Bounds      : in Arc);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Arcs
    --
    --  <Description>
    --
    --  This procedure draws multiple filled arcs in the specified
    --  drawable. The x and y components of the arcs are drawn relative
    --  to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Arcs         : Specifies an array of arcs to draw.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Fill_Arcs ( Display_Id  : in Display;
                          Drawable_Id : in Drawable;
                          Gc          : in Graphic_Context;
                          Arcs        : in Arc_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Polygon
    --
    --  <Description>
    --
    --  This procedure draws a filled polygon in the drawable (pixmap or).
    --  window Coordinate_Mode is Origin, all points are relative to the
    --  drawable's origin. If the mode is Previous, all points after the
    --  first are relative to the previous point. If the shape is Complex,
    --  the path may intersect. If Nonconvex, the path does not intersect
    --  but the shape is not convex. If shape is Convex, the path is
    --  wholly convex.
    --
    --  <Parameters>
    --
    --  Display_Id      : Specifies the connection to the X Server.
    --  Drawable_Id     : Specifies the drawable (pixmap or window).
    --  GC              : Specifies the graphic context.
    --  Points          : Specifies an array of point locations.
    --  Shape           : Specifies Complex, Nonconvex or Convex.
    --  Coordinate_Mode : Specifies Previous or Origin.
    --
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Fill_Polygon ( Display_Id      : in Display;
                             Drawable_Id     : in Drawable;
                             Gc              : in Graphic_Context;
                             Points          : in Point_Array;
                             Shape           : in Shape_Type;
                             Coordinate_Mode : in Coordinate_Mode_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Rectangle
    --
    --  <Description>
    --
    --  This procedure draws a filled rectangle in the specified drawable.
    --  The x and y components of the rectangle are drawn relative
    --  to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Bounds       : Specifies the rectangle to fill.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Fill_Rectangle ( Display_Id  : in Display;
                               Drawable_Id : in Drawable;
                               Gc          : in Graphic_Context;
                               Bounds      : in Rectangle);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Fill_Rectangles
    --
    --  <Description>
    --
    --  This procedure draws multiple filled rectangles in the specified
    --  drawable The x and y components of the rectangles are drawn
    --  relative to the origin of the drawable.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Drawable_Id  : Specifies the drawable (pixmap or window).
    --  GC           : Specifies the graphic context.
    --  Rectangles   : Specifies an array of rectangles to fill.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if the destination window is input only.
    --  Bad_Drawable is raised if the drawable is invalid.
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Fill_Rectangles ( Display_Id  : in Display;
                                Drawable_Id : in Drawable;
                                Gc          : in Graphic_Context;
                                Rectangles  : in Rectangle_Array);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Free_GC
    --
    --  <Description>
    --
    --  This procedure destroys the specified graphic context as well
    --  as the shadow copy.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  GC         : Returns the graphic context ID.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Free_Gc ( Display_Id : in Display;
                        Gc         : in out Graphic_Context);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Query_Best_Size
    --
    --  <Description>
    --
    --  This procedure returns the best or closest size to the requested
    --  size for the specified shape class.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Shape_Class  : Specifies Tile_Shape, Cursor_Shape or Stipple_Shape
    --  Which_Screen : Specifies the drawable on a screen.
    --  Width        : Specifies the requested width.
    --               : Returns the best or closest width.
    --  Height       : Specifies the requested height.
    --               : Returns the best or closest height.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if an Input_Only window is the drawable for
    --  Tile_Shape or Stipple_Shape.
    --  Bad_Drawable is raised if the drawable is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Query_Best_Size ( Display_Id   : in Display;
                                Shape_Class  : in Shape_Class_Type;
                                Which_Screen : in Drawable;
                                Width        : in out Pixels;
                                Height       : in out Pixels);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Best_Stipple
    --
    --  <Description>
    --
    --  This procedure returns the best or closest size to the requested size
    --  for stippling.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Which_Screen : Specifies the drawable on a screen.
    --  Width        : Specifies the requested width.
    --               : Returns the best or closest width.
    --  Height       : Specifies the requested height.
    --               : Returns the best or closest height.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if an Input_Only window is the drawable.
    --  Bad_Drawable is raised if the drawable is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Best_Stipple ( Display_Id   : in Display;
                                   Which_Screen : in Drawable;
                                   Width        : in out Pixels;
                                   Height       : in out Pixels);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Query_Best_Tile
    --
    --  <Description>
    --
    --  This procedure returns the best or closest size to the requested size
    --  for tileing.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  Which_Screen : Specifies the drawable on a screen.
    --  Width        : Specifies the requested width.
    --               : Returns the best or closest width.
    --  Height       : Specifies the requested height.
    --               : Returns the best or closest height.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if an Input_Only window is the drawable.
    --  Bad_Drawable is raised if the drawable is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Query_Best_Tile ( Display_Id   : in Display;
                                Which_Screen : in Drawable;
                                Width        : in out Pixels;
                                Height       : in out Pixels);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Resource_Gc_From_Graphic_Context
    --
    --  <Description>
    --
    --  This function returns the Resource_Gc for the specified
    --  Graphic_Context.
    --
    -- <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Resource_Gc_From_Graphic_Context
       (Gc : in Graphic_Context) return Resource_Gc;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Arc_Mode
    --
    --  <Description>
    --
    --  This procedure sets the arc mode in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Arc_Mode     : Specifies either Arc_Chord or Arc_Pie_Slice.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Arc_Mode ( Display_Id : in Display;
                             Gc         : in Graphic_Context;
                             Arc_Mode   : in Arc_Mode_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Background
    --
    --  <Description>
    --
    --  This procedure sets the background component for the specified
    --  graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Background  : Specifies the background value to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Background ( Display_Id : in Display;
                               Gc         : in Graphic_Context;
                               Background : in Pixel_Value);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Clip_Mask
    --
    --  <Description>
    --
    --  This procedure sets the Clip_Mask in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Clip_Mask    : Specifies the clip mask.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Pixmap is raised if the clip mask is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Clip_Mask ( Display_Id : in Display;
                              Gc         : in Graphic_Context;
                              Clip_Mask  : in Pixmap);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Clip_Origin
    --
    --  <Description>
    --
    --  This procedure sets the clip origin in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Origin       : Specifies the origin of the tile or stipple.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Clip_Origin ( Display_Id : in Display;
                                Gc         : in Graphic_Context;
                                Origin     : in Point);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Clip_Rectangles
    --
    --  <Description>
    --
    --  This procedure sets the clip mask in the specified graphic context
    --  to the specified list of rectangles.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Origin       : Specifies the origin of the tile or stipple.
    --  Rectangles   : Specifies the list of rectangles.
    --  Ordering     : Specifies the ordering relations on the rectangles.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Alloc is raised if the rectangles cannot be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Clip_Rectangles ( Display_Id : in Display;
                                    Gc         : in Graphic_Context;
                                    Origin     : in Point;
                                    Rectangles : in Rectangle_Array;
                                    Ordering   : in Ordering_Type) ;
 
    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Dashes
    --
    --  <Description>
    --
    --  This procedure sets the dash_offset and dash_list in the specified
    --  graphic context for dashed line styles.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Dash_Offset : Specifies phase of the pattern for the dashed line
    --                style you want to set.
    --  Dash_List   : Specifies the dash list for the dashed line style
    --                you want to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --  Bad_Value is raised if dash_offset is empty or dash_list is zero.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Dashes ( Display_Id  : in Display;
                           Gc          : in Graphic_Context;
                           Dash_Offset : in Pixels;
                           Dashes      : in Dash_List);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Fill_Rule
    --
    --  <Description>
    --
    --  This procedure sets the fill rule in the specified graphic context
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Fill_Rule   : Specifies either Even_Odd_Rule or Winding_Rule.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Fill_Rule ( Display_Id : in Display;
                              Gc         : in Graphic_Context;
                              Fill_Rule  : in Fill_Rule_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Fill_Style
    --
    --  <Description>
    --
    --  This procedure sets the fill style in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Fill_Style  : Specifies the fill style to be set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Fill_Style ( Display_Id : in Display;
                               Gc         : in Graphic_Context;
                               Fill_Style : in Fill_Style_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Font
    --
    --  <Description>
    --
    --  This procedure sets the current font in the specified graphic
    --  context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Font_Id      : Specifies the requested font.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Gc is raised if the graphic context is invalid.
    --  Bad_Font is raised if the font is invalid.
    --  Bad_Alloc is raised if space for the font could not be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Font ( Display_Id : in Display;
                         Gc         : in Graphic_Context;
                         Font_Id    : in Font);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Foreground
    --
    --  <Description>
    --
    --  This procedure sets the foreground component for the specified
    --  graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Foreground  : Specifies the foreground value to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Foreground ( Display_Id : in Display;
                               Gc         : in Graphic_Context;
                               Foreground : in Pixel_Value);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Function
    --
    --  <Description>
    --
    --  This procedure sets the function component for the specified
    --  graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  GX_Function : Specifies the graphic function to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Function ( Display_Id  : in Display;
                             Gc          : in Graphic_Context;
                             Gx_Function : in Gx_Function_Code);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Graphics_Exposures
    --
    --  <Description>
    --
    --  This procedure sets the graphics_exposures flag in the specified
    --  graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id         : Specifies the connection to the X Server.
    --  GC                 : Specifies the graphic context.
    --  Graphics_Exposures : Specifies if events are to be reported when
    --                       calling Copy_Area and Copy_Plane.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Graphics_Exposures
        ( Display_Id         : in Display;
          Gc                 : in Graphic_Context;
          Graphics_Exposures : in Boolean);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Line_Attributes
    --
    --  <Description>
    --
    --  This procedure sets the line drawing components
    --  of the graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Line_Width  : Specifies the line width to set.
    --  Line_Style  : Specifies the line style to set.
    --  Cap_Style   : Specifies the cap style to set.
    --  Join_Style  : Specifies the join style to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --  Bad_Value is raised if any of the values are invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Line_Attributes ( Display_Id : in Display;
                                    Gc         : in Graphic_Context;
                                    Line_Width : in Pixels;
                                    Line_Style : in Line_Style_Type;
                                    Cap_Style  : in Cap_Style_Type;
                                    Join_Style : in Join_Style_Type);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Plane_Mask
    --
    --  <Description>
    --
    --  This procedure sets the plane component for the specified
    --  graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Plane       : Specifies the plane mask to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Plane_Mask ( Display_Id : in Display;
                               Gc         : in Graphic_Context;
                               Plane      : in Plane_Mask);


    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_State
    --
    --  <Description>
    --
    --  This procedure sets the foreground, background, plane and function
    --  components for the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id  : Specifies the connection to the X Server.
    --  GC          : Specifies the graphic context ID.
    --  Foreground  : Specifies the foreground value to set.
    --  Background  : Specifies the background value to set.
    --  GX_Function : Specifies the graphic function to set.
    --  Plane       : Specifies the plane mask to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the GC is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_State ( Display_Id  : in Display;
                          Gc          : in Graphic_Context;
                          Foreground  : in Pixel_Value;
                          Background  : in Pixel_Value;
                          Gx_Function : in Gx_Function_Code;
                          Plane       : in Plane_Mask);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Stipple
    --
    --  <Description>
    --
    --  This procedure sets the stipple in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Stipple      : Specifies the stipple to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Pixmap is raised if the stipple is invalid.
    --  Bad_Match is raised if an Input_Only window is the drawable.
    --  Bad_Alloc is raised if space for the stipple cannot be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Stipple ( Display_Id : in Display;
                            Gc         : in Graphic_Context;
                            Stipple    : in Pixmap);


    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Subwindow_Mode
    --
    --  <Description>
    --
    --  Sets the subwindow mode in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id     : Specifies the connection to the X Server.
    --  GC             : Specifies the graphic context.
    --  Subwindow_Mode : Specifies Clip_By_Children or Include_Inferiors.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Subwindow_Mode ( Display_Id     : in Display;
                                   Gc             : in Graphic_Context;
                                   Subwindow_Mode : in Subwindow_Mode_Type);


    --===================================================================
    --  <Subprogram Name>
    --
    --  Set_Tile
    --
    --  <Description>
    --
    --  This procedure sets the fill tile in the specified graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Tile         : Specifies the fill tile to set.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --  Bad_Pixmap is raised if the tile is invalid.
    --  Bad_Match is raised if an Input_Only window is the drawable for
    --            Tile_Shape.
    --  Bad_Alloc is raised if space for the tile cannot be allocated.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Set_Tile ( Display_Id : in Display;
                         Gc         : in Graphic_Context;
                         Tile       : in Pixmap);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Set_Ts_Origin
    --
    --  <Description>
    --
    --  This procedure sets the tile or stipple origin in the graphic context.
    --
    --  <Parameters>
    --
    --  Display_Id   : Specifies the connection to the X Server.
    --  GC           : Specifies the graphic context.
    --  Origin       : Specifies the origin of the tile or stipple.
    --
    --  <Exceptions Raised>
    --
    --  Bad_GC is raised if the graphic context is invalid.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Set_Ts_Origin ( Display_Id : in Display;
                              Gc         : in Graphic_Context;
                              Origin     : in Point);


end X_Graphic_Output;
