---------------------------------------------------------------
--
--                    Top Graph'X
--
-- For any information call J.C. Mahieux :
-- Tel : (33) 1 69 26 97 88
-- Fax : (33) 1 69 26 97 89
--
-- Top Graph'X - 10 Allee de la Mare Jacob
--               91290 La Norville- FRANCE
--
---------------------------------------------------------------
--
------------------ Distribution and Copyright -----------------
-- Derivation   : MIT X Window System, V11.R4
--
-- This Ada language binding to the X Window System inherits
-- the MIT copyright. The following copyright must be included
-- in all software utilising this binding.
--
-- Copyright 1990, 1991 J.C. Mahieux and B. Maudry
-- Copyright 1985, 1986, 1987 by the Massachusetts Institute of Technology
--
with X_Types ;

package X_Keyboard is
    use X_Types ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Auto_Repeat_On
    --
    --  <Description>
    --
    --  This Procedure turns on the auto-repeat for the keyboard on the
    --  display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Auto_Repeat_On (Display_Id : in Display);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Auto_Repeat_Off
    --
    --  <Description>
    --
    --  This Procedure turns off the auto-repeat for the keyboard on the
    --  display.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Auto_Repeat_Off (Display_Id : in Display);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Bell
    --
    --  <Description>
    --
    --  This Procedure rings the bell on the keyboard on the specified
    --  display, if possible. The specified volume is relative to the base
    --  volume for the keyboard.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Volume     : Specifies the base volume for the bell.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Bell ( Display_Id : in Display ;
                     Volume     : in Bell_Volume_Type ) ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Change_Keyboard_Control
    --
    --  <Description>
    --
    --  This Procedure controls the keyboard characteristics defined by
    --  the Keyboard_Control record.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Value_Mask : Specifies which values are to be changed.
    --  Values     : Specifies the new values.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Match is raised if an led is specified without an led mode or
    --            a key is specified without an auto repeat mode.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Change_Keyboard_Control
        ( Display_Id : in Display;
          Value_Mask : in Keyboard_Control_Mask;
          Values     : in Keyboard_Control_Record);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Change_Keyboard_Mapping
    --
    --  <Description>
    --
    --  This Procedure defines the symbols for the specified number of
    --  keycodes beginning with the First_Keycode. As above, the first
    --  keycode and number of key codes is set by the indexes of the
    --  key_encoding_array. The length of the keysyms list for each key
    --  code must be a fixed length.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Symbols    : Specifies key symbols for each key code in the list.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Value is raised if lower index is less than the value returned
    --            by Min_Keycode (Display_Id) or if the upper index is
    --            greater than Max_Keycode (Display_Id).
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Change_Keyboard_Mapping
        ( Display_Id : in Display;
          Symbols    : in Keyboard_Encoding_Array);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Convert_Case
    --
    --  <Description>
    --
    --  This function converts the given Key_Sym to its lower and upper
    --  case values. If this has no meaning for this Key_Sym, both
    --  returned values contains the given Key_Sym.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  The_Symbol : Specifies the Key_Sym to convert.
    --  L_Symbol   : The lower case value of Symbol returned.
    --  U_Symbol   : The upper case value of Symbol returned.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Convert_Case ( Display_Id : in Display ;
                             The_Symbol : in Key_Sym ;
                             L_Symbol   : out Key_Sym ;
                             U_Symbol   : out Key_Sym ) ;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Delete_Modifier_Map_Entry
    --
    --  <Description>
    --
    --  This procedure deletes a Key : Modifier mapping in a client
    --  modifier mapping.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map        : Specifies the client map.
    --  Key        : The key to delete
    --  Modifier   : The modifier mapped to the key
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Delete_Modifier_Map_Entry ( Map      : in out Modifier_Keymap;
                                          Key      : in Keycode;
                                          Modifier : in Modifier_Type);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Keyboard_Control
    --
    --  <Description>
    --
    --  This function returns the current control values for the keyboard.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Keyboard_State : returns the State of the keyboard.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Keyboard_Control
        (Display_Id : in Display) return Keyboard_State_Record;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Get_Modifier_Mapping
    --
    --  <Description>
    --
    --  This function returns the modifier:key code mapping
    --  for the X Server.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Get_Modifier_Mapping
        ( Display_Id : in Display ) return Modifier_Keymap;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Insert_Modifier_Map_Entry
    --
    --  <Description>
    --
    --  This procedure inserts a Key : Modifier mapping into a client
    --  modifier mapping.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Map        : Specifies the client map.
    --  Key        : The key to insert
    --  Modifier   : The modifier to map to the key
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    procedure Insert_Modifier_Map_Entry ( Map      : in out Modifier_Keymap;
                                          Key      : in Keycode;
                                          Modifier : in Modifier_Type);

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_Cursor_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a Cursor Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Cursor_Key (Sym : in Key_Sym) return Boolean;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_Function_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a Function Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Function_Key (Sym : in Key_Sym) return Boolean;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_Keypad_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a Keypad Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Keypad_Key (Sym : in Key_Sym) return Boolean;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_Misc_Function_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a Misc Function Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Misc_Function_Key (Sym : in Key_Sym) return Boolean;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_Modifier_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a Modifier Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Modifier_Key (Sym : in Key_Sym) return Boolean;

    --======================================================================
    --  <Subprogram Name>
    --
    --  Is_PF_Key
    --
    --  <Description>
    --
    --  This Function returns true if the Key_Sym is a PF Key.
    --
    --  <Exceptions Raised>
    --
    --  <Called By>
    --
    --  External.
    --
    --======================================================================

    function Is_Pf_Key (Sym : in Key_Sym) return Boolean;

    pragma INLINE (Is_Keypad_Key, Is_Cursor_Key, Is_Pf_Key, Is_Function_Key,
                   Is_Misc_Function_Key, Is_Modifier_Key);
    
    --====================================================================
    --  <Subprogram Name>
    --
    --  Keyboard_Mapping
    --
    --  <Description>
    --
    --  This function returns the symbols for the range of keycodes
    --  specified by the provided First_Key and Last_Key.
    --  The length of the Key_Syms array for each key code is
    --  automaticaly set in the list returned.
    --
    --  <Parameters>
    --
    --  Display_Id    : Specifies the connection to the X Server.
    --  First_Key     : The first Keycode in the range to look up.
    --  Last_Key      : The last Keycode in the range to look up.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Value is raised if First_Key is less than the value returned
    --            by Min_Keycode (Display_Id) or if Last_Key is
    --            greater than Max_Keycode (Display_Id).
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Keyboard_Mapping
        ( Display_Id : in Display;
          First_Key  : in Keycode;
          Last_Key   : in Keycode ) return Keyboard_Encoding_List ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Keycode_To_Keysym
    --
    --  <Description>
    --
    --  This function returns the Key_Sym defined for the specified
    --  keycode.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Key        : Specifies the Keycode to be searched for.
    --  Row        : Specifies the element of the Key_Code vector.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Keycode_To_Keysym ( Display_Id : in Display ;
                                 Key        : in Keycode ;
                                 Row        : in Natural )
        return Key_Sym ;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Keysym_To_Keycode
    --
    --  <Description>
    --
    --  This Function converts a Key_Sym to the appropriate key_code. Zero
    --  is returned when the specified Key_Sym is not defined.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  The_Symbol : Specifies the Key_Sym to be converted.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Keysym_To_Keycode ( Display_Id : in Display;
                                 The_Symbol : in Key_Sym) return Keycode;

    --====================================================================
    --  <Subprogram Name>
    --
    --  Query_Keymap
    --
    --  <Description>
    --
    --  This function returns a bit vector for the keyboard where each one
    --  bit indicates that the corresponding key is down.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Key_Map    : Returns the key map vector.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Query_Keymap
        (Display_Id : in Display) return Key_Vector_Mask_Type;


    --====================================================================
    --  <Subprogram Name>
    --
    --  Rebind_Keysym
    --
    --  <Description>
    --
    --  This Procedure rebinds the meaning of a Key_Sym for a client.
    --  It does not redefine the keycode in the X Server but merely
    --  provides an easy way for long strings to be attached to keys.
    --  Lookup_String returns this string when the appropriate set of
    --  modifier keys are pressed and when the Key_Sym would have been
    --  used for the translation.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  The_Symbol : Specifies the Key_Sym to be rebound.
    --  Modifiers  : Specifies a list of Key_Syms to be used as modifiers.
    --  Text       : Specifies the string to be returned by Lookup_String.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    procedure Rebind_Keysym ( Display_Id : in Display;
                              The_Symbol : in Key_Sym;
                              Modifiers  : in Key_Sym_List;
                              Text       : in String);

    --====================================================================
    --  <Subprogram Name>
    --
    --  Set_Modifier_Mapping
    --
    --  <Description>
    --
    --  This function maps the key codes to be used as modifiers.
    --
    --  <Parameters>
    --
    --  Display_Id : Specifies the connection to the X Server.
    --  Modifiers  : Specifies a Modifier_Keymap containing
    --               modifier:keycode mappings for each modifier.
    --               This length of this list is set by the
    --               Max_Keys_Per_Modifier discriminant.
    --
    --  <Exceptions Raised>
    --
    --  Bad_Value is raised if the key codes are not in the range
    --            Min_Keycode(Display_Id) .. Max_Keycode(Display_Id) or
    --            if two Modifier_Keymaps have the same key code value.
    --
    --  <Called By>
    --
    --  External.
    --
    --====================================================================

    function Set_Modifier_Mapping ( Display_Id : in Display;
                                    Modifiers  : in Modifier_Keymap)
        return Map_Status ;

end X_Keyboard;
